<?php

namespace IncomeSchool\AffiliateLinkRefer;


class AdminInterface
{
    /**
     * The directory for the admin templates
     * @var string
     */
    private $templatesDir = AFFILIATE_LINK_REFER_PLUGIN_DIR  . '/assets/html/admin/';

    /**
     * Construct
     */
    public function __construct()
    {
        add_action( 'admin_menu', [$this, 'addPages'], 99);

        //Form post handlers
        add_action( 'admin_post_handle_refer_amazon_settings_form', [$this, 'handleAffiliateSettingsForm'], 99);
        add_action( 'admin_post_handle_refer_legal_disclaimers_settings_form', [$this, 'handleLegalDisclaimersSettingsForm'], 99);

        //Ajax request handlers
        add_action('wp_ajax_handle_check_affiliate_link', [$this, 'handleCheckAffiliateLink'], 99);
        add_action('wp_ajax_handle_render_affiliate_edit_template', [$this, 'handleRenderAffiliateEditTemplate'], 99);
        add_action('wp_ajax_handle_post_affiliate_edit', [$this, 'handlePostAffiliateEdit'], 99);
        add_action('wp_ajax_handle_post_affiliate_add', [$this, 'handlePostAffiliateAdd'], 99);
        add_action('wp_ajax_handle_render_affiliate_delete_modal_template', [$this, 'handleRenderAffiliateDeleteModalTemplate'], 99);
        add_action('wp_ajax_handle_post_affiliate_delete', [$this, 'handlePostAffiliateDelete'], 99);
        add_action('wp_ajax_handle_render_affiliate_insert_shortcode_modal_template', [$this, 'handleRenderAffiliateInsertShortcodeModalTemplate'], 99);
        add_action('wp_ajax_handle_post_verify_amazon_shortlink_converter', [$this, 'handlePostVerifyAffiliateShortlinkConverter'], 99);

        // ajax request handlers for countdown/clocks
        add_action('wp_ajax_handle_post_refer_post_countdown_add', [$this, 'handlePostCountdownAdd'], 99);
        add_action('wp_ajax_handle_render_countdown_edit_template', [$this, 'handleRenderCountdownEditTemplate'], 99);
        add_action('wp_ajax_handle_render_countdown_add_template', [$this, 'handleRenderCountdownAddTemplate'], 99);
        add_action('wp_ajax_handle_render_clocks_list', [$this, 'handle_render_clocks_list'], 99);
        add_action('wp_ajax_handle_post_refer_countdown_edit', [$this, 'handlePostCountdownEdit'], 99);
        add_action('wp_ajax_handle_render_countdown_delete_modal_template', [$this, 'handleRenderCountdownDeleteModalTemplate'], 99);
        add_action('wp_ajax_handle_render_countdown_preview_modal_template', [$this, 'handleRenderCountdownPreviewModalTemplate'], 99);
        add_action('wp_ajax_handle_post_countdown_delete', [$this, 'handlePostCountdownDelete'], 99);


        //admin notices
        add_action('admin_notices', [$this, 'renderVerifyConverterAlert'], 99);
        //add_action('admin_notices', [$this, 'renderIncomeSchoolNotices'], 99);
    }

    /**
     * [addAdminPages description]
     */
    public function addPages()
    {
        add_menu_page('Affiliate REFER', 'REFER', 'manage_options', 'affiliate-link-refer', [$this, 'renderDashboardPage'], AFFILIATE_LINK_REFER_PLUGIN_URL . 'assets/images/wp-icon-refer.png');
        add_submenu_page('affiliate-link-refer', 'Affiliate Tracker Amazon Settings', 'Amazon Setup', 'manage_options', 'affiliate-link-refer-api-settings', [$this, 'renderAffiliateSettingsPage']);
        // add_submenu_page('affiliate-link-refer', 'Affiliate Tracker Other Affiliates', 'Other Affiliates', 'manage_options', 'affiliate-link-refer-other-affiliates', [$this, 'renderAffiliatesPage']);
        add_submenu_page('affiliate-link-refer', 'Clocks', 'Clocks', 'manage_options', 'affiliate-link-refer-product-countdowns', [$this, 'renderCountdownsPage']);
        add_submenu_page('affiliate-link-refer', 'Affiliate Tracker Legal Disclaimers', 'Smart Disclaimers', 'manage_options', 'affiliate-link-refer-legal-disclaimers', [$this, 'renderLegalDisclaimersSettingsPage']);
        add_submenu_page('affiliate-link-refer', 'Affiliate Tracker Reports', 'Reports', 'manage_options', 'affiliate-link-refer-reports', [$this, 'renderReportsDashboardPage']);
        add_submenu_page('affiliate-link-refer', 'Affiliate Link Manager', 'Link Manager', 'manage_options', 'affiliate-link-refer-link-manager', [$this, 'renderLinkManagerPage']);
        add_submenu_page('affiliate-link-refer', 'Trees', 'Trees', 'manage_options', 'affiliate-link-refer-trees', [$this, 'renderTreesPage']);
        add_submenu_page('affiliate-link-refer', 'Automations', 'Automations', 'manage_options', 'affiliate-link-refer-automations', [$this, 'renderAutomationsPage']);

        //Reports subpages. Leave off admin menu
        add_submenu_page('', 'Affiliate Tracker Report - By Author', 'Report By Author', 'manage_options', 'affiliate-link-refer-report-by-author', [$this, 'renderReportByAuthorPage']);
        add_submenu_page('', 'Affiliate Tracker Report - By Category', 'Report By Categpry', 'manage_options', 'affiliate-link-refer-report-by-category', [$this, 'renderReportByCategoryPage']);
        add_submenu_page('', 'Affiliate Tracker Report - By Post', 'Report By Post', 'manage_options', 'affiliate-link-refer-report-by-post', [$this, 'renderReportByPostPage']);
        add_submenu_page('', 'Affiliate Tracker Report - By Product', 'Report By Product', 'manage_options', 'affiliate-link-refer-report-by-product', [$this, 'renderReportByProductPage']);
        add_submenu_page('', 'Affiliate Tracker Report - By Product', 'Report By Word Count', 'manage_options', 'affiliate-link-refer-report-by-word-count', [$this, 'renderReportByWordCountPage']);

    }

    /**
     * [renderAffiliateSettingsPage description]
     * @return [type] [description]
     */
    public function renderDashboardPage()
    {
        include($this->templatesDir  . 'dashboard.php');
    }

    /**
     * [renderAffiliateSettingsPage description]
     * @return [type] [description]
     */
    public function renderAffiliateSettingsPage()
    {
        $affiliateId = get_option('refer_amazon_affiliate_id', '');
        $amazon_api_key = get_option('refer_amazon_api_key', '');
        $amazon_api_secret = get_option('refer_amazon_api_secret', '');
        $walmart_api_key = get_option('refer_walmart_api_key', '');
        //$oneLinkCode = stripslashes(get_option('refer_amazon_onlink_code', ''));

        $converterTool = AffiliateLinkRefer()->classes['affiliate_shortlink_converter'];

        $shortlinkToolVerified = $converterTool->getVerfiedWorking();
        $shortlinkToolFailedCount = $converterTool->getFailedCount();

        $queueCount = $converterTool->getQueueCount();

        global $wpdb;
        $queue = $wpdb->get_results("SELECT * FROM " . $wpdb->prefix . 'refer_amazon_shortlinks_queue');
        // var_dump($queue);
        include($this->templatesDir  . 'affiliate-settings.php');
    }

    /**
     * [handleAffiliateSettingsForm description]
     */
    public function handleAffiliateSettingsForm()
    {
        $valid = true;
        if(!isset($_POST['_wpnonce'])) {
            $valid = false;
        } elseif(!wp_verify_nonce($_POST['_wpnonce'], 'save_affiliate_settings')) {
            $valid = false;
        }

        if(!$valid) {
            wp_die('Invalid nonce specified', 'Error', array(
                'response'  => 403,
            ));
        }

        $affiliateId = isset($_POST['amazon_affiliate_id']) ? sanitize_text_field( $_POST['amazon_affiliate_id']) : false;
        if($affiliateId)
            update_option('refer_amazon_affiliate_id', $affiliateId);

        $amazon_api_key = isset($_POST['amazon_api_key']) ? sanitize_text_field( $_POST['amazon_api_key']) : false;
        if($amazon_api_key)
            update_option('refer_amazon_api_key', $amazon_api_key);

        $amazon_api_secret = isset($_POST['amazon_api_secret']) ? sanitize_text_field( $_POST['amazon_api_secret']) : false;
        if($amazon_api_secret)
            update_option('refer_amazon_api_secret', $amazon_api_secret);

        $walmart_api_key = isset($_POST['walmart_api_key']) ? sanitize_text_field( $_POST['walmart_api_key']) : false;
        if($walmart_api_key)
            update_option('refer_walmart_api_key', $walmart_api_key);

        wp_redirect( admin_url('admin.php?page=affiliate-link-refer-api-settings&saved=true'));
        exit;
    }

    /**
     * [renderLegalDisclaimersSettingsPage description]
     * @return [type] [description]
     */
    public function renderLegalDisclaimersSettingsPage()
    {

        $topDisclaimerDefaultCopy = "This post contains affiliate links.";
        $longDisclaimerDefaultCopy = "(Insert your company name) is a participant in the Amazon Services LLC Associates Program, an affiliate advertising program designed to provide a means for sites to earn advertising fees by advertising and linking to Amazon.com. We also participate in other affiliate programs which compensate us for referring traffic.";

        $topOfPost = (get_option('legal_disclaimers_top_of_post', '') != "") ? get_option('legal_disclaimers_top_of_post', '') : '';
        $bottomOfPost = (get_option('legal_disclaimers_bottom_of_post', '') != "") ? get_option('legal_disclaimers_bottom_of_post', '') : '';
        $footer = (get_option('legal_disclaimers_footer', '') != "") ? get_option('legal_disclaimers_footer', '') : '';
        $sidebar = (get_option('legal_disclaimers_sidebar', '') != "") ? get_option('legal_disclaimers_sidebar', '') : '';

        include($this->templatesDir  . 'legal-disclaimers.php');
    }

    /**
     * [handleAffiliateSettingsForm description]
     */
    public function handleLegalDisclaimersSettingsForm()
    {
        $valid = true;
        if(!isset($_POST['_wpnonce'])) {
            $valid = false;
        } elseif(!wp_verify_nonce($_POST['_wpnonce'], 'save_legal_disclaimers_settings')) {
            $valid = false;
        }

        if(!$valid) {
            wp_die('Invalid nonce specified', 'Error', array(
                'response'  => 403,
            ));
        }

        if(isset($_POST['legal_disclaimers_allow_top_of_post'])) {
            $topOfPost = isset($_POST['legal_disclaimers_top_of_post']) ? sanitize_text_field( $_POST['legal_disclaimers_top_of_post']) : '';
            update_option('legal_disclaimers_top_of_post', $topOfPost);
        } else {
            update_option('legal_disclaimers_top_of_post', '');
        }

        if(isset($_POST['legal_disclaimers_allow_bottom_of_post'])) {
            $bottomOfPost = isset($_POST['legal_disclaimers_bottom_of_post']) ? sanitize_text_field( $_POST['legal_disclaimers_bottom_of_post']) : '';
            update_option('legal_disclaimers_bottom_of_post', $bottomOfPost);
        } else {
            update_option('legal_disclaimers_bottom_of_post', '');
        }

        if(isset($_POST['legal_disclaimers_allow_footer'])) {
            $footer = isset($_POST['legal_disclaimers_footer']) ? sanitize_text_field( $_POST['legal_disclaimers_footer']) : '';
            update_option('legal_disclaimers_footer', $footer);
        } else {
            update_option('legal_disclaimers_footer', '');
        }

        if(isset($_POST['legal_disclaimers_allow_sidebar'])) {
            $sidebar = isset($_POST['legal_disclaimers_sidebar']) ? sanitize_text_field( $_POST['legal_disclaimers_sidebar']) : '';
            update_option('legal_disclaimers_sidebar', $sidebar);
        } else {
            update_option('legal_disclaimers_sidebar', '');
        }

        wp_redirect( admin_url('admin.php?page=affiliate-link-refer-legal-disclaimers&saved=true'));
        exit;
    }

    /**
     * [renderAffiliatesPage description]
     * @return [type] [description]
     */
    public function renderAffiliatesPage()
    {
        global $wpdb, $paged, $max_num_pages;

        $orderByClause = "label ASC";
        $sortByLabel = "DESC";
        $sortByUrl = "ASC";
        $sortByText = "ASC";
        $sortByHash = "ASC";

        if(isset($_GET['sortOtherAffiliateByLabel']) || isset($_GET['sortOtherAffiliateByUrl']) || isset($_GET['sortOtherAffiliateByText']) || isset($_GET['sortOtherAffiliateByShortcode'])) {
            // $orderByClause = "";
            if(isset($_GET['sortOtherAffiliateByLabel'])) {
                $orderByClause = "label {$_GET['sortOtherAffiliateByLabel']}";
                $sortUrlParamForPaging = "&sortOtherAffiliateByLabel={$_GET['sortOtherAffiliateByLabel']}";
                if($_GET['sortOtherAffiliateByLabel'] == 'ASC') {
                    $sortByLabel = "DESC";
                }
                else {
                    $sortByLabel = "ASC";
                }
            }
            elseif(isset($_GET['sortOtherAffiliateByUrl'])) {
                $orderByClause = "url {$_GET['sortOtherAffiliateByUrl']}";
                $sortUrlParamForPaging = "&sortOtherAffiliateByUrl={$_GET['sortOtherAffiliateByUrl']}";
                if($_GET['sortOtherAffiliateByUrl'] == 'ASC') {
                    $sortByUrl = "DESC";
                }
                else {
                    $sortByUrl = "ASC";
                }
            }
            elseif(isset($_GET['sortOtherAffiliateByText'])) {
                $orderByClause = "text {$_GET['sortOtherAffiliateByText']}";
                $sortUrlParamForPaging = "&sortOtherAffiliateByText={$_GET['sortOtherAffiliateByText']}";
                if($_GET['sortOtherAffiliateByText'] == 'ASC') {
                    $sortByText = "DESC";
                }
                else {
                    $sortByText = "ASC";
                }
            }
            else {
                $orderByClause = "hash {$_GET['sortOtherAffiliateByShortcode']}";
                $sortUrlParamForPaging = "&sortOtherAffiliateByShortcode={$_GET['sortOtherAffiliateByShortcode']}";
                if($_GET['sortOtherAffiliateByShortcode'] == 'ASC') {
                    $sortByHash = "DESC";
                }
                else {
                    $sortByHash = "ASC";
                }
            }
        }

        $paged = isset($_GET['paged']) ? $_GET['paged'] : 1;
        $postsPerPage = 20;
        $offset = ($paged - 1)*$postsPerPage;
        $tableName = $wpdb->prefix . 'refer_affiliate_links';

        $baseSql = " FROM $tableName WHERE is_amazon = 0 ";
        $sqlParms = [];
        $affiliates = [];
        $total = 0;
        if(isset($_GET['search']) && $_GET['search'] != '') {
            $baseSql .= " AND ( label LIKE %s OR url LIKE %s OR text LIKE %s )";
            $sqlParms = array_merge(['%' . $_GET['search'] . '%', '%' . $_GET['search'] . '%', '%' . $_GET['search'] . '%']);
            $affiliates = $wpdb->get_results($wpdb->prepare("SELECT * " . $baseSql  . " ORDER BY " . $orderByClause . " LIMIT $offset, $postsPerPage", $sqlParms));
            $total = $wpdb->get_var($wpdb->prepare("SELECT count(id) " . $baseSql, $sqlParms));
        }
        else {
        // error_log("sqlParms = " . print_r($sqlParms, true));
            $affiliates = $wpdb->get_results("SELECT * " . $baseSql  . " ORDER BY " . $orderByClause . " LIMIT $offset, $postsPerPage");
            $total = $wpdb->get_var("SELECT count(id) " . $baseSql);
        }

        // $total = $wpdb->get_var($wpdb->prepare("SELECT count(id) " . $baseSql, $sqlParms));

        $maxNumberPages = $max_num_pages = ceil($total/$postsPerPage);

        include($this->templatesDir  . 'affiliates.php');
    }

    /**
     * [handleRenderAffiliateEditTemplate description]
     * @return string
     */
    public function handleCheckAffiliateLink()
    {
        global $wpdb;

        $valid = true;
        if(!isset($_GET['_wpnonce'])) {
            $valid = false;
        } elseif(!wp_verify_nonce($_GET['_wpnonce'], 'view_edit_affiliate_' . $_GET['id'])) {
            $valid = false;
        }

        if(!$valid) {
            wp_die('Invalid nonce specified', 'Error', [
                'response'  => 403,
            ]);
        }

        $tableName = $wpdb->prefix . 'refer_affiliate_links';

        $affiliate = $wpdb->get_row($wpdb->prepare("SELECT * FROM $tableName WHERE hash = %s LIMIT 1", [$_GET['id']]));

        if(!$affiliate) {
            echo json_encode(['success' => false, 'errors' => ['Invalid affiliate.']]);
            wp_die();
        }

        //run link 404 checker 
        $link_checker = $GLOBALS['income_school']['refer_affiliates']->classes['affiliate_shortlink_converter']->check_affiliate_links($affiliate->id);

        echo json_encode(['success' => true, 'affiliate' => $affiliate, 'link_checker' => $link_checker]);
        wp_die();
    }

    /**
     * [handleRenderAffiliateEditTemplate description]
     * @return string
     */
    public function handleRenderAffiliateEditTemplate()
    {
        global $wpdb;

        $valid = true;
        if(!isset($_GET['_wpnonce'])) {
            $valid = false;
        } elseif(!wp_verify_nonce($_GET['_wpnonce'], 'view_edit_affiliate_' . $_GET['id'])) {
            $valid = false;
        }

        if(!$valid) {
            wp_die('Invalid nonce specified', 'Error', [
                'response'  => 403,
            ]);
        }

        $tableName = $wpdb->prefix . 'refer_affiliate_links';

        $affiliate = $wpdb->get_row($wpdb->prepare("SELECT * FROM $tableName WHERE hash = %s LIMIT 1", [$_GET['id']]));

        if(!$affiliate) {
            echo json_encode(['success' => false, 'errors' => ['Invalid affiliate.']]);
            wp_die();
        }

        ob_start();
        include($this->templatesDir  . '_partials/affiliate-item-row-edit.php');
        $template = ob_get_clean();

        echo json_encode(['success' => true, 'affiliate' => $affiliate, 'template' => $template]);
        wp_die();
    }

    /**
     * [handleRenderAffiliateEditTemplate description]
     * @return string
     */
    public function handlePostAffiliateEdit()
    {
        global $wpdb;

        $valid = true;
        if(!isset($_POST['_wpnonce'])) {
            $valid = false;
        } elseif(!wp_verify_nonce($_POST['_wpnonce'], 'edit_affiliate_' . $_POST['id'])) {
            $valid = false;
        }

        if(!$valid) {
            wp_die('Invalid nonce specified', 'Error', [
                'response'  => 403,
            ]);
        }

        $tableName = $wpdb->prefix . 'refer_affiliate_links';

        $affiliate = $wpdb->get_row($wpdb->prepare("SELECT * FROM $tableName WHERE hash = %s LIMIT 1", [$_POST['id']]));

        if(!$affiliate) {
            echo json_encode(['success' => false, 'errors' => ['Invalid affiliate.']]);
            wp_die();
        }

        $errors = [];

        if(!isset($_POST['label']) || $_POST['label'] == '') {
            $errors['label'] = 'Label is required.';
        }

        if(!isset($_POST['url']) || $_POST['url'] == '') {
            $errors['url'] = 'Url is required.';
        } elseif(!filter_var($_POST['url'], FILTER_VALIDATE_URL)) {
            $errors['url'] = 'Url is invalid.';
        } else {
            $existing = $wpdb->get_row($wpdb->prepare("SELECT * FROM $tableName WHERE url = %s AND hash != %s LIMIT 1", [$_POST['url'], $_POST['id']]));
            if($existing) {
                $errors['url'] = 'Url already exists.';
            }
        }

        if(!empty($errors)) {
            echo json_encode(['success' => false, 'errors' => $errors]);
            wp_die();
        }

        $wpdb->update($tableName, [
            'label' => $_POST['label'],
            'url' => $_POST['url'],
            'text' => isset($_POST['text']) && $_POST['text'] != '' ? $_POST['text'] : null
        ], [
            'hash' => $_POST['id']
        ]);


        $affiliate = $wpdb->get_row($wpdb->prepare("SELECT * FROM $tableName WHERE hash = %s LIMIT 1", [$_POST['id']]));

        ob_start();
        include($this->templatesDir  . '_partials/affiliate-item-row-view.php');
        $template = ob_get_clean();

        echo json_encode(['success' => true, 'affiliate' => $affiliate, 'template' => $template]);
        wp_die();
    }

    /**
     * [handlePostAffiliateAddSave description]
     * @return [type] [description]
     */
    public function handlePostAffiliateAdd()
    {
        global $wpdb;

        $valid = true;
        if(!isset($_POST['_wpnonce'])) {
            $valid = false;
        } elseif(!wp_verify_nonce($_POST['_wpnonce'], 'add_affiliate')) {
            $valid = false;
        }

        if(!$valid) {
            wp_die('Invalid nonce specified', 'Error', [
                'response'  => 403,
            ]);
        }

        $tableName = $wpdb->prefix . 'refer_affiliate_links';

        $errors = [];

        if(!isset($_POST['label']) || $_POST['label'] == '') {
            $errors['label'] = 'Label is required.';
        }

        if(!isset($_POST['url']) || $_POST['url'] == '') {
            $errors['url'] = 'Url is required.';
        } elseif(!filter_var($_POST['url'], FILTER_VALIDATE_URL)) {
            $errors['url'] = 'Url is invalid.';
        } else {
            $existing = $wpdb->get_row($wpdb->prepare("SELECT * FROM $tableName WHERE url = %s LIMIT 1", [$_POST['url']]));
            if($existing) {
                $errors['url'] = 'Url already exists.';
            }
        }

        if(!empty($errors)) {
            echo json_encode(['success' => false, 'errors' => $errors]);
            wp_die();
        }

        $hash = false;
        while(!$hash) {
            $tryHash = substr(md5(uniqid('', true)), 0, 8);
            if(!is_numeric($tryHash)) {
                $exists = $wpdb->get_row($wpdb->prepare("SELECT * FROM $tableName WHERE hash = %s LIMIT 1", [$tryHash]));
                if(!$exists) {
                    $hash = $tryHash;
                }
            }
        }

        $wpdb->insert($tableName, [
            'hash' => $hash,
            'label' => $_POST['label'],
            'type' => $GLOBALS['income_school']['refer_affiliates']->determine_affiliate_link_type($_POST['url']),
            'url' => $_POST['url'],
            'status' => 'created',
            'text' => isset($_POST['text']) && $_POST['text'] != '' ? $_POST['text'] : null,
            'created_at' => date("Y-m-d H:i:s")
        ]);

        $affiliate = $wpdb->get_row($wpdb->prepare("SELECT * FROM $tableName WHERE hash = %s LIMIT 1", [$hash]));

        ob_start();
        include($this->templatesDir  . '_partials/affiliate-item-row-add.php');
        include($this->templatesDir  . '_partials/affiliate-item-row-view.php');
        $template = ob_get_clean();

        echo json_encode(['success' => true, 'affiliate' => $affiliate, 'template' => $template]);
        wp_die();
    }


    /**
     * [handleRenderAffiliateDeleteTemplate description]
     * @return [type] [description]
     */
    public function handleRenderAffiliateDeleteModalTemplate()
    {
        global $wpdb;

        $valid = true;
        if(!isset($_GET['_wpnonce'])) {
            $valid = false;
        } elseif(!wp_verify_nonce($_GET['_wpnonce'], 'view_delete_affiliate_' . $_GET['id'])) {
            $valid = false;
        }

        if(!$valid) {
            wp_die('Invalid nonce specified', 'Error', [
                'response'  => 403,
            ]);
        }

        $tableName = $wpdb->prefix . 'refer_affiliate_links';

        $affiliate = $wpdb->get_row($wpdb->prepare("SELECT * FROM $tableName WHERE hash = %s LIMIT 1", [$_GET['id']]));

        if(!$affiliate) {
            echo json_encode(['success' => false, 'errors' => ['Invalid affiliate.']]);
            wp_die();
        }

        ob_start();
        include($this->templatesDir  . '_partials/affiliate-item-delete-modal.php');
        $template = ob_get_clean();

        echo json_encode(['success' => true, 'affiliate' => $affiliate, 'template' => $template]);
        wp_die();
    }

    /**
     * [handlePostAffiliateDelete description]
     * @return [type] [description]
     */
    public function handlePostAffiliateDelete()
    {
        global $wpdb;

        $valid = true;
        if(!isset($_POST['_wpnonce'])) {
            $valid = false;
        } elseif(!wp_verify_nonce($_POST['_wpnonce'], 'delete_affiliate_' . $_POST['id'])) {
            $valid = false;
        }

        if(!$valid) {
            wp_die('Invalid nonce specified', 'Error', [
                'response'  => 403,
            ]);
        }

        $tableName = $wpdb->prefix . 'refer_affiliate_links';

        $affiliate = $wpdb->get_row($wpdb->prepare("SELECT * FROM $tableName WHERE  hash = %s LIMIT 1", [$_POST['id']]));

        if(!$affiliate) {
            echo json_encode(['success' => false, 'errors' => ['Invalid affiliate.']]);
            wp_die();
        }

        $deleted = $wpdb->delete($tableName, [
            'id' => $affiliate->id
        ]);

        echo json_encode(['success' => true]);
        wp_die();
    }


    /**
     * [handleRenderAffiliateInsertShortcodeModalTemplate description]
     * @return [type] [description]
     */
    public function handleRenderAffiliateInsertShortcodeModalTemplate()
    {
        global $wpdb, $paged, $max_num_pages;

        $paged = isset($_REQUEST['paged']) ? $_REQUEST['paged'] : 1;
        $postsPerPage = 6;
        $offset = ($paged - 1)*$postsPerPage;
        $tableName = $wpdb->prefix . 'refer_affiliate_links';

        $baseSql = " FROM $tableName WHERE is_amazon = 0 ";
        $sqlParms = [];
        $search = $_REQUEST['search'];
        if($search != '') {
            $baseSql .= " AND ( label LIKE %s OR url LIKE %s OR text LIKE %s )";
            $sqlParms = array_merge(['%' . $search . '%', '%' . $search . '%', '%' . $search . '%']);
        }

        $affiliates = $wpdb->get_results($wpdb->prepare("SELECT * " . $baseSql  . " ORDER BY label ASC LIMIT $offset, $postsPerPage", $sqlParms));

        $total = $wpdb->get_var($wpdb->prepare("SELECT count(id) " . $baseSql, $params));

        $maxNumberPages = $max_num_pages = ceil($total/$postsPerPage);

        ob_start();
        include($this->templatesDir  . '_partials/affiliate-insert-shortcode-modal.php');

        echo json_encode(['success' => true, 'affiliates' => $affiliates, 'template' => ob_get_clean()]);
        wp_die();
    }


    public function renderAutomationsPage()
    {
        include($this->templatesDir  . 'automations.php');
    }


    public function renderTreesPage()
    {
        include($this->templatesDir  . 'trees.php');
    }


    public function renderLinkManagerPage()
    {
        include($this->templatesDir  . 'link-manager.php');
    }


    /**
     * [renderReportByAuthorPage description]
     * @return [type] [description]
     */
    public function renderReportsDashboardPage()
    {
        $membershipEmail = isset($_POST['membershipEmail']) ? $_POST['membershipEmail'] : '';
        if($membershipEmail != "") {
            update_option("refer-membership-email", $membershipEmail);
        }

        $affiliateType = isset($_GET['affiliateType']) ? $_GET['affiliateType'] : 'all';
        $numDays = isset($_GET['numDays']) ? $_GET['numDays'] : 30;
        $page = isset($_GET['page']) ? $_GET['page'] : "";

        $currentParams = "&numDays={$numDays}&affiliateType={$affiliateType}";

        /******************************** Countdowns ******************************/

        /**
         * Query the countdown click by post data
         */
        $sortPostReportByTitle = "ASC";
        $sortPostReportByViews = "DESC";
        $sortPostReportByClicks = "DESC";

        $cachedReportData = get_option('refer-plugin-countdown-totals-by-post-report-data', array());

        //set the report key for countdowns
        $reportsDataKey = $numDays . '_days';

        $rows = array();
        if(isset($cachedReportData['data'][$reportsDataKey])) {
            $rows = $cachedReportData['data'][$reportsDataKey];
        }

        if(isset($_GET['sortPostReportByTitle']) || isset($_GET['sortPostReportByViews']) || isset($_GET['sortPostReportByClicks'])) {
            if(isset($_GET['sortPostReportByTitle'])) {
                if($_GET['sortPostReportByTitle'] == 'ASC') {
                    $sortUrlParamForPaging = "&sortPostReportByTitle=ASC";
                    $sortPostReportByTitle = "ASC";
                    array_multisort(array_column($rows, 'title'), SORT_ASC, $rows);
                } else {
                    $sortUrlParamForPaging = "&sortPostReportByTitle=DESC";
                    $sortPostReportByTitle = "DESC";
                    array_multisort(array_column($rows, 'title'), SORT_DESC, $rows);
                }
            } elseif(isset($_GET['sortPostReportByViews'])) {
                if($_GET['sortPostReportByViews'] == 'ASC') {
                    $sortUrlParamForPaging = "&sortPostReportByViews=ASC";
                    $sortPostReportByViews = "ASC";
                    array_multisort(array_column($rows, 'views'), SORT_ASC, $rows);
                } else {
                    $sortUrlParamForPaging = "&sortPostReportByViews=DESC";
                    $sortPostReportByViews = "DESC";
                    array_multisort(array_column($rows, 'views'), SORT_DESC, $rows);
                }
            } else {

                if($_GET['sortPostReportByClicks'] == 'ASC') {
                    $sortUrlParamForPaging = "&sortPostReportByClicks=ASC";
                    $sortPostReportByClicks = "ASC";
                    array_multisort(array_column($rows, 'clicks'), SORT_ASC, $rows);
                } else {
                    $sortUrlParamForPaging = "&sortPostReportByClicks=DESC";
                    $sortPostReportByClicks = "DESC";
                    array_multisort(array_column($rows, 'clicks'), SORT_DESC, $rows);
                }
            }
        } else {
            array_multisort(array_column($rows, 'clicks'), SORT_DESC, $rows);
        }

        $countdownPostStats = array_slice($rows, 0, 30);


        /**
         * Query The total countdown clicks chart Data
         */

        $cachedReportData = get_option('refer-plugin-total-countdowns-report-data', array());

        $rows = array();
        if(isset($cachedReportData['data'])) {
            $rows = $cachedReportData['data'][$numDays . '_days'];
        }

        $countdownsChartData = $rows;

        /******************************** Affiliates ******************************/


        //change the report key for affiliates
        $reportsDataKey = $numDays . '_days_' . $affiliateType . '_type';

        /**
         * Query the affiliate click by post data
         */
        $sortPostReportByTitle = "ASC";
        $sortPostReportByViews = "DESC";
        $sortPostReportByClicks = "DESC";

        $cachedReportData = get_option('refer-plugin-totals-by-post-report-data', array());


        $rows = array();
        if(isset($cachedReportData['data'][$reportsDataKey])) {
            $rows = $cachedReportData['data'][$reportsDataKey];
        }

        if(isset($_GET['sortPostReportByTitle']) || isset($_GET['sortPostReportByViews']) || isset($_GET['sortPostReportByClicks'])) {
            if(isset($_GET['sortPostReportByTitle'])) {
                if($_GET['sortPostReportByTitle'] == 'ASC') {
                    $sortUrlParamForPaging = "&sortPostReportByTitle=ASC";
                    $sortPostReportByTitle = "ASC";
                    array_multisort(array_column($rows, 'title'), SORT_ASC, $rows);
                } else {
                    $sortUrlParamForPaging = "&sortPostReportByTitle=DESC";
                    $sortPostReportByTitle = "DESC";
                    array_multisort(array_column($rows, 'title'), SORT_DESC, $rows);
                }
            } elseif(isset($_GET['sortPostReportByViews'])) {
                if($_GET['sortPostReportByViews'] == 'ASC') {
                    $sortUrlParamForPaging = "&sortPostReportByViews=ASC";
                    $sortPostReportByViews = "ASC";
                    array_multisort(array_column($rows, 'views'), SORT_ASC, $rows);
                } else {
                    $sortUrlParamForPaging = "&sortPostReportByViews=DESC";
                    $sortPostReportByViews = "DESC";
                    array_multisort(array_column($rows, 'views'), SORT_DESC, $rows);
                }
            } else {

                if($_GET['sortPostReportByClicks'] == 'ASC') {
                    $sortUrlParamForPaging = "&sortPostReportByClicks=ASC";
                    $sortPostReportByClicks = "ASC";
                    array_multisort(array_column($rows, 'clicks'), SORT_ASC, $rows);
                } else {
                    $sortUrlParamForPaging = "&sortPostReportByClicks=DESC";
                    $sortPostReportByClicks = "DESC";
                    array_multisort(array_column($rows, 'clicks'), SORT_DESC, $rows);
                }
            }
        } else {
            array_multisort(array_column($rows, 'clicks'), SORT_DESC, $rows);
        }

        $postStats = array_slice($rows, 0, 30);

        /**
         * Query The total affilate clicks chart Data
         */

        //examine Daily click tracking data
        // global $wpdb;
        // $daily_trackings = $wpdb->get_results("SELECT * FROM " . $wpdb->prefix . 'refer_link_daily_trackings');
        // var_dump($daily_trackings);

        $cachedReportData = get_option('refer-plugin-total-clicks-report-data', array());
        
        // var_dump($cachedReportData);

        $rows = array(0);
        if(isset($cachedReportData['data'][$reportsDataKey])) {
            $rows = $cachedReportData['data'][$reportsDataKey];
        }

        $affiliatesChartData = $rows;

        /**
         * Query the affiliate click by product data
         */

        $sortProductReportByName = "ASC";
        $sortProductReportByViews = "DESC";
        $sortProductReportByClicks = "DESC";

        $cachedReportData = get_option('refer-plugin-totals-by-product-report-data', array());

        $rows = array();
        if(isset($cachedReportData['data'][$reportsDataKey])) {
            $rows = $cachedReportData['data'][$reportsDataKey];
        }

        if(isset($_GET['sortProductReportByName']) || isset($_GET['sortProductReportByViews']) || isset($_GET['sortProductReportByClicks'])) {
            if(isset($_GET['sortProductReportByName'])) {
                if($_GET['sortProductReportByName'] == 'ASC') {
                    $sortUrlParamForPaging = "&sortProductReportByName=ASC";
                    $sortProductReportByName = "ASC";
                    array_multisort(array_column($rows, 'name'), SORT_ASC, $rows);
                } else {
                    $sortUrlParamForPaging = "&sortProductReportByName=DESC";
                    $sortProductReportByName = "DESC";
                    array_multisort(array_column($rows, 'name'), SORT_DESC, $rows);
                }
            } elseif(isset($_GET['sortProductReportByViews'])) {
                if($_GET['sortProductReportByViews'] == 'ASC') {
                    $sortUrlParamForPaging = "&sortProductReportByViews=ASC";
                    $sortProductReportByViews = "ASC";
                    array_multisort(array_column($rows, 'views'), SORT_ASC, $rows);
                } else {
                    $sortUrlParamForPaging = "&sortProductReportByViews=DESC";
                    $sortProductReportByViews = "DESC";
                    array_multisort(array_column($rows, 'views'), SORT_DESC, $rows);
                }
            } else {

                if($_GET['sortProductReportByClicks'] == 'ASC') {
                    $sortUrlParamForPaging = "&sortProductReportByClicks=ASC";
                     $sortProductReportByClicks = "ASC";
                    array_multisort(array_column($rows, 'clicks'), SORT_ASC, $rows);
                } else {
                    $sortUrlParamForPaging = "&sortProductReportByClicks=DESC";
                    $sortProductReportByClicks = "DESC";
                    array_multisort(array_column($rows, 'clicks'), SORT_DESC, $rows);
                }
            }
        } else {
            array_multisort(array_column($rows, 'clicks'), SORT_DESC, $rows);
        }

        $productStats = array_slice($rows, 0, 30);


        /**
         * Query the affiliate click by author
         */

        $sortAuthorReportByName = "ASC";
        $sortAuthorReportByViews = "DESC";
        $sortAuthorReportByClicks = "DESC";
        $sortAuthorReportByAvg = "DESC";

        $cachedReportData = get_option('refer-plugin-totals-by-author-report-data', array());

        $rows = array();
        if(isset($cachedReportData['data'][$reportsDataKey])) {
            $rows = $cachedReportData['data'][$reportsDataKey];
        }

        if(isset($_GET['sortAuthorReportByName']) || isset($_GET['sortAuthorReportByViews']) || isset($_GET['sortAuthorReportByClicks']) || isset($_GET['sortAuthorReportByAvg'])) {
            if(isset($_GET['sortAuthorReportByName'])) {
                if($_GET['sortAuthorReportByName'] == 'ASC') {
                    $sortUrlParamForPaging = "&sortAuthorReportByName=ASC";
                    $sortByName = "DESC";
                    array_multisort(array_column($rows, 'name'), SORT_ASC, $rows);
                } else {
                    $sortUrlParamForPaging = "&sortAuthorReportByName=DESC";
                    $sortByName = "DESC";
                    array_multisort(array_column($rows, 'name'), SORT_DESC, $rows);
                }
            } elseif(isset($_GET['sortAuthorReportByViews'])) {
                if($_GET['sortAuthorReportByViews'] == 'ASC') {
                    $sortUrlParamForPaging = "&sortAuthorReportByViews=ASC";
                    $sortAuthorReportByViews = "ASC";
                    array_multisort(array_column($rows, 'views'), SORT_ASC, $rows);
                } else {
                    $sortUrlParamForPaging = "&sortAuthorReportByViews=DESC";
                    $sortAuthorReportByViews = "DESC";
                    array_multisort(array_column($rows, 'views'), SORT_DESC, $rows);
                }
            } elseif(isset($_GET['sortAuthorReportByClicks'])) {
                if($_GET['sortAuthorReportByClicks'] == 'ASC') {
                    $sortUrlParamForPaging = "&sortAuthorReportByClicks=ASC";
                    $sortAuthorReportByClicks = "ASC";
                    array_multisort(array_column($rows, 'clicks'), SORT_ASC, $rows);
                } else {
                    $sortUrlParamForPaging = "&sortAuthorReportByClicks=DESC";
                    $sortAuthorReportByClicks = "DESC";
                    array_multisort(array_column($rows, 'clicks'), SORT_DESC, $rows);
                }
            } elseif(isset($_GET['sortAuthorReportByStatus'])) {
                if($_GET['sortAuthorReportByStatus'] == 'ASC') {
                    $sortUrlParamForPaging = "&sortAuthorReportByStatus=ASC";
                    $sortAuthorReportByStatus = "ASC";
                    array_multisort(array_column($rows, 'Status'), SORT_ASC, $rows);
                } else {
                    $sortUrlParamForPaging = "&sortAuthorReportByStatus=DESC";
                    $sortAuthorReportByStatus = "DESC";
                    array_multisort(array_column($rows, 'Status'), SORT_DESC, $rows);
                }
            } else {
                if($_GET['sortAuthorReportByAvg'] == 'ASC') {
                    $sortUrlParamForPaging = "&sortAuthorReportByAvg=ASC";
                    $sortAuthorReportByAvg = "ASC";
                    array_multisort(array_column($rows, 'avg'), SORT_ASC, $rows);
                } else {
                    $sortUrlParamForPaging = "&sortAuthorReportByAvg=DESC";
                    $sortAuthorReportByAvg = "DESC";
                    array_multisort(array_column($rows, 'avg'), SORT_DESC, $rows);
                }
            }
        } else {
            array_multisort(array_column($rows, 'clicks'), SORT_DESC, $rows);
        }

        $authorStats = array_slice($rows, 0, 30);



        /**
         * Query the affiliate click by category data
         */

        $sortCatReportByTitle = "ASC";
        $sortCatReportByWordCount = "ASC";
        $sortCatReportByViews = "DESC";
        $sortCatReportByClicks = "DESC";

        $cachedReportData = get_option('refer-plugin-totals-by-category-report-data', array());

        $rows = array();
        if(isset($cachedReportData['data'][$reportsDataKey])) {
            $rows = $cachedReportData['data'][$reportsDataKey];
        }

        if(isset($_GET['sortCatReportByTitle']) || isset($_GET['sortCatReportByViews']) || isset($_GET['sortCatReportByClicks']) || isset($_GET['sortCatReportByWordCount'])) {
            if(isset($_GET['sortCatReportByTitle'])) {
                if($_GET['sortCatReportByTitle'] == 'ASC') {
                    $sortUrlParamForPaging = "&sortCatReportByTitle=ASC";
                    $sortCatReportByTitle = "ASC";
                    array_multisort(array_column($rows, 'title'), SORT_ASC, $rows);
                } else {
                    $sortUrlParamForPaging = "&sortCatReportByTitle=DESC";
                    $sortCatReportByTitle = "DESC";
                    array_multisort(array_column($rows, 'title'), SORT_DESC, $rows);
                }
            } elseif(isset($_GET['sortCatReportByViews'])) {
                if($_GET['sortCatReportByViews'] == 'ASC') {
                    $sortUrlParamForPaging = "&sortCatReportByViews=ASC";
                    $sortCatReportByViews = "ASC";
                    array_multisort(array_column($rows, 'views'), SORT_ASC, $rows);
                } else {
                    $sortUrlParamForPaging = "&sortCatReportByViews=DESC";
                    $sortCatReportByViews = "DESC";
                    array_multisort(array_column($rows, 'views'), SORT_DESC, $rows);
                }
             } elseif(isset($_GET['sortCatReportByWordCount'])) {
                if($_GET['sortCatReportByWordCount'] == 'ASC') {
                    $sortUrlParamForPaging = "&sortCatReportByWordCount=ASC";
                    $sortCatReportByWordCount = "ASC";
                    array_multisort(array_column($rows, 'word_count'), SORT_ASC, $rows);
                } else {
                    $sortUrlParamForPaging = "&sortCatReportByWordCount=DESC";
                    $sortCatReportByWordCount = "DESC";
                    array_multisort(array_column($rows, 'word_count'), SORT_DESC, $rows);
                }
            }else {
                if($_GET['sortCatReportByClicks'] == 'ASC') {
                    $sortUrlParamForPaging = "&sortCatReportByClicks=ASC";
                    $sortCatReportByClicks = "ASC";
                    array_multisort(array_column($rows, 'clicks'), SORT_ASC, $rows);
                } else {
                    $sortUrlParamForPaging = "&sortCatReportByClicks=DESC";
                    $sortCatReportByClicks = "DESC";
                    array_multisort(array_column($rows, 'clicks'), SORT_DESC, $rows);
                }
            }
        } else {
            array_multisort(array_column($rows, 'clicks'), SORT_DESC, $rows);
        }

        $taxStats = array_slice($rows, 0, 30);


        /**
         * Query the affiliate click by word count data
         */

        $cachedReportData = get_option('refer-plugin-totals-by-word-count-report-data', array());

        $rows = array();
        if(isset($cachedReportData['data'][$reportsDataKey])) {
            $rows = $cachedReportData['data'][$reportsDataKey];
        }

        if(isset($_GET['sortWordCountReportByTitle']) || isset($_GET['sortWordCountReportByViews']) || isset($_GET['sortWordCountReportByClicks'])) {
            if(isset($_GET['sortWordCountReportByTitle'])) {
                if($_GET['sortWordCountReportByTitle'] == 'ASC') {
                    $sortUrlParamForPaging = "&sortWordCountReportByTitle=ASC";
                    $sortWordCountReportByTitle = "DESC";
                    array_multisort(array_column($rows, 'title'), SORT_ASC, $rows);
                } else {
                    $sortUrlParamForPaging = "&sortWordCountReportByTitle=DESC";
                    $sortWordCountReportByTitle = "DESC";
                    array_multisort(array_column($rows, 'title'), SORT_DESC, $rows);
                }
            } elseif(isset($_GET['sortWordCountReportByViews'])) {
                if($_GET['sortWordCountReportByViews'] == 'ASC') {
                    $sortUrlParamForPaging = "&sortWordCountReportByViews=ASC";
                    $sortWordCountReportByViews = "ASC";
                    array_multisort(array_column($rows, 'views'), SORT_ASC, $rows);
                } else {
                    $sortUrlParamForPaging = "&sortWordCountReportByViews=DESC";
                    $sortWordCountReportByViews = "DESC";
                    array_multisort(array_column($rows, 'views'), SORT_DESC, $rows);
                }
            } else {
                if($_GET['sortWordCountReportByClicks'] == 'ASC') {
                    $sortUrlParamForPaging = "&sortWordCountReportByClicks=ASC";
                    $sortWordCountReportByClicks = "ASC";
                    array_multisort(array_column($rows, 'clicks'), SORT_ASC, $rows);
                } else {
                    $sortUrlParamForPaging = "&sortWordCountReportByClicks=DESC";
                    $sortWordCountReportByClicks = "DESC";
                    array_multisort(array_column($rows, 'clicks'), SORT_DESC, $rows);
                }
            }
        } else {
            array_multisort(array_column($rows, 'clicks'), SORT_DESC, $rows);
        }

        $wordCountStats = $rows;

        include($this->templatesDir . 'reports-dashboard.php');
    }

    /**
     * [renderReportByAuthorPage description]
     * @return [type] [description]
     */
    public function renderReportByAuthorPage()
    {

        $cachedReportData = get_option('refer-plugin-totals-by-author-report-data', array());

        $rows = array();
        if(isset($cachedReportData['data']['all_days_all_type'])) {
            $rows = $cachedReportData['data']['all_days_all_type'];
        }

        //defaults
        $sortByName = "ASC";
        $sortByViews = "DESC";
        $sortByClicks = "DESC";
        $sortByAvg = "DESC";
        $sortUrlParamForPaging = "";
        if(isset($_GET['sortAuthorReportByName']) || isset($_GET['sortAuthorReportByViews']) || isset($_GET['sortAuthorReportByClicks']) || isset($_GET['sortAuthorReportByAvg'])) {
            if(isset($_GET['sortAuthorReportByName'])) {
                if($_GET['sortAuthorReportByName'] == 'ASC') {
                    $sortUrlParamForPaging = "&sortAuthorReportByName=ASC";
                    $sortByName = "DESC";
                    array_multisort(array_column($rows, 'name'), SORT_ASC, $rows);
                } else {
                    $sortUrlParamForPaging = "&sortAuthorReportByName=DESC";
                    $sortByName = "DESC";
                    array_multisort(array_column($rows, 'name'), SORT_DESC, $rows);
                }
            } elseif(isset($_GET['sortAuthorReportByViews'])) {
                if($_GET['sortAuthorReportByViews'] == 'ASC') {
                    $sortUrlParamForPaging = "&sortAuthorReportByViews=ASC";
                    $sortByViews = "ASC";
                    array_multisort(array_column($rows, 'views'), SORT_ASC, $rows);
                } else {
                    $sortUrlParamForPaging = "&sortAuthorReportByViews=DESC";
                    $sortByViews = "DESC";
                    array_multisort(array_column($rows, 'views'), SORT_DESC, $rows);
                }
            } elseif(isset($_GET['sortAuthorReportByClicks'])) {
                if($_GET['sortAuthorReportByClicks'] == 'ASC') {
                    $sortUrlParamForPaging = "&sortAuthorReportByClicks=ASC";
                    $sortByClicks = "ASC";
                    array_multisort(array_column($rows, 'clicks'), SORT_ASC, $rows);
                } else {
                    $sortUrlParamForPaging = "&sortAuthorReportByClicks=DESC";
                    $sortByClicks = "DESC";
                    array_multisort(array_column($rows, 'clicks'), SORT_DESC, $rows);
                }
            } else {
                if($_GET['sortAuthorReportByAvg'] == 'ASC') {
                    $sortUrlParamForPaging = "&sortAuthorReportByAvg=ASC";
                    $sortByAvg = "ASC";
                    array_multisort(array_column($rows, 'avg'), SORT_ASC, $rows);
                } else {
                    $sortUrlParamForPaging = "&sortAuthorReportByAvg=DESC";
                    $sortByAvg = "DESC";
                    array_multisort(array_column($rows, 'avg'), SORT_DESC, $rows);
                }
            }
        } else {
            array_multisort(array_column($rows, 'clicks'), SORT_DESC, $rows);
        }

        $limit = 20; //post per page
        $paged = 1;
        if(isset($_GET['paged'])) {
            $paged = $_GET['paged'];
        }
        $rowCount = count($rows);
        $totalPages = ceil($rowCount/$limit);
        $page = max($paged, 1);
        $page = min($paged, $totalPages);
        $offset = ($page - 1) * $limit;

        $authorStats = array_slice($rows, $offset, $limit);

        include($this->templatesDir . 'report-by-author.php');
     }

    /**
     * [renderReportByCategoryPage description]
     * @return [type] [description]
     */
    public function renderReportByCategoryPage()
    {

        $cachedReportData = get_option('refer-plugin-totals-by-category-report-data', array());

        $rows = array();
        if(isset($cachedReportData['data']['all_days_all_type'])) {
            $rows = $cachedReportData['data']['all_days_all_type'];
        }

        //defaults
        $sortByTitleDirection = "ASC";
        $sortByViews = "DESC";
        $sortByClicks = "DESC";
        $sortUrlParamForPaging = "";
        if(isset($_GET['sortCatReportByTitle']) || isset($_GET['sortCatReportByViews']) || isset($_GET['sortCatReportByClicks'])) {
            if(isset($_GET['sortCatReportByTitle'])) {
                if($_GET['sortCatReportByTitle'] == 'ASC') {
                    $sortUrlParamForPaging = "&sortCatReportByTitle=ASC";
                    $sortByTitleDirection = "DESC";
                    array_multisort(array_column($rows, 'title'), SORT_ASC, $rows);
                } else {
                    $sortUrlParamForPaging = "&sortCatReportByTitle=DESC";
                    $sortByTitleDirection = "DESC";
                    array_multisort(array_column($rows, 'title'), SORT_DESC, $rows);
                }
            } elseif(isset($_GET['sortCatReportByViews'])) {
                if($_GET['sortCatReportByViews'] == 'ASC') {
                    $sortUrlParamForPaging = "&sortCatReportByViews=ASC";
                    $sortByViews = "ASC";
                    array_multisort(array_column($rows, 'views'), SORT_ASC, $rows);
                } else {
                    $sortUrlParamForPaging = "&sortCatReportByViews=DESC";
                    $sortByViews = "DESC";
                    array_multisort(array_column($rows, 'views'), SORT_DESC, $rows);
                }
            } else {
                if($_GET['sortCatReportByClicks'] == 'ASC') {
                    $sortUrlParamForPaging = "&sortCatReportByClicks=ASC";
                    $sortByClicks = "ASC";
                    array_multisort(array_column($rows, 'clicks'), SORT_ASC, $rows);
                } else {
                    $sortUrlParamForPaging = "&sortCatReportByClicks=DESC";
                    $sortByClicks = "DESC";
                    array_multisort(array_column($rows, 'clicks'), SORT_DESC, $rows);
                }
            }
        } else {
            array_multisort(array_column($rows, 'clicks'), SORT_DESC, $rows);
        }

        $limit = 20; //post per page
        $paged = 1;
        if(isset($_GET['paged'])) {
            $paged = $_GET['paged'];
        }
        $rowCount = count($rows);
        $totalPages = ceil($rowCount/$limit);
        $page = max($paged, 1);
        $page = min($paged, $totalPages);
        $offset = ($page - 1) * $limit;

        $taxStats = array_slice($rows, $offset, $limit);

        include($this->templatesDir . 'report-by-category.php');
    }

    /**
     * [renderReportByPostPage description]
     * @return [type] [description]
     */
    public function renderReportByPostPage()
    {
        $cachedReportData = get_option('refer-plugin-totals-by-post-report-data', array());

        $rows = array();
        if(isset($cachedReportData['data']['all_days_all_type'])) {
            $rows = $cachedReportData['data']['all_days_all_type'];
        }

        //defaults
        $sortByTitleDirection = "ASC";
        $sortByViews = "DESC";
        $sortByClicks = "DESC";
        $sortUrlParamForPaging = "";
        if(isset($_GET['sortPostReportByTitle']) || isset($_GET['sortPostReportByViews']) || isset($_GET['sortPostReportByClicks'])) {
            if(isset($_GET['sortPostReportByTitle'])) {
                if($_GET['sortPostReportByTitle'] == 'ASC') {
                    $sortUrlParamForPaging = "&sortPostReportByTitle=ASC";
                    $sortByTitleDirection = "DESC";
                    array_multisort(array_column($rows, 'title'), SORT_ASC, $rows);
                } else {
                    $sortUrlParamForPaging = "&sortPostReportByTitle=DESC";
                    $sortByTitleDirection = "DESC";
                    array_multisort(array_column($rows, 'title'), SORT_DESC, $rows);
                }
            } elseif(isset($_GET['sortPostReportByViews'])) {
                if($_GET['sortPostReportByViews'] == 'ASC') {
                    $sortUrlParamForPaging = "&sortPostReportByViews=ASC";
                    $sortByViews = "ASC";
                    array_multisort(array_column($rows, 'views'), SORT_ASC, $rows);
                } else {
                    $sortUrlParamForPaging = "&sortPostReportByViews=DESC";
                    $sortByViews = "DESC";
                    array_multisort(array_column($rows, 'views'), SORT_DESC, $rows);
                }
            } else {

                if($_GET['sortPostReportByClicks'] == 'ASC') {
                    $sortUrlParamForPaging = "&sortPostReportByClicks=ASC";
                    $sortByClicks = "ASC";
                    array_multisort(array_column($rows, 'clicks'), SORT_ASC, $rows);
                } else {
                    $sortUrlParamForPaging = "&sortPostReportByClicks=DESC";
                    $sortByClicks = "DESC";
                    array_multisort(array_column($rows, 'clicks'), SORT_DESC, $rows);
                }
            }
        } else {
            array_multisort(array_column($rows, 'clicks'), SORT_DESC, $rows);
        }

        $limit = 20; //post per page
        $paged = 1;
        if(isset($_GET['paged'])) {
            $paged = $_GET['paged'];
        }
        $rowCount = count($rows);
        $totalPages = ceil($rowCount/$limit);
        $page = max($paged, 1);
        $page = min($paged, $totalPages);
        $offset = ($page - 1) * $limit;

        $postStats = array_slice($rows, $offset, $limit);

        include($this->templatesDir . 'report-by-post.php');
    }

    /**
     * [renderReportByProductPage description]
     * @return [type] [description]
     */
    public function renderReportByProductPage()
    {
        $cachedReportData = get_option('refer-plugin-totals-by-product-report-data', array());

        $rows = array();
        if(isset($cachedReportData['data']['all_days_all_type'])) {
            $rows = $cachedReportData['data']['all_days_all_type'];
        }

        //defaults
        $sortByName = "ASC";
        $sortByViews = "DESC";
        $sortByClicks = "DESC";
        $sortUrlParamForPaging = "";
        if(isset($_GET['sortProductReportByName']) || isset($_GET['sortProductReportByViews']) || isset($_GET['sortProductReportByClicks'])) {
            if(isset($_GET['sortProductReportByName'])) {
                if($_GET['sortProductReportByName'] == 'ASC') {
                    $sortUrlParamForPaging = "&sortProductReportByName=ASC";
                    $sortByName = "DESC";
                    array_multisort(array_column($rows, 'name'), SORT_ASC, $rows);
                } else {
                    $sortUrlParamForPaging = "&sortProductReportByName=DESC";
                    $sortByName = "DESC";
                    array_multisort(array_column($rows, 'name'), SORT_DESC, $rows);
                }
            } elseif(isset($_GET['sortProductReportByViews'])) {
                if($_GET['sortProductReportByViews'] == 'ASC') {
                    $sortUrlParamForPaging = "&sortProductReportByViews=ASC";
                    $sortByViews = "ASC";
                    array_multisort(array_column($rows, 'views'), SORT_ASC, $rows);
                } else {
                    $sortUrlParamForPaging = "&sortProductReportByViews=DESC";
                    $sortByViews = "DESC";
                    array_multisort(array_column($rows, 'views'), SORT_DESC, $rows);
                }
            } else {

                if($_GET['sortProductReportByClicks'] == 'ASC') {
                    $sortUrlParamForPaging = "&sortProductReportByClicks=ASC";
                    $sortByClicks = "ASC";
                    array_multisort(array_column($rows, 'clicks'), SORT_ASC, $rows);
                } else {
                    $sortUrlParamForPaging = "&sortProductReportByClicks=DESC";
                    $sortByClicks = "DESC";
                    array_multisort(array_column($rows, 'clicks'), SORT_DESC, $rows);
                }
            }
        } else {
            array_multisort(array_column($rows, 'clicks'), SORT_DESC, $rows);
        }

        $limit = 20; //post per page
        $paged = 1;
        if(isset($_GET['paged'])) {
            $paged = $_GET['paged'];
        }
        $rowCount = count($rows);
        $totalPages = ceil($rowCount/$limit);
        $page = max($paged, 1);
        $page = min($paged, $totalPages);
        $offset = ($page - 1) * $limit;

        $productStats = array_slice($rows, $offset, $limit);

        include($this->templatesDir . 'report-by-product.php');
    }

    /**
     * [renderReportByWordCount description]
     * @return [type] [description]
     */
    public function renderReportByWordCountPage()
    {
        $cachedReportData = get_option('refer-plugin-totals-by-word-count-report-data', array());

        $rows = array();
        if(isset($cachedReportData['data']['all_days_all_type'])) {
            $rows = $cachedReportData['data']['all_days_all_type'];
        }

        array_multisort(array_column($rows, 'clicks'), SORT_DESC, $rows);

        $wordCountStats = $rows;

        include($this->templatesDir . 'report-by-word-count.php');
    }

    /**
     * generate a list of countdowns for the dashboard page(s) including search and sort.
     * returns an array of two params: links (list of coundown objects and total # of links)
     */
    public function listCountdowns() {
        global $wpdb, $paged, $max_num_pages;

        // wp_enqueue_media();
        $orderByClause = "product_name ASC";
        $sortByName = "DESC";
        $sortByUrl = "ASC";
        $sortByEndDate = "ASC";
        $sortAuthorReportByStatus = "DESC";
        $sortByViews = "DESC";
        $sortByStatus = "DESC";
        $sortByClicks = "DESC";

        $tableName = $wpdb->prefix . 'refer_countdown_links';
        $total = $wpdb->get_var("SELECT count(id) FROM $tableName");
        $maxNumberPages = $total;

        if(isset($_GET['sortCountdownByName']) || isset($_GET['sortCountdownByUrl']) || isset($_GET['sortCountdownByEndDate']) || isset($_GET['sortCountdownByViews'])) {

            if(isset($_GET['sortCountdownByName'])) {
                $orderByClause = "product_name {$_GET['sortCountdownByName']}";
                if($_GET['sortCountdownByName'] == 'ASC') {
                    $sortByName = "DESC";
                }
                else {
                    $sortByName = "ASC";
                }
            }
            elseif(isset($_GET['sortCountdownByUrl'])) {
                $orderByClause = "purchase_link {$_GET['sortCountdownByUrl']}";
                if($_GET['sortCountdownByUrl'] == 'ASC') {
                    $sortByUrl = "DESC";
                }
                else {
                    $sortByUrl = "ASC";
                }
            }
            elseif(isset($_GET['sortCountdownByEndDate'])) {
                $orderByClause = "end_date {$_GET['sortCountdownByEndDate']}";
                if($_GET['sortCountdownByEndDate'] == 'ASC') {
                    $sortByEndDate = "DESC";
                }
                else {
                    $sortByEndDate = "ASC";
                }
            }
            elseif(isset($_GET['sortCountdownByViews'])) {
                $orderByClause = "views {$_GET['sortCountdownByViews']}";
                if($_GET['sortCountdownByViews'] == 'ASC') {
                    $sortByViews = "DESC";
                }
                else {
                    $sortByViews = "ASC";
                }
            }
            elseif(isset($_GET['sortCountdownByStatus'])) {
                $orderByClause = "Status {$_GET['sortCountdownByStatus']}";
                if($_GET['sortCountdownByStatus'] == 'ASC') {
                    $sortByStatus = "DESC";
                }
                else {
                    $sortByStatus = "ASC";
                }
            }
            elseif(isset($_GET['sortCountdownByClicks'])) {
                $orderByClause = "Clicks {$_GET['sortCountdownByClicks']}";
                if($_GET['sortCountdownByClicks'] == 'ASC') {
                    $sortByClicks = "DESC";
                }
                else {
                    $sortByClicks = "ASC";
                }
            }
        }

        $paged = isset($_GET['paged']) ? $_GET['paged'] : 1;
        $postsPerPage = 20;
        $offset = ($paged - 1)*$postsPerPage;

        $baseSql = " FROM $tableName WHERE 1=1 ";
        $sqlParms = [];
        $links = [];
        $total = 0;
        if(isset($_GET['search']) && $_GET['search'] != '') {
            $baseSql .= " AND ( product_name LIKE %s OR purchase_link LIKE %s OR button_label LIKE %s  OR call_to_action LIKE %s)";
            $sqlParms = array_merge(['%' . $_GET['search'] . '%', '%' . $_GET['search'] . '%', '%' . $_GET['search'] . '%', '%' . $_GET['search'] . '%']);
            $links = $wpdb->get_results($wpdb->prepare("SELECT * " . $baseSql  . " ORDER BY " . $orderByClause . " LIMIT $offset, $postsPerPage", $sqlParms));
            $total = $wpdb->get_var($wpdb->prepare("SELECT count(id) " . $baseSql, $sqlParms));
        }
        else {
            // error_log("sqlParms = " . print_r($sqlParms, true));
            $links = $wpdb->get_results("SELECT * " . $baseSql  . " ORDER BY " . $orderByClause . " LIMIT $offset, $postsPerPage");

            $total = $wpdb->get_var("SELECT count(id) " . $baseSql);
        }

        return array(
            'paged' => $paged,
            'links' => $links,
            'total' => $total,
            'sortByName' => $sortByName,
            'sortByUrl' => $sortByUrl,
            'sortByEndDate' => $sortByEndDate,
            'sortAuthorReportByStatus' => $sortAuthorReportByStatus,
            'sortByViews' => $sortByViews,
            'sortByStatus' => $sortByStatus,
            'sortByClicks' => $sortByClicks,
        );
    }

    /**
     * render the main countdowns dashboard page.
     */
    public function renderCountdownsPage() {
        
        $countdowns_array = $this->listCountdowns();
        $paged = $countdowns_array['paged'];
        $links = $countdowns_array['links'];
        $total = $countdowns_array['total'];
        $sortByName = $countdowns_array['sortByName'];
        $sortByUrl = $countdowns_array['sortByUrl'];
        $sortByEndDate = $countdowns_array['sortByEndDate'];
        $sortAuthorReportByStatus = $countdowns_array['sortAuthorReportByStatus'];
        $sortByViews = $countdowns_array['sortByViews'];
        $sortByStatus = $countdowns_array['sortByStatus'];
        $sortByClicks = $countdowns_array['sortByClicks'];

        include($this->templatesDir . 'countdown-links.php');
    }


    /**
     * Ajax callack to add new record.
     * @return void
     */
    public function handlePostCountdownAdd() {
        global $wpdb;
        // $id = $_POST['id'];
        
        $tableName = $wpdb->prefix . 'refer_countdown_links';

        $valid = true;
        if(!isset($_POST['_wpnonce'])) {
            $valid = false;
        } elseif(!wp_verify_nonce($_POST['_wpnonce'], 'add_countdown')) {
            $valid = false;
        }

        if(!$valid) {
            wp_die('Invalid nonce specified', 'Error', [
                'response'  => 403,
            ]);
        }

        $displayTimer           = array_key_exists('display_timer', $_POST) ? $_POST['display_timer'] : "";
        $clockName              = array_key_exists('clock_name', $_POST) ? $_POST['clock_name'] : "";
        $productName            = array_key_exists('product_name', $_POST) ? $_POST['product_name'] : "";
        $product_description    = array_key_exists('product_description', $_POST) ? $_POST['product_description'] : "";
        $mediaId                = array_key_exists('product_image_id', $_POST) ? $_POST['product_image_id'] : "";
        $callToAction           = array_key_exists('call_to_action', $_POST) ? $_POST['call_to_action'] : "";
        $buttonLabel            = array_key_exists('button_label', $_POST) ? $_POST['button_label'] : "";
        $context_button_label   = array_key_exists('context_button_label', $_POST) ? $_POST['context_button_label'] : "";
        $full_price_label       = array_key_exists('full_price_label', $_POST) ? $_POST['full_price_label'] : "";
        $url                    = array_key_exists('purchase_link', $_POST) ? $_POST['purchase_link'] : "";
        $countdown_ends         = array_key_exists('countdown_ends', $_POST) ? $_POST['countdown_ends'] : null;
        $startDate              = array_key_exists('start_date', $_POST) ? $_POST['start_date'] : date('Y-m-d H:i:s');
        $startTime              = array_key_exists('start_time', $_POST) ? $_POST['start_time'] : null;
        $endDate                = array_key_exists('end_date', $_POST) ? $_POST['end_date'] : null;
        $endTime                = array_key_exists('end_time', $_POST) ? $_POST['end_time'] : null;
        $relative_time          = array_key_exists('relative_time', $_POST) ? $_POST['relative_time'] : null;
        $location_header_home   = array_key_exists('location_header_home', $_POST) ? $_POST['location_header_home'] : null;
        $location_header_pages  = array_key_exists('location_header_pages', $_POST) ? $_POST['location_header_pages'] : null;
        $location_post_top      = array_key_exists('location_post_top', $_POST) ? $_POST['location_post_top'] : null;
        $pages                  = array_key_exists('pages', $_POST) ? $_POST['pages'] : null;
        $posts                  = array_key_exists('posts', $_POST) ? $_POST['posts'] : null;
        $cats                   = array_key_exists('cats', $_POST) ? $_POST['cats'] : null;

        $header_background_color       = array_key_exists('header_background_color', $_POST) ? $_POST['header_background_color'] : null;
        $header_text_color             = array_key_exists('header_text_color', $_POST) ? $_POST['header_text_color'] : null;
        $header_button_color           = array_key_exists('header_button_color', $_POST) ? $_POST['header_button_color'] : null;
        $header_button_text_color      = array_key_exists('header_button_text_color', $_POST) ? $_POST['header_button_text_color'] : null;

        $background_color       = array_key_exists('background_color', $_POST) ? $_POST['background_color'] : null;
        $text_color             = array_key_exists('text_color', $_POST) ? $_POST['text_color'] : null;
        $button_color           = array_key_exists('button_color', $_POST) ? $_POST['button_color'] : null;
        $button_text_color      = array_key_exists('button_text_color', $_POST) ? $_POST['button_text_color'] : null;


        $errors = [];

        if ($product_description) {
            if( strlen($product_description) >= 250 ) {
                $errors['product_description'] = 'Description is limited to 250 characters. Yours is ' . strlen($product_description);
            }
        }

        if($clockName == '') {
            $errors['clock_name'] = 'Clock Name is required';
        } else {
            // validate against duplicates
            $existing = $wpdb->get_row($wpdb->prepare(
                "SELECT * FROM $tableName WHERE clock_name = %s LIMIT 1", [$clockName]));
            // error_log( "existing" . var_dump($existing));
            if(!empty($existing)) {
                $errors['clock_name'] = 'Clock Name already exists.';
            }
        }

        if($productName == '') {
            $errors['product_name'] = 'Product Name is required';
        } else {
            // validate against duplicates
            $existing = $wpdb->get_row($wpdb->prepare(
                "SELECT * FROM $tableName WHERE product_name = %s LIMIT 1", [$productName]));
            if($existing) {
                $errors['product_name'] = 'Product Name already exists.';
            }
        }

        if ($callToAction == '') {
            $errors['call_to_action'] = "Call to Action is required";
        }
        if ($buttonLabel == "" ) {
            $errors['button_label'] = "Button Label is required";
        }
        if ($endDate == "") {
            $errors['end_date'] = 'End Date is required';
        }

        if(!$url) {
            $errors['purchase_link'] = 'Product URL';
        } else

        if(!filter_var($url, FILTER_VALIDATE_URL) && $url != "") {
            $errors['purchase_link'] = 'Product URL is invalid.';
        } else if( $url != "" ){
            $existing = $wpdb->get_row($wpdb->prepare(
                "SELECT * FROM $tableName WHERE purchase_link = %s LIMIT 1",
                [$url]
            ));
            if($existing) {
                $errors['purchase_link'] = 'Product URL already exists.';
            }
        }

        if( is_string($cats) ) {
            if( strlen($cats) >= 100 ){
                $errors['cats'] = 'Too many categories! Select fewer categories';
            }
        }

        if( is_string($posts) ) {
            if( strlen($posts) >= 100 ){
                $errors['posts'] = 'Too many posts! Select fewer posts';
            }
        }

        if(!empty($errors)) {
            echo json_encode(['success' => false, 'errors' => $errors]);
            wp_die();
        }

        // code to add table column for new fields on the fly!
        // TY https://stackoverflow.com/questions/31291962/add-multiple-columns-in-custom-wordpress-table-if-it-does-not-exist
        $new_fields = Array(
            'display_timer' => '', 
            'clock_name' => '', 
			'product_name' => '',
			'product_description' => '',
			'product_image_id' => '',
			'call_to_action' => '',
			'button_label' => '',
            'context_button_label' => '',
            'full_price_label' => '',
			'purchase_link' => '',
			'countdown_ends' => '', 
			'end_date' => '',
            'relative_time' => '', 
            'start_date' => '', 
            'location_header_home' => '', 
            'location_header_pages' => '', 
            'location_post_top' => 'checked', 
            'pages' => '', 
            'posts' => '', 
            'cats' => '', 
            'background_color' => '',
            'text_color' => '',
            'button_color' => '',
            'button_text_color' => '',
			'header_background_color' => '',
			'header_text_color' => '',
            'header_button_color' => '',
            'header_button_text_color' => '',
            'header_context_button_label' => '',
            'header_full_price_label' => '',
        );

        foreach ($new_fields as $field_name => $default) {

            $test_row = $wpdb->get_results(  "SELECT COLUMN_NAME FROM INFORMATION_SCHEMA.COLUMNS WHERE TABLE_NAME = '" . $tableName . "' AND COLUMN_NAME = '".$field_name."'"  );

            if(empty($test_row)){
                $size = ($field_name == 'product_description')? '250' : '100';
                $wpdb->query("ALTER TABLE " . $tableName . " ADD ".$field_name." varchar(".$size.") NOT NULL DEFAULT '".$default."'");
            }
        }

        //$tz = new \DateTimeZone(date_default_timezone_get());
        $startDateValue = $startDate;
        if ($startDate && !$startTime) {
            try {
                $startDate = new \DateTime($startDate);
//                $startDate->setTimezone($tz);
                $startDateValue = $startDate->format("Y-m-d H:i:s");
            } catch (\Exception $e) {
                $startDate = null;
            }
        }
        if ($startDate && $startTime) {
            try {
                $startDate = new \DateTime($startDate . ' ' . $startTime);
                //$startDate->setTimezone($tz);
                $startDateValue = $startDate->format("Y-m-d H:i:s");
            } catch (\Exception $e) {
                $startDate = null;
            }
        }

        //$tz = new \DateTimeZone(date_default_timezone_get());
		if($countdown_ends == 'relative') {
			$endDate = date('Y-m-d', strtotime($startDateValue.' +60 days'));
		}
        $endDateValue = $endDate;
        if ($endDate && !$endTime) {
            try {
                $endDate = new \DateTime($endDate);
//                $endDate->setTimezone($tz);
                $endDateValue = $endDate->format("Y-m-d H:i:s");
            } catch (\Exception $e) {
                $endDate = null;
            }
        }
        if ($endDate && $endTime) {
            try {
                $endDate = new \DateTime($endDate . ' ' . $endTime);
                //$endDate->setTimezone($tz);
                $endDateValue = $endDate->format("Y-m-d H:i:s");
            } catch (\Exception $e) {
                $endDate = null;
            }
        }

        $wpdb->insert($tableName, [
            'display_timer' => $displayTimer,
            'clock_name' => $clockName,
            'product_name' => $productName,
            'product_description' => $product_description,
            'product_image_id' => $mediaId,
            'call_to_action' => $callToAction,
            'button_label' => $buttonLabel,
            'context_button_label' => $context_button_label,
            'full_price_label' => $full_price_label,
            'purchase_link' => $url,
            'countdown_ends' => $countdown_ends,
            'end_date' => $endDateValue,
            'relative_time' => $relative_time,
            'start_date' => $startDateValue,
            'location_header_home' => $location_header_home,
            'location_header_pages' => $location_header_pages,
            'location_post_top' => $location_post_top,
            'pages' => $pages,
            'posts' => $posts,
            'cats' => $cats,
            'background_color' => $background_color,
            'text_color' => $text_color,
            'button_color' => $button_color,
            'button_text_color' => $button_text_color,
            'header_background_color' => $header_background_color,
            'header_text_color' => $header_text_color,
            'header_button_color' => $header_button_color,
            'header_button_text_color' => $header_button_text_color,
        ]);

        // prepare for render edit template
        $startDate = new \DateTime($startDateValue);
        $startDateValue = $startDate->format("Y-m-d");
        $startTimeValue = $startDate->format('H:i');

        $endDate = new \DateTime($endDateValue);
        $endDateValue = $endDate->format("Y-m-d");
        $endTimeValue = $endDate->format('H:i');

        $linkId = $wpdb->insert_id;
        $countdown = $wpdb->get_row($wpdb->prepare("SELECT * FROM $tableName WHERE id = %s LIMIT 1", [$linkId]));

        ob_start();
        include($this->templatesDir  . '_partials/countdown-item-row-edit.php');
        $template = ob_get_clean();

        echo json_encode(['success' => true, 'countdown' => $countdown, 'template' => $template ]);
        wp_die();
    }

    /**
     * Ajax callback to render an edit template.
     * @return void
     * @throws \Exception DateTime Exception.
     */
    public function handleRenderCountdownEditTemplate()
    {
        global $wpdb;

        $valid = true;
        if(!isset($_GET['_wpnonce'])) {
            $valid = false;
        } elseif(!wp_verify_nonce($_GET['_wpnonce'], 'view_edit_countdown_' . $_GET['id'])) {
            $valid = false;
        }

        if(!$valid) {
            wp_die('Invalid nonce specified', 'Error', [
                'response'  => 403,
            ]);
        }

        $tableName = $wpdb->prefix . 'refer_countdown_links';

        $countdown = $wpdb->get_row($wpdb->prepare(
            "SELECT * FROM $tableName WHERE id = %s LIMIT 1",
            [$_GET['id']]
        ));

        if(!$countdown) {
            echo json_encode(['success' => false, 'errors' => ['Invalid countdown item.']]);
            wp_die();
        }

        $startDateValue = null;
        $startTimeValue = null;
        if ($countdown->start_date) {

            $startDate = new \DateTime($countdown->start_date);
            $startDateValue = $startDate->format('Y-m-d');
            $startTimeValue = $startDate->format('H:i');
        }

        $endDateValue = null;
        $endTimeValue = null;
        if ($countdown->end_date) {

            $endDate = new \DateTime($countdown->end_date);
            $endDateValue = $endDate->format('Y-m-d');
            $endTimeValue = $endDate->format('H:i');
        }

        ob_start();
        include($this->templatesDir  . '_partials/countdown-item-row-edit.php');
        $template = ob_get_clean();

        echo json_encode(['success' => true, 'countdown' => $countdown, 'template' => $template]);
        wp_die();
    }

    /**
     * Ajax callback to render an add template.
     * @return void
     */
    public function handleRenderCountdownAddTemplate()
    {

        ob_start();
        include($this->templatesDir  . '_partials/countdown-item-row-add.php');
        $template = ob_get_clean();

        echo json_encode(['success' => true, 'template' => $template]);
        wp_die();
    }

    /**
     * Ajax callback to render clocks list.
     * @return void
     */
    public function handle_render_clocks_list()
    {
        global $wpdb;
        $tableName = $wpdb->prefix . 'refer_countdown_links';
        $total = $wpdb->get_var("SELECT count(id) FROM $tableName");
        $maxNumberPages = $total;
        $baseSql = " FROM $tableName WHERE 1=1 ";
        $paged = isset($_GET['paged']) ? $_GET['paged'] : 1;
        $postsPerPage = 20;
        $offset = ($paged - 1)*$postsPerPage;

        //lay out defaults
        $orderByClause = "product_name ASC";
        $sortByName = "DESC";
        $sortByUrl = "ASC";
        $sortByEndDate = "ASC";
        $sortAuthorReportByStatus = "DESC";
        $sortByViews = "DESC";
        $sortByStatus = "DESC";
        $sortByClicks = "DESC";

        if(isset($_GET['sortCountdownByName']) || isset($_GET['sortCountdownByUrl']) || isset($_GET['sortCountdownByEndDate']) || isset($_GET['sortCountdownByViews'])) {

            if(isset($_GET['sortCountdownByName'])) {
                $orderByClause = "product_name {$_GET['sortCountdownByName']}";
                if($_GET['sortCountdownByName'] == 'ASC') {
                    $sortByName = "DESC";
                }
                else {
                    $sortByName = "ASC";
                }
            }
            elseif(isset($_GET['sortCountdownByUrl'])) {
                $orderByClause = "purchase_link {$_GET['sortCountdownByUrl']}";
                if($_GET['sortCountdownByUrl'] == 'ASC') {
                    $sortByUrl = "DESC";
                }
                else {
                    $sortByUrl = "ASC";
                }
            }
            elseif(isset($_GET['sortCountdownByEndDate'])) {
                $orderByClause = "end_date {$_GET['sortCountdownByEndDate']}";
                if($_GET['sortCountdownByEndDate'] == 'ASC') {
                    $sortByEndDate = "DESC";
                }
                else {
                    $sortByEndDate = "ASC";
                }
            }
            elseif(isset($_GET['sortCountdownByViews'])) {
                $orderByClause = "views {$_GET['sortCountdownByViews']}";
                if($_GET['sortCountdownByViews'] == 'ASC') {
                    $sortByViews = "DESC";
                }
                else {
                    $sortByViews = "ASC";
                }
            }
            elseif(isset($_GET['sortCountdownByStatus'])) {
                $orderByClause = "Status {$_GET['sortCountdownByStatus']}";
                if($_GET['sortCountdownByStatus'] == 'ASC') {
                    $sortByStatus = "DESC";
                }
                else {
                    $sortByStatus = "ASC";
                }
            }
            elseif(isset($_GET['sortCountdownByClicks'])) {
                $orderByClause = "Clicks {$_GET['sortCountdownByClicks']}";
                if($_GET['sortCountdownByClicks'] == 'ASC') {
                    $sortByClicks = "DESC";
                }
                else {
                    $sortByClicks = "ASC";
                }
            }
        }

        $links = [];
        $total = 0;
        if(isset($_GET['search']) && $_GET['search'] != '') {
            $baseSql .= " AND ( product_name LIKE %s OR purchase_link LIKE %s OR button_label LIKE %s  OR call_to_action LIKE %s)";
            $sqlParms = array_merge(['%' . $_GET['search'] . '%', '%' . $_GET['search'] . '%', '%' . $_GET['search'] . '%', '%' . $_GET['search'] . '%']);
            $links = $wpdb->get_results($wpdb->prepare("SELECT * " . $baseSql  . " ORDER BY " . $orderByClause . " LIMIT $offset, $postsPerPage", $sqlParms));
            $total = $wpdb->get_var($wpdb->prepare("SELECT count(id) " . $baseSql, $sqlParms));
        }
        else {
            // error_log("sqlParms = " . print_r($sqlParms, true));
            $links = $wpdb->get_results("SELECT * " . $baseSql  . " ORDER BY " . $orderByClause . " LIMIT $offset, $postsPerPage");
            $total = $wpdb->get_var("SELECT count(id) " . $baseSql);
        }

        ob_start();
        include($this->templatesDir  . '_partials/countdown-item-list.php');
        $template = ob_get_clean();

        echo json_encode(['success' => true, 'template' => $template, 'GET' => $_GET ]);
        wp_die();
    }

    /**
     * Ajax call to save an existing post.
     * @return void
     */
    public function handlePostCountdownEdit()
    {
        global $wpdb;

        $valid = true;
        if(!isset($_POST['_wpnonce'])) {
            $valid = false;
        } elseif(!wp_verify_nonce($_POST['_wpnonce'], 'edit_countdown_' . $_POST['id'])) {
            $valid = false;
        }

        if(!$valid) {
            wp_die('Invalid nonce specified', 'Error', [
                'response'  => 403,
            ]);
        }

        $tableName = $wpdb->prefix . 'refer_countdown_links';

        $countdown = $wpdb->get_row($wpdb->prepare(
            "SELECT * FROM $tableName WHERE id = %s LIMIT 1",
            [$_POST['id']]
        ));

        if(!$countdown) {
            echo json_encode(['success' => false, 'errors' => ['Invalid countdown item.']]);
            wp_die();
        }

        // error_log( print_r($_POST, true) );

        $id = $_POST['id'];
        $displayTimer           = array_key_exists('display_timer', $_POST) ? $_POST['display_timer'] : null;
        $clockName              = array_key_exists('clock_name', $_POST) ? $_POST['clock_name'] : null;
        $productName            = array_key_exists('product_name', $_POST) ? $_POST['product_name'] : null;
        $product_description    = array_key_exists('product_description', $_POST) ? $_POST['product_description'] : null;
        $mediaId                = array_key_exists('product_image_id', $_POST) ? $_POST['product_image_id'] : 55;
        $callToAction           = array_key_exists('call_to_action', $_POST) ? $_POST['call_to_action'] : null;
        $buttonLabel            = array_key_exists('button_label', $_POST) ? $_POST['button_label'] : null;
        $context_button_label   = array_key_exists('context_button_label', $_POST) ? $_POST['context_button_label'] : null;
        $full_price_label       = array_key_exists('full_price_label', $_POST) ? $_POST['full_price_label'] : null;
        $countdown_ends         = array_key_exists('countdown_ends', $_POST) ? $_POST['countdown_ends'] : null;
        $url                    = array_key_exists('purchase_link', $_POST) ? $_POST['purchase_link'] : null;
        $startDate              = array_key_exists('start_date', $_POST) ? $_POST['start_date'] : null;
        $startTime              = array_key_exists('start_time', $_POST) ? $_POST['start_time'] : null;                
        $endDate                = array_key_exists('end_date', $_POST) ? $_POST['end_date'] : null;
        $endTime                = array_key_exists('end_time', $_POST) ? $_POST['end_time'] : null;
        $relative_time          = array_key_exists('relative_time', $_POST) ? $_POST['relative_time'] : null;
        $location_header_home   = array_key_exists('location_header_home', $_POST) ? $_POST['location_header_home'] : null;        
        $location_header_pages  = array_key_exists('location_header_pages', $_POST) ? $_POST['location_header_pages'] : null;        
        $location_post_top      = array_key_exists('location_post_top', $_POST) ? $_POST['location_post_top'] : null;        
        $pages                  = array_key_exists('pages', $_POST) ? $_POST['pages'] : null;
        $posts                  = array_key_exists('posts', $_POST) ? $_POST['posts'] : null;
        $cats                   = array_key_exists('cats', $_POST) ? $_POST['cats'] : null;

        $header_background_color   = array_key_exists('header_background_color', $_POST) ? $_POST['header_background_color'] : null;
        $header_text_color         = array_key_exists('header_text_color', $_POST) ? $_POST['header_text_color'] : null;
        $header_button_color       = array_key_exists('header_button_color', $_POST) ? $_POST['header_button_color'] : null;
        $header_button_text_color  = array_key_exists('header_button_text_color', $_POST) ? $_POST['header_button_text_color'] : null;

        $background_color   = array_key_exists('background_color', $_POST) ? $_POST['background_color'] : null;
        $text_color         = array_key_exists('text_color', $_POST) ? $_POST['text_color'] : null;
        $button_color       = array_key_exists('button_color', $_POST) ? $_POST['button_color'] : null;
        $button_text_color  = array_key_exists('button_text_color', $_POST) ? $_POST['button_text_color'] : null;


        // code to add table column for new fields on the fly!
        // TY https://stackoverflow.com/questions/31291962/add-multiple-columns-in-custom-wordpress-table-if-it-does-not-exist
        $new_fields = Array(
            'display_timer' => '', 
            'clock_name' => '', 
            'countdown_ends' => '', 
            'product_description' => '', 
            'relative_time' => '', 
            'location_header_home' => '', 
            'location_header_pages' => '', 
            'location_post_top' => 'checked', 
            'pages' => '', 
            'posts' => '', 
            'cats' => '',
            'start_date' => '',
            'background_color' => '',
            'text_color' => '',
            'button_color' => '',
            'button_text_color' => '',
            'header_background_color' => '',
            'header_text_color' => '',
            'header_button_color' => '',
            'header_button_text_color' => '',            
            'context_button_label' => '',
            'full_price_label' => '',
        );

        foreach ($new_fields as $field_name => $default) {

            $test_row = $wpdb->get_results(  "SELECT COLUMN_NAME FROM INFORMATION_SCHEMA.COLUMNS WHERE TABLE_NAME = '" . $tableName . "' AND COLUMN_NAME = '".$field_name."'"  );

            if(empty($test_row)){
                $size = ($field_name == 'product_description')? '250' : '100';
                $wpdb->query("ALTER TABLE " . $tableName . " ADD ".$field_name." varchar(".$size.") NOT NULL DEFAULT '".$default."'");
            }
        }

        // error handling....
        $errors = [];

        if ($product_description) {
            if( strlen($product_description) >= 250 ) {
                $errors['product_description'] = 'Description is limited to 250 characters. Yours is ' . strlen($product_description);
            }
        }

        if(!$clockName) {
            $errors['clock_name'] = 'Clock Name is required';
        } else {
            // validate against duplicates
            $existing = $wpdb->get_row($wpdb->prepare(
                "SELECT * FROM $tableName WHERE clock_name = %s AND id != %s LIMIT 1", [$clockName, $id]));
            if($existing) {
                $errors['clock_name'] = 'Clock Name already exists.';
            }
        }

        if(!$productName) {
            $errors['product_name'] = 'Product Name is required';
        } else {
            $existing = $wpdb->get_row($wpdb->prepare(
                "SELECT * FROM $tableName WHERE product_name = %s AND id != %s LIMIT 1", [$productName, $id]));
            if($existing) {
                $errors['product_name'] = 'Product Name already exists.';
            }
        }

        if (!$callToAction) {
            $errors['call_to_action'] = "Call to Action is required";
        }
        if (!$buttonLabel ) {
            $errors['button_label'] = "Button Label is required";
        }
        if (!$endDate) {
            $errors['end_date'] = 'End Date is required';
        }

        if(!$url) {
            $errors['purchase_link'] = 'Product URL is required.';
        } else

        if(!filter_var($url, FILTER_VALIDATE_URL) && $url != "" ) {
            $errors['purchase_link'] = 'Product URL is invalid.';
        } else if( $url != "" ){
            $existing = $wpdb->get_row($wpdb->prepare(
                "SELECT * FROM $tableName WHERE purchase_link = %s AND id != %s LIMIT 1",
                [$url, $id]
            ));
            if($existing) {
                $errors['purchase_link'] = 'Product URL already exists.';
            }
        }

        if(!empty($errors)) {
            echo json_encode(['success' => false, 'errors' => $errors]);
            wp_die();
        }

        // parse date and time to provide a single value for Mysql
        $startDateValue = $startDate;
        if ($startDate && !$startTime) {
            try {
                $startDate = new \DateTime($startDate);
                $startDateValue = $startDate->format("Y-m-d H:i:s");
            } catch (\Exception $e) {
                $startDate = null;
            }
        }
        if ($startDate && $startTime) {
            try {
                $startDate = new \DateTime($startDate . ' ' . $startTime);
                $startDateValue = $startDate->format("Y-m-d H:i:s");
            } catch (\Exception $e) {
                $startDate = null;
            }
        }

        // parse date and time to provide a single value for Mysql
        $endDateValue = $endDate;
        if ($endDate && !$endTime) {
            try {
                $endDate = new \DateTime($endDate);
                $endDateValue = $endDate->format("Y-m-d H:i:s");
            } catch (\Exception $e) {
                $endDate = null;
            }
        }
        if ($endDate && $endTime) {
            try {
                $endDate = new \DateTime($endDate . ' ' . $endTime);
                $endDateValue = $endDate->format("Y-m-d H:i:s");
            } catch (\Exception $e) {
                $endDate = null;
            }
        }

        $wpdb->update($tableName, [
            'display_timer' => $displayTimer,
            'clock_name' => $clockName,   
            'product_name' => $productName,
            'product_description' => $product_description,
            'purchase_link' => $url,
            'call_to_action' => $callToAction,
            'button_label' => $buttonLabel,
            'context_button_label' => $context_button_label,
            'full_price_label' => $full_price_label,
            'countdown_ends' => $countdown_ends,
            'end_date' => $endDateValue,
            'start_date' => $startDateValue,
            'relative_time' => $relative_time,
            'product_image_id' => $mediaId,
            'location_header_home' => $location_header_home,
            'location_header_pages' => $location_header_pages,
            'location_post_top' => $location_post_top,
            'pages' => $pages,
            'posts' => $posts,
            'cats' => $cats,
            'background_color' => $background_color,
            'text_color' => $text_color,
            'button_color' => $button_color,
            'button_text_color' => $button_text_color,
            'header_background_color' => $header_background_color,
            'header_text_color' => $header_text_color,
            'header_button_color' => $header_button_color,
            'header_button_text_color' => $header_button_text_color,
        ], [
            'id' => $id
        ]);

        $countdown = $wpdb->get_row($wpdb->prepare(
            "SELECT * FROM $tableName WHERE id = %s LIMIT 1",
            [$_POST['id']]
        ));

        //convert start and end dates and times back to separate values for tempate render
        // prepare for render edit template
        $startDate = new \DateTime($startDateValue);
        $startDateValue = $startDate->format("Y-m-d");
        $startTimeValue = $startDate->format('H:i');

        $endDate = new \DateTime($endDateValue);
        $endDateValue = $endDate->format("Y-m-d");
        $endTimeValue = $endDate->format('H:i');

        // Foopbar!//
        
        ob_start();
        include($this->templatesDir  . '_partials/countdown-item-row-edit.php');
        $template = ob_get_clean();

        echo json_encode([
            'success' => true, 
            'countdown' => $countdown, 
            'template' => $template,
            // '$_POST' => $_POST,
        ]);
        wp_die();
    }

    /**
     * Ajax callback to render delete confirmation modal.
     * @return void
     */
    public function handleRenderCountdownPreviewModalTemplate()
    {
        global $wpdb;

        $valid = true;
        if(!isset($_GET['id'])) {
            $valid = false;
        }

        if(!$valid) {
            wp_die('Invalid nonce specified', 'Error', [
                'response'  => 403,
            ]);
        }

        $tableName = $wpdb->prefix . 'refer_countdown_links';

        $item = $wpdb->get_row($wpdb->prepare("SELECT * FROM $tableName WHERE id = %s LIMIT 1", [$_GET['id']]));

        if(!$item) {
            echo json_encode(['success' => false, 'errors' => ['Invalid item.']]);
            wp_die();
        }

        foreach ($_GET as $key => $value) {
            if( property_exists($item, $key) ){
                $item->{$key} = $value;
            }
        }

        $params = Array(
            'item' => $item,
            // 'id' => $_GET['id'],
        );

        // var_dump($item->end_date);

        ob_start(); ?>

            <div class="modal-dialog modal-dialog-centered" role="document">
				<div class="modal-content">
					<div class="modal-header">
						<h5 class="modal-title">Preview</h5>
						<button type="button" class="close" data-dismiss="modal" aria-label="Close">
							<span aria-hidden="true">&times;</span>
						</button>
					</div>
					<div class="modal-body">
						<div class="field">
							<div class="notice notice-success">
								<span><sup>*</sup>Changes will not be final until you click "Save"</span>
							</div>

							<label class="sub-label">Header Banner</label>
							<div class="header-banner-preview">
								<?php 
									$params['context'] = 'header-banner-preview';
									$GLOBALS['income_school']['refer_affiliates']->get_template_part_with_params('sale-countdown-template.php', $params ); 
								 ?>
							</div>
						</div>

						<div class="field">
							<label class="sub-label">In-Content Banner</label>
							<div class="in-content-banner-preview">
								<?php 
									$params['context'] = 'in-content-banner-preview';
									$GLOBALS['income_school']['refer_affiliates']->get_template_part_with_params('sale-countdown-template.php', $params ); 
								 ?>
							</div>
						</div>
						<!--<div class="modal-footer">

							<div class="buttons">
								<a href="#" class="save hover-effect refer-button">Save</a>
								<a href="#" class="cancel hover-effect gray refer-button" data-dismiss="modal">Cancel</a>
							</div>

						</div>-->
					</div>
				</div>
			</div>

        <?php $template = ob_get_clean();
		
        echo json_encode(['success' => true, 'item' => $item, 'template' => $template]);
        wp_die();
    }

    /**
     * Ajax callback to render delete confirmation modal.
     * @return void
     */
    public function handleRenderCountdownDeleteModalTemplate()
    {
        global $wpdb;

        $tableName = $wpdb->prefix . 'refer_countdown_links';

        $countdown = $wpdb->get_row($wpdb->prepare("SELECT * FROM $tableName WHERE id = %s LIMIT 1", [$_GET['id']]));

        if(!$countdown) {
            echo json_encode(['success' => false, 'errors' => ['Invalid item.']]);
            wp_die();
        }

        ob_start();
        include($this->templatesDir  . '_partials/countdown-item-delete-modal.php');
        $template = ob_get_clean();

        echo json_encode(['success' => true, 'countdown' => $countdown, 'template' => $template]);
        wp_die();
    }

    /**
     * Ajax callback to handle delete upon confirmation
     * @return void
     */
    public function handlePostCountdownDelete()
    {
        global $wpdb;

        $valid = true;
        if(!isset($_POST['_wpnonce'])) {
            $valid = false;
        } elseif(!wp_verify_nonce($_POST['_wpnonce'], 'delete_countdown_' . $_POST['id'])) {
            $valid = false;
        }

        if(!$valid) {
            wp_die('Invalid nonce specified', 'Error', [
                'response'  => 403,
            ]);
        }

        $tableName = $wpdb->prefix . 'refer_countdown_links';

        $countdown = $wpdb->get_row($wpdb->prepare(
            "SELECT * FROM $tableName WHERE id = %s LIMIT 1",
            [$_POST['id']]
        ));

        if(!$countdown) {
            echo json_encode(['success' => false, 'errors' => ['Invalid countdown item.']]);
            wp_die();
        }

        $deleted = $wpdb->delete($tableName, [
            'id' => $countdown->id
        ]);

        //load new clock template just in case
        ob_start();
        include($this->templatesDir  . '_partials/countdown-item-row-add.php');
        $template = ob_get_clean();

        echo json_encode(['success' => true, 'template' => $template]);
        wp_die();
    }

    /**
     * [handlePostVerifyAffiliateShortlinkConverter description]
     */
    public function handlePostVerifyAffiliateShortlinkConverter()
    {
        $valid = true;
        if(!isset($_POST['_wpnonce'])) {
            $valid = false;
        } elseif(!wp_verify_nonce($_POST['_wpnonce'], 'refer_amazon_shortlink_converter_verify')) {
            $valid = false;
        }

        if(!$valid) {
            wp_die('Invalid nonce specified', 'Error', array(
                'response'  => 403,
            ));
        }

        $converterTool = AffiliateLinkRefer()->classes['affiliate_shortlink_converter'];

        // $verfied = $converterTool->convert('https://amzn.to/39iN96W');
        $verfied = $converterTool->convert('https://amzn.to/2Q5texS');

        $verifyFailed = !$verfied;

        if(!$verifyFailed) {
            $converterTool->setVerfiedWorking(true);
            $converterTool->resetFailedCount();
        }

        $shortlinkToolVerified = $converterTool->getVerfiedWorking();
        $shortlinkToolFailedCount = $converterTool->getFailedCount();

        $queueCount = 0;

        ob_start();
        include($this->templatesDir  . '_partials/amazon-shortlink-converter-verify.php');
        $template = ob_get_clean();

        echo json_encode(['success' => true, 'template' => $template]);
        wp_die();
    }

    public function renderVerifyConverterAlert()
    {
        if(isset($_GET['page']) && $_GET['page'] == 'affiliate-link-refer-api-settings') {
            return;
        }

        $converterTool = AffiliateLinkRefer()->classes['affiliate_shortlink_converter'];

        $amazonSettingsPage = admin_url('admin.php?page=affiliate-link-refer-api-settings');

        if($converterTool->getVerfiedWorking() && $converterTool->getFailedCount() >= 5) {
            echo '<div class="notice notice-warning is-dismissible">
                <p>The Amazon shortlink converter tool has been previous been verified but started failing and has been disabled. Your hosting configuration might have changed. <a href="' . $amazonSettingsPage . '">If you think this may be an error or you have updated your hosting configuration please click to re-verify.</a></p>
            </div>';
        } elseif(!$converterTool->getVerfiedWorking()) {
            echo '<div class="notice notice-warning is-dismissible">
                 <p>The REFER Amazon shortlink converter tool needs to be tested as working for your site configuration. <a href="' . $amazonSettingsPage . '">Click here to test and verify your hosting.</a></p>
            </div>';
        }
    }
    
    /*public function renderIncomeSchoolNotices()
    {

        include($this->templatesDir  . 'notifications.php');

        echo generate_is_admin_notice();
    }*/
    
}
