<?php

namespace IncomeSchool\AffiliateLinkRefer;

class AffiliateLinkLogger
{

    private $affiliateId = false;

    /**
     * Construct
     */
    public function __construct()
    {
        $this->affiliateId = get_option('refer_amazon_affiliate_id', false);

        add_action('wp_ajax_affiliate_link_log_click', [$this, 'logClick'], 99);
        add_action('wp_ajax_nopriv_affiliate_link_log_click', [$this, 'logClick'], 99);
        add_action('wp_ajax_amazon_link_log_view', [$this, 'logView'], 99);
        add_action('wp_ajax_nopriv_amazon_link_log_view', [$this, 'logView'], 99);

        //add_action('loop_start', [$this, 'debug_post_log']);
    }

    public function debug_post_log(){

        global $post, $wpdb;

        if( $post != NULL){

            $trackingTableName = $wpdb->prefix . 'refer_link_daily_trackings';

            $today = date('Y-m-d');

            $existingDaily = $wpdb->get_row("
                SELECT *
                FROM $trackingTableName
                WHERE post_id = $post->ID
                AND date = '$today'
            ");

           var_dump($existingDaily);
        }
    }

    /**
     * [log description]
     * @return [type] [description]
     */
    public function logView()
    {
        global $post, $wpdb;

        $valid = true;
        if(!isset($_POST['nonce'])) {
            $valid = false;
        } elseif(!wp_verify_nonce($_POST['nonce'], 'update_refer_link_tracking')) {
            $valid = false;
        }

        if(!$valid) {
            error_log("Invalid nonce specified");
            wp_die('Invalid nonce specified', 'Error', [
                'response'  => 403,
            ]);
        }

        if(!$this->affiliateId) {
            error_log("Affiliate id not set");
            wp_die('Affiliate id not set.', 'Error');
        }

        $postId = isset($_POST['postId']) ? $_POST['postId'] : null;
        $urls = isset($_POST['urls']) && is_array($_POST['urls']) ? $_POST['urls'] : false;

        if(!$urls) {
            error_log("Invalid url specified");
            wp_die('Invalid url specified', 'Error');
        }

        $count = 0;

        foreach($urls as $url) {

            $affiliate = $this->getAffiliate($url);

            if($affiliate) {
                $trackingTableName = $wpdb->prefix . 'refer_link_daily_trackings';

                $today = date('Y-m-d');

                $existingDaily = $wpdb->get_row("
                    SELECT *
                    FROM $trackingTableName
                    WHERE
                        link_id = $affiliate->id
                    AND
                        post_id = $postId
                    AND
                        date = '$today'
                    LIMIT 1
                ");

                $is_amazon = property_exists($affiliate, 'is_amazon')? $affiliate->is_amazon : NULL;
                //hacky strpos TY https://stackoverflow.com/questions/6284553/using-an-array-as-needles-in-strpos
                $is_amazon = ( str_replace( array('amazon','amzn'), '', $affiliate->url) != $affiliate->url );

                if(!$existingDaily) {
                    $wpdb->insert($trackingTableName, [
                        'post_id' => $postId,
                        'link_id' => $affiliate->id,
                        'is_amazon' => $is_amazon,
                        'views' => 1,
                        'date' => $today
                    ]);
                } else {
                    $wpdb->update($trackingTableName, [
                        'views' => $existingDaily->views+1,
                        'is_amazon' => $is_amazon,
                    ], [
                        'id' => $existingDaily->id
                    ]);
                }

                //increment view log on link itself
                $wpdb->update($wpdb->prefix . 'refer_affiliate_links', [
                    'views' => $affiliate->views + 1
                ], [
                    'id' => $affiliate->id
                ]);

                $count++;
            }
        }

        // error_log("Log Link View: Count " .  $count);


        echo json_encode(['success' => 'true', 'count' => $count]);
        wp_die();
    }

    /**
     * [log description]
     * @return [type] [description]
     */
    public function logClick()
    {
        global $post, $wpdb;

        // error_log("Log Link Click");

        $valid = true;
        if(!isset($_POST['nonce'])) {
            $valid = false;
        } elseif(!wp_verify_nonce($_POST['nonce'], 'update_refer_link_tracking')) {
            $valid = false;
        }

        if(!$valid) {
            wp_die('Invalid nonce specified', 'Error', [
                'response'  => 403,
            ]);
        }

        if(!$this->affiliateId) {
            wp_die('Affiliate id not set.', 'Error');
        }

        $postId = isset($_POST['postId']) ? $_POST['postId'] : null;
        $url = isset($_POST['url']) ? $_POST['url'] : false;

        if(!$url) {
            wp_die('Invalid url specified', 'Error');
        }

        $affiliate = $this->getAffiliate($url);

        if(!$affiliate) {
            wp_die('Invalid url specified', 'Error');
        }

        $trackingTableName = $wpdb->prefix . 'refer_link_daily_trackings';

        $today = date('Y-m-d');

        $existingDaily = $wpdb->get_row("
            SELECT *
            FROM $trackingTableName
            WHERE
                link_id = $affiliate->id
            AND
                post_id = $postId
            AND
                date = '$today'
            LIMIT 1
        ");

        $is_amazon = property_exists($affiliate, 'is_amazon')? $affiliate->is_amazon : NULL;
        //hacky strpos TY https://stackoverflow.com/questions/6284553/using-an-array-as-needles-in-strpos
        $is_amazon = ( str_replace( array('amazon','amzn'), '', $affiliate->url) != $affiliate->url );

        if(!$existingDaily) {
            $wpdb->insert($trackingTableName, [
                'post_id' => $postId,
                'link_id' => $affiliate->id,
                'is_amazon' => $is_amazon,
                // 'type' =>  $affiliate->type,
                'clicks' => 1,
                'date' => $today
            ]);
        } else {
            $wpdb->update($trackingTableName, [
                'clicks' => $existingDaily->clicks+1,
                'is_amazon' => $is_amazon,
                'type' => $affiliate->type,
            ], [
                'id' => $existingDaily->id
            ]);
        }

        //increment click tally on affiliate itself
        $wpdb->update($wpdb->prefix . 'refer_affiliate_links', [
            'clicks' => $affiliate->clicks + 1
        ], [
            'id' => $affiliate->id
        ]);

        echo json_encode(['success' => 'true']);
        wp_die();
    }

    /**
     * [getAffiliate description]
     * @param  [type] $url [description]
     * @return [type]      [description]
     */
    private function getAffiliate($url)
    {
        global $post, $wpdb;
        // first, try with raw url
        $affiliateTableName = $wpdb->prefix . 'refer_affiliate_links';

        $first_try = $wpdb->get_row($wpdb->prepare("SELECT * FROM $affiliateTableName WHERE url = '%s'", $url));
        if( is_object($first_try) )
            return $first_try;

        $urlParts = parse_url($url);

        if( !isset($urlParts['path']) ||  !isset($urlParts['scheme']) ||  !isset($urlParts['host']) ) {
           return false;
        }

        $matchFound = false;
        $path = str_replace('gp/product/', 'dp/', $urlParts['path']);

        $pathMatches1 = array();
        $pathMatches2 = array();

        preg_match("/(?:\/" . 'dp' . "\/|$)([A-Z0-9]{10})/", $urlParts['path'], $pathMatches1);
        if(isset($pathMatches1[0])) {
            $path = $pathMatches1[0];
            $matchFound = true;
        }

        if(!$path) {
           return false;
        }

        $amazonUrl = strtolower($urlParts['scheme'] . '://' . $urlParts['host'] . $path . '?' . http_build_query(array(
            'tag' => $this->affiliateId
        )));

        return $wpdb->get_row($wpdb->prepare("SELECT * FROM $affiliateTableName WHERE url = '%s'", $amazonUrl));
    }
}
