<?php

namespace IncomeSchool\AffiliateLinkRefer;

class AffiliateLinkParser
{

    /**
     * [$affiliateId description]
     * @var boolean
     */
    private $affiliateId = false;

    /**
     * [__construct description]
     */
    public function __construct()
    {
        //Include url normalizer class made by Glen Scott
        include_once AFFILIATE_LINK_REFER_PLUGIN_DIR . '/includes/class-normalizer.php';

        $this->affiliateId = get_option('refer_amazon_affiliate_id', false);

        add_filter('the_content', [$this, 'updateContentLinks'], 99, 1);
        add_filter('widget_display_callback', [$this, 'updateWidgetLinks'], 99, 3);
        add_filter('wp_nav_menu_objects', [$this, 'updateMenuLinks'], 99, 2);
    }

    /**
     * [updateContentLinks description]
     * @param  string $content [inputted content]
     * @return string $content [outputted content]
     */
    public function updateContentLinks($content)
    {
        if(is_admin()) {
            return $content;
        }

        if(!$this->affiliateId) {
            return $content;
        }

        $urlMatches = [];

        preg_match_all('#\bhttps?://[^,\s()<>]+(?:\([\w\d]+\)|([^,[:punct:]\s]|/))#', $content, $urlMatches);

        $urlsToParse = [];
        foreach($urlMatches[0] as $match) {
            $urlsToParse[$match] = $match;
        }

        $urlsToReplace = AffiliateLinkParser::processUrls($urlsToParse);
        // error_log(print_r($urlsToReplace, true));
        foreach($urlsToReplace as $search => $replace) {
            $content = str_replace($search, $replace, $content);
        }

        return $content;
    }

    /**
     * [updateWidgetLinks description]
     * @param  array $instance [widget content array]
     * @param  array $widget [widget content array]
     * @param  array $args [widget content array]
     * @return [type]          [description]
     */
    public function updateWidgetLinks($instance, $widget, $args)
    {
        $text = isset($instance['text']) ? $instance['text'] : false;
        if(!$text) {
            $text = isset($instance['content']) ? $instance['content'] : false;
        }

        if(!$this->affiliateId || !$text) {
            return $instance;
        }
        // error_log("INSTANCE = " . print_r($instance, true));
        $urlMatches = [];

        preg_match_all('#\bhttps?://[^,\s()<>]+(?:\([\w\d]+\)|([^,[:punct:]\s]|/))#', $text, $urlMatches);

        $urlsToParse = [];
        foreach($urlMatches[0] as $match) {
            $urlsToParse[$match] = $match;
        }

        $urlsToReplace = AffiliateLinkParser::processUrls($urlsToParse);

        foreach($urlsToReplace as $search => $replace) {
            if(isset($instance['text'])) {
                $instance['text'] = str_replace($search, $replace, $instance['text']);
            } elseif(isset($instance['content'])) {
                $instance['content'] = str_replace($search, $replace, $instance['content']);
            }
        }

        return $instance;
    }

    /**
     * [updateMenuLinks get all menu links and process with AffiliateLinkParser]
     * @param  array $items [nav menu objects]
     * @return [array]          [description]
     */
    function updateMenuLinks($items, $args)
    {
        if(!$this->affiliateId) {
            return $items;
        }

        $urlsToParse = [];

        foreach($items as $item) {

            $urlMatches = [];

            preg_match_all('#\bhttps?://[^,\s()<>]+(?:\([\w\d]+\)|([^,[:punct:]\s]|/))#', $item->url, $urlMatches);

            foreach($urlMatches[0] as $match) {
                $urlsToParse[$match] = $match;
            }

            $urlsToReplace = AffiliateLinkParser::processUrls($urlsToParse);

            foreach($urlsToReplace as $search => $replace) {
                $item->url = str_replace($search, $replace, $item->url);
            }
        }
        return $items;
    }

    /**
     * [processUrls]  [check urls to find amazon links and store them in the refer_affiliate_links table.]
     * @param  array [array of urls found in content]
     * @return [array] [urls with amazon affiliate id appended]
     */
    private function processUrls(array $urlsToParse)
    {
      // error_log("processUrls()");
        $productPathCodes = array('dp', 'gp');
        $urlsToReplace = [];

        foreach($urlsToParse as $url) {
            $urlParts = parse_url($url);
            if(strpos($urlParts['host'], 'amazon.com') !== false || strpos($urlParts['host'], 'amzn.to') !== false) {
                $params = [];
                $params['tag'] = $this->affiliateId;
                $amazon_url = '';

                $path = $urlParts['path'];
                $productCodeMatchFound = false;

                //Do not process amazon url if no country codes are found in $path
                foreach($productPathCodes as $productPathCode) {
                    if(strpos($path, '/' . $productPathCode . '/') !== false) {
                        $productCodeMatchFound = true;
                        break;
                    }
                }

                //If country code match wasn't found, break out of the main foreach loop
                if(!$productCodeMatchFound) {
                    continue;
                }

                $path = str_replace('gp/product/', 'dp/', $urlParts['path']);

                $matchFound = false;

                $pathMatches1 = array();
                $pathMatches2 = array();

                preg_match("/(?:\/" . 'dp' . "\/|$)([A-Z0-9]{10})/", $path, $pathMatches1);
                if(isset($pathMatches1[0])) {
                    $path = $pathMatches1[0];
                    $matchFound = true;
                }

                if($matchFound) {
                    $amazon_url = $urlParts['scheme'] . '://' . $urlParts['host'] . '/' . $path . '?' . http_build_query($params);

                    //Normalize url
                    $un = new Normalizer($amazon_url);
                    $normalized_url = $un->normalize();

                    //Create md5 hash from url
                    $hash = substr(md5($normalized_url), 0, 8);

                    global $wpdb;

                    $tableName = $wpdb->prefix . 'refer_affiliate_links';

                    //Check if row exists
                    $hashRecord = $wpdb->get_var("SELECT hash FROM $tableName WHERE hash = '" . $hash . "'");
                    // error_log("hashRecord: " . $hashRecord);

                    //Store amazon url in database if a record doesn't exist
                    if($hashRecord == "") {
                        //Insert row
                        $wpdb->insert($tableName, [
                            'hash' => $hash,
                            'is_amazon' => 1,
                            'url' => $normalized_url
                        ]);
                    }

                    //Add normalized url to urlsToReplace
                    $urlsToReplace[$url] = $normalized_url;
                }
            }
        }
        return $urlsToReplace;
    }
}

class AmazonCountryCodesAmazonCountryCodes
{
    public static function get() {
        return ['dp', 'gp', 'fr', 'jp', 'ca', 'de', 'es', 'it', 'in', 'mx', 'br', 'au'];
    }
}
