<?php


namespace IncomeSchool\AffiliateLinkRefer;


class CountdownDisplay {

    /**
     * @var string
     */
    protected $tableName = 'refer_countdown_links';

    public $acabado_post_types = Array(
        'theme_recipe',
        'theme_book_review',
        'theme_product_review',
        'theme_video_review',
    );

    /**
     * The directory for the admin templates
     * @var string
     */
    private $templatesDir = AFFILIATE_LINK_REFER_PLUGIN_DIR  . '/assets/html/';


    public function __construct() {
        // render the product countdown
        add_filter('the_content', [$this, 'showProductCountdownInPost'] );
        add_action('wp_footer', [$this, 'showProductCountdownInPage'] );
    }

    /**
     * @param $content string post content
     *
     * @return void
     */
    public function showProductCountdownInPage() {
        global $post;

        $page = false;

        if( $post ){
            if( $post->post_type == 'page'){
                $page = true;
            }
        }
		
		if( is_category() ) {
			$page = true;
		}
		
        if( is_archive() || is_404() ){
            $page = true;
            if( isset($post) ){
                $post = (object)Array("post_type" => "archive_404_page");
            }
        }

        if( is_home() && get_option( 'page_for_posts' ) != NULL ){
            $page = true;
            $post = (object)Array("post_type" => "page", "ID" => get_option( 'page_for_posts' ) );
        }

        if( $page == true ){

            $cookie = null;
            if (isset($_COOKIE['refer-cd-seen'])) {
                $cookie = $_COOKIE['refer-cd-seen'];
            }

            if ( $cookie == true ) { 
                return;
            }

            $item = $this->getRandomSale($post); //includes logic for specific pages/posts/cats

            if (! $item) {
                return;
            }

            // $this->incrementViewCount($item);

            $params = Array(
                'item' => $item,
            );

            ob_start();
            echo $GLOBALS['income_school']['refer_affiliates']->get_template_part_with_params('sale-countdown-template.php', $params); 
            //echo require_once $this->templatesDir . 'sale-countdown-template.php';
            $template = ob_get_clean();
            $mobile_height = isset($item->display_timer)? ($item->display_timer == "checked")? '200px' : '150px' : '150px';
            echo '<style class="countdown-sticky-top">@media(min-width:768px){body{margin-top:100px!important;}}@media(max-width:768px){body{margin-top:' . $mobile_height . '!important;}}</style>';
            echo $template;
            $GLOBALS['income_school']['refer_affiliates']->classes['countdown_logger']->logView($item->id);

        }
    }

    /**
     * @param $content string post content
     *
     * @return void
     */
    public function showProductCountdownInPost( $content ) {
        global $post;
        
        // $post = get_post();
        if( !isset($post)){
            return $content;
        }

        if( !in_array($post->post_type, $this->acabado_post_types) && $post->post_type != 'post' ){
            return $content;
        }

        $cookie = null;
        if (isset($_COOKIE['refer-cd-seen'])) {
            $cookie = $_COOKIE['refer-cd-seen'];
        }

        if ( $cookie == true ) { //removed $post->post_type != "post" || 
            return $content;
        }

        $item = $this->getRandomSale($post); //includes logic for specific pages/posts/cats
        
        // var_dump($item);

        if (! $item) {
            return $content;
        }

        ob_start();
        require_once $this->templatesDir . 'sale-countdown-template.php';
        $template = ob_get_clean();
        if( in_array($post->post_type, $this->acabado_post_types) || $post->post_type == 'post' ){
            if( property_exists($item, 'location_post_top') ){
                if( $item->location_post_top == 'checked'){
                    $GLOBALS['income_school']['refer_affiliates']->classes['countdown_logger']->logView($item->id);
                    return  $content . $template ;
                }
            }
        }

        return $content;
    }

    //includes logic for specific pages/posts/cats
    public function getRandomSale($post = NULL) {

        $user = wp_get_current_user();
        if ( in_array( 'administrator', (array) $user->roles ) ) {
            //TODO uncomment below after testing complete
            return false;
        }
        
        global $wpdb;
        $today = current_time('mysql');
        $tableName = $wpdb->prefix . $this->tableName; 

        if( $post != NULL){

            if( in_array($post->post_type, $this->acabado_post_types) || $post->post_type == 'post' ){

                //build query to filter for posts and cats
                $query = "SELECT * from {$tableName} WHERE DATEDIFF(end_date, '".$today."') > 0 
                    AND DATEDIFF(start_date, '".$today."') <= 0 
                    AND (
                        (location_post_top = 'checked' AND ( INSTR(cats,'all')>0 OR cats = '' ) AND ( INSTR(posts,'all')>0 OR posts = '' )) 
                        OR (location_post_top = 'checked' AND INSTR(posts,{$post->ID})>0 AND ( INSTR(cats,'all')>0 OR cats = '' )) ";
                
                $post_cats = array();

                $tax_array = get_object_taxonomies($post->post_type);
                if( is_array($tax_array) ) {
                    foreach ($tax_array as $taxonomy) {
                        $terms = wp_get_post_terms( $post->ID, $taxonomy );
                        if( is_array($terms) ){
                            $post_cats = $post_cats + $terms;
                        }
                    }
                }

                foreach ($post_cats as $i => $term) { 
                    if(isset($term->term_id)){
                        $query .= "OR (location_post_top = 'checked' AND INSTR(cats,{$term->term_id})>0 AND ( INSTR(posts,'all')>0 OR posts = '' )) ";
                    }
                }

                $query .= ")";

                $sales = $wpdb->get_results( $query );
//echo 'post';echo $query;

            }else if( $post->post_type == 'page' ){

                $query = "SELECT * from {$tableName} WHERE DATEDIFF(end_date, '".$today."') > 0 
                    AND DATEDIFF(start_date, '".$today."') <= 0  
                    AND (
                        location_header_pages = 'checked' AND INSTR(pages,'all')>0 
                        OR location_header_pages = 'checked' AND INSTR(pages,{$post->ID})>0 ";
                        

                //add query for home page top
                if( $post->ID == get_option( 'page_on_front' ) ){
                    $query .= " OR location_header_home = 'checked' ";
                }

                $query .= ")";

                $sales = $wpdb->get_results( $query );

//echo 'page';echo $query;
            }else if( $post->post_type == 'archive_404_page' ){

                $blog_id = get_option( 'page_for_posts' );

                $query = "SELECT * from {$tableName} WHERE DATEDIFF(end_date, '".$today."') > 0 
                    AND DATEDIFF(start_date, '".$today."') <= 0  
                    AND location_header_pages = 'checked' 
                    ";

				if(is_archive()) {
					$category = get_queried_object();
					$query .= " AND INSTR(cats,'{$category->term_id}')>0 ";
				}else{
					" AND INSTR(pages,'all')>0";
				}

                $sales = $wpdb->get_results( $query );
//echo 'archive_404_page';echo $query;
            }//end if post / page / archive /404

        }else{
            $sales = $wpdb->get_results("SELECT * from {$tableName} WHERE DATEDIFF(end_date, '".$today."') > 0 AND DATEDIFF(start_date, '".$today."') <= 0 ");
        }

        //remove sales where countdown_ends is set to relative and cookie time has run out
        foreach ($sales as $i => $sale) {

            // var_dump("<br>Name: " . $sale->product_name);
            // var_dump("<br>Starts: " . $sale->start_date);
            // var_dump("<br>Ends: " . $sale->end_date);

            if( isset( $sale->countdown_ends ) && isset( $sale->relative_time )){
                if( $sale->countdown_ends == 'relative' ){
                    if( isset($_COOKIE['refer-' . $sale->id . '-relative-end-date']) ){
                        if( $_COOKIE['refer-' . $sale->id . '-relative-end-date'] - time() <= 0 ){
                            unset( $sales[$i] );
                        }
                    }
                }
            }
        }

        // get number of items
        if (empty($sales)) {
            return false;
        }

        return $sales[array_rand($sales, 1)];
    }


}
