<?php

namespace IncomeSchool\AffiliateLinkRefer;

class CountdownLogger
{

    private $countdownId = false;

    /**
     * Construct
     */
    public function __construct()
    {
        $this->affiliateId = get_option('refer_amazon_affiliate_id', false);

        add_action('wp_ajax_countdown_log_click', [$this, 'logAjaxClick'], 99);
        add_action('wp_ajax_nopriv_countdown_log_click', [$this, 'logAjaxClick'], 99);

        /* Simple function to log existing daily trackings for debugging purposes
        /* comment out when not in active use */
        // add_action('loop_start', [$this, 'debug_post_log']);

    }

    public function debug_post_log(){

        global $post, $wpdb;

        if( $post != NULL){

            $trackingTableName = $wpdb->prefix . 'countdown_daily_trackings';

            $today = date('Y-m-d');

            $existingDaily = $wpdb->get_row("
                SELECT *
                FROM $trackingTableName
                WHERE post_id = $post->ID
                AND date = '$today'
            ");

            var_dump($existingDaily);
        }
    }


    /**
     * [log description]
     * @return [type] [description]
     */
    public function logAjaxClick()
    {

        // error_log("logAjaxClick()");

        $valid = true;
        if(!isset($_POST['nonce'])) {
            $valid = false;
        } elseif(!wp_verify_nonce($_POST['nonce'], 'update_refer_link_tracking')) { //this is set when adding js object via wp_localize_script()
            $valid = false;
        }

        if(!$valid) {
            wp_die('Invalid nonce specified', 'Error', [
                'response'  => 403,
            ]);
        }

        $post_id = isset($_POST['postId']) ? $_POST['postId'] : null;
        $clock_id = isset($_POST['clockId']) ? $_POST['clockId'] : false;

        if(!$clock_id) {
            wp_die('Invalid clock_id specified', 'Error');
        }

        $countdown = $this->getCountdown($clock_id);

        if(!$countdown) {
            wp_die('Invalid clock_id specified', 'Error');
        }

        $success = $this->logInteraction($clock_id, 'clicks', $post_id);

        echo json_encode(['success' => $success? "true" : "false" ]);
        wp_die();
    }

    public function logView($clock_id)
    {
        global $post;
        if( isset($post) ){
            if( !is_archive() && !is_home() ){

                $this->logInteraction($clock_id, 'views', $post->ID);
            }
        }
    }

   
    public function logInteraction($clock_id, $type = 'views', $post_id)
    {
        global $wpdb;

        /* The variable $type can be either 'views' or 'clicks' */

        $countdown = $this->getCountdown($clock_id);

        if($countdown) {
            $trackingTableName = $wpdb->prefix . 'refer_countdown_daily_trackings';

            $today = date('Y-m-d');

            $existingDaily = $wpdb->get_row("
                SELECT *
                FROM $trackingTableName
                WHERE
                    clock_id = $clock_id
                AND
                    post_id = $post_id
                AND
                    date = '$today'
                LIMIT 1
            ");

            if( !$existingDaily ) {
                $wpdb->insert($trackingTableName, [
                    'post_id' => $post_id,
                    'clock_id' => $clock_id,
                    $type => 1,
                    'date' => $today
                ]);
            } else {
                $wpdb->update($trackingTableName, [
                    $type => $existingDaily->{$type}+1
                ], [
                    'id' => $existingDaily->id
                ]);
            }

            //increment count on clock itself
            $wpdb->update($wpdb->prefix . 'refer_countdown_links', [
                $type => $countdown->{$type}+1
            ], [
                'id' => $clock_id
            ]);

            return true;
     
        }else{
            error_log("invalid countdown id (class-countdown-logger.php");

        }
        return false;
    }

    /**
     * [getCountdown description]
     * @param  [type] $url [description]
     * @return [type]      [description]
     */
    private function getCountdown($id)
    {
        global $post, $wpdb;

        $countdownTableName = $wpdb->prefix . 'refer_countdown_links';

        return $wpdb->get_row($wpdb->prepare("SELECT * FROM $countdownTableName WHERE id = '%s'", $id));
    }
}
