<?php

namespace IncomeSchool\AffiliateLinkRefer;

class DatabaseUpdater
{
    /**
     * Construct
     */
    public function __construct()
    {
        add_action('init', [$this, 'update']);

        add_action('refer_converted_daily_tracking_data', [$this, 'convertDailyTrackingData']);
    }
	
    /**
     * The base tables from version 1.0.
     * This should run in plugin activation hook.
     * And sequential updates take place afterward.
	 * Activate runs AFTER Update
     * @return [type] [description]
     */
    public function activate()
    {
        global $wpdb;

        $charsetCollate = $wpdb->get_charset_collate();

        require_once(ABSPATH . 'wp-admin/includes/upgrade.php');
		
		$version = get_option('affiliate-link-sherpa-plugin-version');
		
		if(!$version) {
			$version = get_option('affiliate-link-refer-plugin-version');
		}
		
		//write_log('1activate: version = '.$version);
		
		if(!$version) {
			//write_log('2activate: version is false ');
            
			$tableName = $wpdb->prefix . 'refer_link_daily_trackings';
            dbDelta("CREATE TABLE $tableName (
                  `id` int(11) unsigned NOT NULL AUTO_INCREMENT,
                  `link_id` int(11) DEFAULT NULL,
                  `post_id` int(11) DEFAULT NULL,
                  `views` int(11) DEFAULT '0',
                  `clicks` int(11) DEFAULT '0',
                  `date` date DEFAULT NULL,
                  `is_amazon` int(1) DEFAULT NULL,
                  PRIMARY KEY (`id`),
                  KEY `link_id` (`link_id`),
                  KEY `post_id` (`post_id`),
                  KEY `date` (`date`)
            ) $charsetCollate;");

			$tableName = $wpdb->prefix . 'refer_affiliate_links';

			dbDelta("CREATE TABLE $tableName (
				`id` int(11) unsigned NOT NULL AUTO_INCREMENT,
				`hash` varchar(8) DEFAULT NULL,
				`label` varchar(255) DEFAULT NULL,
				`text` varchar(255) DEFAULT NULL,
				`url` text DEFAULT NULL,
				`is_amazon` int(1) DEFAULT '0',
				`clicks` int(11) DEFAULT '0',
				`views` int(11) DEFAULT '0',
				`created_at` timestamp NULL DEFAULT CURRENT_TIMESTAMP,
				`status` varchar(255) DEFAULT NULL,
				PRIMARY KEY (`id`),
				UNIQUE KEY `hash` (`hash`)
			) $charsetCollate;");
			
			
			$tableName = $wpdb->prefix . 'refer_countdown_links';
			
            dbDelta("CREATE TABLE $tableName (
                `id` int(11) unsigned NOT NULL AUTO_INCREMENT,
				`product_name` varchar(255) DEFAULT 'click',
				`product_image_id` varchar(255) DEFAULT 'click',
				`call_to_action` varchar(500) DEFAULT NULL,
				`button_label` varchar(255) DEFAULT 'click',
				`purchase_link` varchar(255) DEFAULT NULL,
				`views` int(11) DEFAULT '0',
				`clicks` int(11) DEFAULT '0',
				`start_date` datetime DEFAULT NULL,
				`end_date` datetime DEFAULT NULL,
				`display_timer` varchar(100) NOT NULL DEFAULT '',
				`clock_name` varchar(100) NOT NULL DEFAULT '',
				`product_description` varchar(250) NOT NULL DEFAULT '',
				`context_button_label` varchar(100) NOT NULL DEFAULT '',
				`full_price_label` varchar(100) NOT NULL DEFAULT '',
				`countdown_ends` varchar(100) NOT NULL DEFAULT '',
				`relative_time` varchar(100) NOT NULL DEFAULT '',
				`location_header_home` varchar(100) NOT NULL DEFAULT '',
				`location_header_pages` varchar(100) NOT NULL DEFAULT '',
				`location_post_top` varchar(100) NOT NULL DEFAULT 'checked',
				`pages` varchar(100) NOT NULL DEFAULT '',
				`posts` varchar(100) NOT NULL DEFAULT '',
				`cats` varchar(100) NOT NULL DEFAULT '',
				`background_color` varchar(100) NOT NULL DEFAULT '',
				`text_color` varchar(100) NOT NULL DEFAULT '',
				`button_color` varchar(100) NOT NULL DEFAULT '',
				`button_text_color` varchar(100) NOT NULL DEFAULT '',
				`header_background_color` varchar(100) NOT NULL DEFAULT '',
				`header_text_color` varchar(100) NOT NULL DEFAULT '',
				`header_button_color` varchar(100) NOT NULL DEFAULT '',
				`header_button_text_color` varchar(100) NOT NULL DEFAULT '',
				`header_context_button_label` varchar(100) NOT NULL DEFAULT '',
				`header_full_price_label` varchar(100) NOT NULL DEFAULT '',
				PRIMARY KEY (`id`),
				KEY `index_start_date` (`start_date`),
				KEY `index_end_date` (`end_date`)
            ) $charsetCollate;");

			
			$tableName = $wpdb->prefix . 'refer_amazon_shortlinks_known';
			
            dbDelta("CREATE TABLE $tableName (
                `id` int(11) unsigned NOT NULL AUTO_INCREMENT,
				`short_link` text DEFAULT NULL,
				`full_link` text DEFAULT NULL,
				`process_time` int(11) DEFAULT NULL,
				PRIMARY KEY (`id`)
            ) $charsetCollate;");
			
			
			$tableName = $wpdb->prefix . 'refer_amazon_shortlinks_queue';
			
            dbDelta("CREATE TABLE $tableName (
                `id` int(11) unsigned NOT NULL AUTO_INCREMENT,
				`post_id` int(11) DEFAULT NULL,
				`widget_id` varchar(255) DEFAULT NULL,
				`created_date` datetime DEFAULT NULL,
				PRIMARY KEY (`id`),
				KEY `index_post_id` (`post_id`),
				KEY `index_widget_id` (`widget_id`(191)),
				KEY `index_created_date` (`created_date`)
            ) $charsetCollate;");
			
			
			$tableName = $wpdb->prefix . 'refer_countdown_daily_trackings';
			
            dbDelta("CREATE TABLE $tableName (
                `id` int(11) unsigned NOT NULL AUTO_INCREMENT,
				`post_id` int(11) DEFAULT NULL,
				`clock_id` int(11) DEFAULT NULL,
				`views` int(11) DEFAULT NULL,
				`clicks` int(11) DEFAULT NULL,
				`date` date DEFAULT NULL,
				PRIMARY KEY (`id`),
				KEY `index_post_id` (`post_id`),
				KEY `index_widget_id` (`clock_id`),
				KEY `index_created_date` (`date`)
            ) $charsetCollate;");
			
			$test = update_option('affiliate-link-refer-plugin-version', '2.1.0');
			//write_log('3activate: update_option affiliate-link-refer-plugin-version to 2.0.1? '.$test);
			
		}else{
			
			if($version == '1.0.0') {
				$this->version100to110();
				$this->version110to111();
				$this->version111to112();
				$this->version112to120();
				$this->version120to201();
			}
			
			elseif($version == '1.1.0') {
				$this->version110to111();
				$this->version111to112();
				$this->version112to120();
				$this->version120to201();
			}

			elseif($version == '1.1.1') {
				$this->version111to112();
				$this->version112to120();
				$this->version120to201();
			}
			
			elseif($version == '1.1.2') {
				$this->version112to120();
				$this->version120to201();
			}
			
			elseif($version == '1.2.0') {
				$this->version120to201();
			}
			
			elseif($version == '2.0.1') {
				$this->version201to210();
			}
            
            elseif($version == '2.1.0') {
                $this->version210to220();
            }
		}

    }

    /**
     * [update description]
     * @return [type] [description]
     */
    public function update()
    {
        $version = get_option('affiliate-link-sherpa-plugin-version');
		if(!$version) {
			$version = get_option('affiliate-link-refer-plugin-version');
		}
		// error_log('4update: affiliate-link-sherpa-plugin-version= '.$version);

        if($version == '1.0.0') {
            $this->version100to110();
            $this->version110to111();
            $this->version111to112();
            $this->version112to120();
            $this->version120to201();
        }

        elseif($version == '1.1.0') {
            $this->version110to111();
            $this->version111to112();
            $this->version112to120();
            $this->version120to201();
        }

        elseif($version == '1.1.1') {
            $this->version111to112();
            $this->version112to120();
            $this->version120to201();
        }

        elseif($version == '1.1.2') {
            $this->version112to120();
            $this->version120to201();
        }

        elseif($version == '1.2.0') {
            $this->version120to201();
        }
        
		elseif($version == '2.0.1') {
            $this->version201to210();
        }
        
        elseif($version == '2.1.0') {
            $this->version210to220();
        }
    }

    /**
     * [version100to110 description]
     * @return [type] [description]
     */
    public function version100to110()
    {
        global $wpdb;

        $charsetCollate = $wpdb->get_charset_collate();

        require_once(ABSPATH . 'wp-admin/includes/upgrade.php');

        // version 1.0 to 1.1 updates
        $oldVersion = get_option('affiliate-link-sherpa-plugin-version', '1.0.0');
        $newVersion = '1.1.0';
        if((version_compare($oldVersion, $newVersion) < 0)) {
            $tableName = $wpdb->prefix . 'sherpa_link_daily_trackings';
            dbDelta("CREATE TABLE $tableName (
                `id` int(11) unsigned NOT NULL AUTO_INCREMENT,
                `link_id` int(11) DEFAULT NULL,
                `post_id` int(11) DEFAULT NULL,
                `views` int(11) DEFAULT 0,
                `is_amazon` int(1) DEFAULT NULL,
                `clicks` int(11) DEFAULT 0,
                `date` date NULL DEFAULT NULL,
                PRIMARY KEY  (`id`),
                KEY link_id (`link_id`),
                KEY post_id (`post_id`)
            ) $charsetCollate;");
            $test  = update_option('affiliate-link-sherpa-plugin-version', $newVersion);
        }
		//write_log('5update version100to110: update_option affiliate-link-sherpa-plugin-version to 1.1.0? '.$test);
        $convertedData = get_option('sherpa-plugin-converted-1.1.0-daily-tracking-data', 0);

        $oldDataTableName = $wpdb->prefix . 'sherpa_link_trackings';
		
        if(!$convertedData) {
			//write_log('6$convertedData false');
            if($wpdb->get_var("SHOW TABLES LIKE '$oldDataTableName'") == $oldDataTableName ) {

                $hasRows = $wpdb->get_row("
                    SELECT id
                    FROM $oldDataTableName
                    LIMIT 1
                ");

                if($hasRows) {
                    if(!wp_next_scheduled( 'refer_converted_daily_tracking_data')) {
                        wp_schedule_event( time(), 'refer_ten_min_interval', 'refer_converted_daily_tracking_data' );
                    }
                } else {
                    update_option('sherpa-plugin-converted-1.1.0-daily-tracking-data', 1);
                }
            } else {
                update_option('sherpa-plugin-converted-1.1.0-daily-tracking-data', 1);
            }
        } else {
            if(wp_next_scheduled( 'sherpa_converted_daily_tracking_data')) {
                wp_clear_scheduled_hook('sherpa_converted_daily_tracking_data');
            }
            //$wpdb->query("DROP TABLE IF EXISTS $oldDataTableName;");
        }
    }


    /**
     * [convertDailyTrackingData description]
     * @return [type] [description]
     */
    public function convertDailyTrackingData()
    {
        //write_log('7convertDailyTrackingData fired');
		global $wpdb;

        if(get_option('sherpa-plugin-converted-1.1.0-daily-tracking-data', 0)) {
            return;
        }

        $oldDataTableName = $wpdb->prefix . 'sherpa_link_trackings';

        $rowsToConvert = $wpdb->get_results("
            SELECT *
            FROM $oldDataTableName
            ORDER BY id DESC
            LIMIT 50000
        ");

        $newDataTableName = $wpdb->prefix . 'sherpa_link_daily_trackings';

        foreach($rowsToConvert as $row) {

            $date = date('Y-m-d', strtotime($row->created_at));

            $existingDaily = $wpdb->get_row("
                SELECT *
                FROM $newDataTableName
                WHERE
                    link_id = $row->link_id
                AND
                    post_id = $row->post_id
                AND
                    date = '$date'
                LIMIT 1
            ");

            if(!$existingDaily) {
                if($row->type == 'view') {
                    $wpdb->insert($newDataTableName, array(
                        'post_id' => $row->post_id,
                        'link_id' => $row->link_id,
                        'views' => 1,
                        'date' => $date
                    ));
                } else {
                    $wpdb->insert($newDataTableName, array(
                        'post_id' => $row->post_id,
                        'link_id' => $row->link_id,
                        'clicks' => 1,
                        'date' => $date
                    ));
                }
            } else {
                if($row->type == 'view') {
                    $wpdb->update($newDataTableName, array(
                        'views' => $existingDaily->views+1
                    ), array(
                        'id' => $existingDaily->id
                    ));
                } else {
                    $wpdb->update($newDataTableName, array(
                        'clicks' => $existingDaily->clicks+1
                    ), array(
                        'id' => $existingDaily->id
                    ));
                }
            }
            $wpdb->delete($oldDataTableName, array('id' => $row->id));
        }
    }


    /**
     * [version110to111 description]
     * @return [type] [description]
     */
    public function version110to111()
    {
        global $wpdb;

        $charsetCollate = $wpdb->get_charset_collate();

        require_once(ABSPATH . 'wp-admin/includes/upgrade.php');

        // version 1.1.0 to 1.1.1 updates
        $oldVersion = get_option('affiliate-link-sherpa-plugin-version', '1.1.0');
        $newVersion = '1.1.1';
        if((version_compare($oldVersion, $newVersion) < 0)) {
            $tableName = $wpdb->prefix . 'sherpa_link_daily_trackings';
            dbDelta("CREATE TABLE $tableName (
                `id` int(11) unsigned NOT NULL AUTO_INCREMENT,
                `link_id` int(11) DEFAULT NULL,
                `post_id` int(11) DEFAULT NULL,
                `views` int(11) DEFAULT 0,
                `is_amazon` int(1) DEFAULT NULL,
                `clicks` int(11) DEFAULT 0,
                `date` date NULL DEFAULT NULL,
                PRIMARY KEY  (`id`),
                KEY link_id (`link_id`),
                KEY post_id (`post_id`),
                KEY date (`date`)
            ) $charsetCollate;");
            $test = update_option('affiliate-link-sherpa-plugin-version', $newVersion);
			//write_log('8update version110to111: update_option affiliate-link-sherpa-plugin-version to 1.1.1? '.$test);
        }
    }

    /**
     * [version111to112 description]
     * @return [type] [description]
     */
    public function version111to112() {
        global $wpdb;

        $charsetCollate = $wpdb->get_charset_collate();

        require_once(ABSPATH . 'wp-admin/includes/upgrade.php');

        // version 1.1.1 to 1.1.2 updates
        $oldVersion = get_option('affiliate-link-sherpa-plugin-version', '1.1.1');
        $newVersion = '1.1.2';
        $tableName = $wpdb->prefix . 'sherpa_countdown_links';
        $query = $wpdb->prepare( 'SHOW TABLES LIKE %s', $wpdb->esc_like( $tableName ) );

        if ( ! $wpdb->get_var( $query ) == $tableName || (version_compare($oldVersion, $newVersion) < 0)) {

            dbDelta("CREATE TABLE $tableName (
                `id` int(11) unsigned NOT NULL AUTO_INCREMENT,
                `product_name` VARCHAR(255) DEFAULT 'click',
                `product_image_id` VARCHAR(255) DEFAULT 'click',
                `call_to_action` VARCHAR(500),
                `button_label` VARCHAR(255) DEFAULT 'click',
                `purchase_link` VARCHAR(255),
                `views` int(11) DEFAULT 0,
                `clicks` int(11) DEFAULT 0,
                `start_date` datetime NULL,
                `end_date` datetime NULL default null ,
                PRIMARY KEY  (`id`),
                KEY index_start_date (`start_date`),
                KEY index_end_date (`end_date`)
            ) $charsetCollate;");
            $test = update_option('affiliate-link-refer-plugin-version', $newVersion);
			
			//write_log('9update version111to112: update_option affiliate-link-sherpa-plugin-version to 1.1.2? '.$test);
        }
    }

    /**
     * [version112to120 description]
     * @return [type] [description]
     */
    public function version112to120() {
        global $wpdb;
        $charsetCollate = $wpdb->get_charset_collate();

        require_once(ABSPATH . 'wp-admin/includes/upgrade.php');

        // version 1.1.2 to 1.2.0 updates
        $oldVersion = get_option('affiliate-link-sherpa-plugin-version', '1.1.2');
        $newVersion = '1.2.0';

        $dbVersionOutdated = version_compare($oldVersion, $newVersion) < 0;

        $tableName = $wpdb->prefix . 'sherpa_amazon_shortlinks_known';
        $query = $wpdb->prepare( 'SHOW TABLES LIKE %s', $wpdb->esc_like( $tableName ) );

        if ( ! $wpdb->get_var( $query ) == $tableName || $dbVersionOutdated) {

            dbDelta("CREATE TABLE $tableName (
                `id` int(11) unsigned NOT NULL AUTO_INCREMENT,
                `short_link` text DEFAULT NULL,
                `full_link` text DEFAULT NULL,
                `process_time` int(11) DEFAULT NULL,
                PRIMARY KEY  (`id`)
            ) $charsetCollate;");
            update_option('affiliate-link-sherpa-plugin-version', $newVersion);
			//write_log('10update version112to120: sherpa_amazon_shortlinks_knowncreated, update_option affiliate-link-sherpa-plugin-version to 1.2.0? '.$test);
        }else{
            //if table exists, add processing time column
            $test_row = $wpdb->get_results(  "SELECT COLUMN_NAME FROM INFORMATION_SCHEMA.COLUMNS WHERE TABLE_NAME = '" . $tableName . "' AND COLUMN_NAME = 'process_time'"  );
            if(empty($test_row)){
                $wpdb->query("ALTER TABLE " . $tableName . " ADD process_time int(11) DEFAULT NULL");
				//write_log('11update version112to120: sherpa_amazon_shortlinks_knowncreated updated ');
            }
        }

        $tableName = $wpdb->prefix . 'sherpa_amazon_shortlinks_queue';
        $query = $wpdb->prepare( 'SHOW TABLES LIKE %s', $wpdb->esc_like( $tableName ) );

        if ( ! $wpdb->get_var( $query ) == $tableName || $dbVersionOutdated) {

            dbDelta("CREATE TABLE $tableName (
                `id` int(11) unsigned NOT NULL AUTO_INCREMENT,
                `post_id` int(11) DEFAULT NULL,
                `widget_id` varchar(255) DEFAULT NULL,
                `created_date` datetime NULL,
                PRIMARY KEY  (`id`),
                KEY index_post_id (`post_id`),
                KEY index_widget_id (`widget_id`),
                KEY index_created_date (`created_date`)
            ) $charsetCollate;");

            $test = update_option('affiliate-link-sherpa-plugin-version', $newVersion);
			//write_log('12update version112to120: sherpa_amazon_shortlinks_queue created, update_option affiliate-link-sherpa-plugin-version to 1.2.0? '.$test);
        }

        $tableName = $wpdb->prefix . 'countdown_daily_trackings';
        $query = $wpdb->prepare( 'SHOW TABLES LIKE %s', $wpdb->esc_like( $tableName ) );

        if ( ! $wpdb->get_var( $query ) == $tableName || $dbVersionOutdated) {

            dbDelta("CREATE TABLE $tableName (
                `id` int(11) unsigned NOT NULL AUTO_INCREMENT,
                `post_id` int(11) DEFAULT NULL,
                `clock_id` int(11) DEFAULT NULL,
                `date` datetime NULL,
                `views` int(11) DEFAULT NULL,
                `clicks` int(11) DEFAULT 0,
                PRIMARY KEY  (`id`),
                KEY index_post_id (`post_id`),
                KEY index_created_date (`date`)
            ) $charsetCollate;");

            $test = update_option('affiliate-link-sherpa-plugin-version', $newVersion);
			//write_log('13update version112to120: countdown_daily_trackings created, update_option affiliate-link-sherpa-plugin-version to 1.2.0? '.$test);
        }

        //add is_amazon field to sherpa_link_daily_trackings table {
        $tableName = $wpdb->prefix . 'sherpa_link_daily_trackings';
        $test_row = $wpdb->get_results("SELECT COLUMN_NAME FROM INFORMATION_SCHEMA.COLUMNS WHERE TABLE_NAME = '" . $tableName . "' AND COLUMN_NAME = 'is_amazon'"  );
        if(empty($test_row)){
            $test = $wpdb->query("ALTER TABLE " . $tableName . " ADD is_amazon int(1) DEFAULT NULL ");
			//write_log('14update version112to120: add is_amazon to sherpa_link_daily_trackings? '.$test);
        }

        //add is_amazon and status fields to sherpa_affiliate_links table {
        $tableName = $wpdb->prefix . 'sherpa_affiliate_links';
        $test_row = $wpdb->get_results(  "SELECT COLUMN_NAME FROM INFORMATION_SCHEMA.COLUMNS WHERE TABLE_NAME = '" . $tableName . "' AND COLUMN_NAME = 'is_amazon'"  );
        if(empty($test_row)){
            $test = $wpdb->query("ALTER TABLE " . $tableName . " ADD is_amazon int(1) DEFAULT NULL");
			//write_log('15update version112to120: add is_amazon to sherpa_affiliate_links? '.$test);
        }
        $test_row = $wpdb->get_results(  "SELECT COLUMN_NAME FROM INFORMATION_SCHEMA.COLUMNS WHERE TABLE_NAME = '" . $tableName . "' AND COLUMN_NAME = 'status'"  );
        if(empty($test_row)){
            $test = $wpdb->query("ALTER TABLE " . $tableName . " ADD status varchar(255) DEFAULT NULL");
			//write_log('16update version112to120: add status to sherpa_affiliate_links? '.$test);
        }
		
		////TODO
		////Copy is_amazon values from wp_refer_affiliate_links to wp_refer_link_daily_trackings
		////update wp_refer_link_daily_trackings set is_amazon = 1 
		////where link_id in 
		////(
		////SELECT * FROM
		////(
		////	select id from wp_refer_affiliate_links where is_amazon = 1
		////) AS alink
		////	)
        
    }
	
	
	 /**
     * [version120to201 description]
     * @return [type] [description]
     */
    public function version120to201() {
        global $wpdb;
        $charsetCollate = $wpdb->get_charset_collate();

        require_once(ABSPATH . 'wp-admin/includes/upgrade.php');

        // version 1.2.0 to 2.0.1 updates
        $oldVersion = get_option('affiliate-link-sherpa-plugin-version', '1.2.0');
        $newVersion = '2.0.1';
		
        $dbVersionOutdated = version_compare($oldVersion, $newVersion) < 0;
		//write_log('17update version120to201: $dbVersionOutdated= '.$dbVersionOutdated);
		if ($dbVersionOutdated){
			$tableName = $wpdb->prefix . 'countdown_daily_trackings';
			$newTableName = $wpdb->prefix . 'refer_countdown_daily_trackings';
			$query = $wpdb->prepare( 'SHOW TABLES LIKE %s', $wpdb->esc_like( $tableName ) );
			if ( $wpdb->get_var( $query ) == $tableName ) {
				$alter = $wpdb->query("ALTER TABLE $tableName RENAME $newTableName");
				$drop = $wpdb->query( "DROP TABLE IF EXISTS $tableName" );
				//write_log('18update version120to201: alter countdown_daily_trackings= '.$alter.' drop countdown_daily_trackings= '.$drop);
			}

			$tableName = $wpdb->prefix . 'sherpa_affiliate_links';
			$newTableName = $wpdb->prefix . 'refer_affiliate_links';
			$query = $wpdb->prepare( 'SHOW TABLES LIKE %s', $wpdb->esc_like( $tableName ) );
			if ( $wpdb->get_var( $query ) == $tableName ) {
				$alter = $wpdb->query("ALTER TABLE $tableName RENAME $newTableName");
				$drop = $wpdb->query( "DROP TABLE IF EXISTS $tableName" );
				//write_log('19update version120to201: alter sherpa_affiliate_links = '.$alter.' drop sherpa_affiliate_links = '.$drop);
			}

			$tableName = $wpdb->prefix . 'sherpa_link_daily_trackings';
			$newTableName = $wpdb->prefix . 'refer_link_daily_trackings';
			$query = $wpdb->prepare( 'SHOW TABLES LIKE %s', $wpdb->esc_like( $tableName ) );
			if ( $wpdb->get_var( $query ) == $tableName ) {
				$alter = $wpdb->query("ALTER TABLE $tableName RENAME $newTableName");
				$drop = $wpdb->query( "DROP TABLE IF EXISTS $tableName" );
				//write_log('20update version120to201: alter sherpa_link_daily_trackings = '.$alter.' drop sherpa_link_daily_trackings = '.$drop);
			}

			$tableName = $wpdb->prefix . 'sherpa_countdown_links';
			$newTableName = $wpdb->prefix . 'refer_countdown_links';
			$query = $wpdb->prepare( 'SHOW TABLES LIKE %s', $wpdb->esc_like( $tableName ) );
			if ( $wpdb->get_var( $query ) == $tableName ) {
				$alter = $wpdb->query("ALTER TABLE $tableName RENAME $newTableName");
				$drop = $wpdb->query( "DROP TABLE IF EXISTS $tableName" );
				//write_log('21update version120to201: alter sherpa_countdown_links = '.$alter.' drop sherpa_countdown_links = '.$drop);
			}

			$tableName = $wpdb->prefix . 'sherpa_amazon_shortlinks_known';
			$newTableName = $wpdb->prefix . 'refer_amazon_shortlinks_known';
			$query = $wpdb->prepare( 'SHOW TABLES LIKE %s', $wpdb->esc_like( $tableName ) );
			if ( $wpdb->get_var( $query ) == $tableName ) {
				$alter = $wpdb->query("ALTER TABLE $tableName RENAME $newTableName");
				$drop = $wpdb->query( "DROP TABLE IF EXISTS $tableName" );
				//write_log('22update version120to201: alter sherpa_amazon_shortlinks_known = '.$alter.' drop sherpa_amazon_shortlinks_known = '.$drop);
			}

			$tableName = $wpdb->prefix . 'sherpa_amazon_shortlinks_queue';
			$newTableName = $wpdb->prefix . 'refer_amazon_shortlinks_queue';
			$query = $wpdb->prepare( 'SHOW TABLES LIKE %s', $wpdb->esc_like( $tableName ) );
			if ( $wpdb->get_var( $query ) == $tableName ) {
				$alter = $wpdb->query("ALTER TABLE $tableName RENAME $newTableName");
				$drop = $wpdb->query( "DROP TABLE IF EXISTS $tableName" );
				//write_log('23update version120to201: alter sherpa_amazon_shortlinks_queue = '.$alter.' drop sherpa_amazon_shortlinks_queue = '.$drop);
			}
			
			//replace 'sherpa' w/ 'refer' for all options
			$update_options = $wpdb->query("UPDATE $wpdb->options SET option_name = REPLACE(option_name, 'sherpa', 'refer') WHERE option_name LIKE '%sherpa%'");
			//write_log('24update version120to201: update options = '.$update_options);
			$delete = delete_option('affiliate-link-sherpa-plugin-version');
			//write_log('25update version120to201: delete affiliate-link-sherpa-plugin-version = '.$delete);
			$update = update_option('affiliate-link-refer-plugin-version', '2.0.1');
			//write_log('26update version120to201: update affiliate-link-refer-plugin-version to 2.0.0 = '.$update);
		}
	}

	/**
     * [version201to210 description]
     * @return [type] [description]
     */
    public function version201to210() {
        global $wpdb;
        $charsetCollate = $wpdb->get_charset_collate();

        require_once(ABSPATH . 'wp-admin/includes/upgrade.php');

        // version 2.0.1 to 2.1.0 updates
        $oldVersion = get_option('affiliate-link-sherpa-plugin-version');
        if(false === $oldVersion){
            $oldVersion = get_option('affiliate-link-refer-plugin-version', '2.0.1');
        }
        
        $newVersion = '2.1.0';
		
        $dbVersionOutdated = version_compare($oldVersion, $newVersion) < 0;
		if ($dbVersionOutdated){
			//CREATE TABLE will ALTER TABLE if table exists!!!!
			$tableName = $wpdb->prefix . 'refer_link_daily_trackings';
            dbDelta("CREATE TABLE $tableName (
                  `id` int(11) unsigned NOT NULL AUTO_INCREMENT,
                  `link_id` int(11) DEFAULT NULL,
                  `post_id` int(11) DEFAULT NULL,
                  `views` int(11) DEFAULT '0',
                  `clicks` int(11) DEFAULT '0',
                  `date` date DEFAULT NULL,
                  `is_amazon` int(1) DEFAULT NULL,
                  PRIMARY KEY (`id`),
                  KEY `link_id` (`link_id`),
                  KEY `post_id` (`post_id`),
                  KEY `date` (`date`)
            ) $charsetCollate;");

			$tableName = $wpdb->prefix . 'refer_affiliate_links';
			dbDelta("CREATE TABLE $tableName (
				`id` int(11) unsigned NOT NULL AUTO_INCREMENT,
				`hash` varchar(8) DEFAULT NULL,
				`label` varchar(255) DEFAULT NULL,
				`text` varchar(255) DEFAULT NULL,
				`url` text DEFAULT NULL,
				`is_amazon` int(1) DEFAULT '0',
				`clicks` int(11) DEFAULT '0',
				`views` int(11) DEFAULT '0',
				`created_at` timestamp NULL DEFAULT CURRENT_TIMESTAMP,
				`status` varchar(255) DEFAULT NULL,
				PRIMARY KEY (`id`),
				UNIQUE KEY `hash` (`hash`)
			) $charsetCollate;");
			
			
			$tableName = $wpdb->prefix . 'refer_countdown_links';		
            dbDelta("CREATE TABLE $tableName (
                `id` int(11) unsigned NOT NULL AUTO_INCREMENT,
				`product_name` varchar(255) DEFAULT 'click',
				`product_image_id` varchar(255) DEFAULT 'click',
				`call_to_action` varchar(500) DEFAULT NULL,
				`button_label` varchar(255) DEFAULT 'click',
				`purchase_link` varchar(255) DEFAULT NULL,
				`views` int(11) DEFAULT '0',
				`clicks` int(11) DEFAULT '0',
				`start_date` datetime DEFAULT NULL,
				`end_date` datetime DEFAULT NULL,
				`display_timer` varchar(100) NOT NULL DEFAULT '',
				`clock_name` varchar(100) NOT NULL DEFAULT '',
				`product_description` varchar(250) NOT NULL DEFAULT '',
				`context_button_label` varchar(100) NOT NULL DEFAULT '',
				`full_price_label` varchar(100) NOT NULL DEFAULT '',
				`countdown_ends` varchar(100) NOT NULL DEFAULT '',
				`relative_time` varchar(100) NOT NULL DEFAULT '',
				`location_header_home` varchar(100) NOT NULL DEFAULT '',
				`location_header_pages` varchar(100) NOT NULL DEFAULT '',
				`location_post_top` varchar(100) NOT NULL DEFAULT 'checked',
				`pages` varchar(100) NOT NULL DEFAULT '',
				`posts` varchar(100) NOT NULL DEFAULT '',
				`cats` varchar(100) NOT NULL DEFAULT '',
				`background_color` varchar(100) NOT NULL DEFAULT '',
				`text_color` varchar(100) NOT NULL DEFAULT '',
				`button_color` varchar(100) NOT NULL DEFAULT '',
				`button_text_color` varchar(100) NOT NULL DEFAULT '',
				`header_background_color` varchar(100) NOT NULL DEFAULT '',
				`header_text_color` varchar(100) NOT NULL DEFAULT '',
				`header_button_color` varchar(100) NOT NULL DEFAULT '',
				`header_button_text_color` varchar(100) NOT NULL DEFAULT '',
				`header_context_button_label` varchar(100) NOT NULL DEFAULT '',
				`header_full_price_label` varchar(100) NOT NULL DEFAULT '',
				PRIMARY KEY (`id`),
				KEY `index_start_date` (`start_date`),
				KEY `index_end_date` (`end_date`)
            ) $charsetCollate;");
			
			$tableName = $wpdb->prefix . 'refer_countdown_daily_trackings';
            dbDelta("CREATE TABLE $tableName (
                `id` int(11) unsigned NOT NULL AUTO_INCREMENT,
				`post_id` int(11) DEFAULT NULL,
				`clock_id` int(11) DEFAULT NULL,
				`views` int(11) DEFAULT NULL,
				`clicks` int(11) DEFAULT NULL,
				`date` date DEFAULT NULL,
				PRIMARY KEY (`id`),
				KEY `index_post_id` (`post_id`),
				KEY `index_widget_id` (`clock_id`),
				KEY `index_created_date` (`date`)
            ) $charsetCollate;");

			$update = update_option('affiliate-link-refer-plugin-version', '2.1.0');
		}
	}

        /**
     * [version210to220 description]
     * @return [type] [description]
     */
    public function version210to220() {

        // error_log("updating DB for version 2.2.0");

        global $wpdb;
        $charsetCollate = $wpdb->get_charset_collate();

        require_once(ABSPATH . 'wp-admin/includes/upgrade.php');

        // version 1.1.2 to 1.2.0 updates
        $oldVersion = get_option('affiliate-link-sherpa-plugin-version', '2.1.0');
        $newVersion = '2.2.0';

        $dbVersionOutdated = version_compare($oldVersion, $newVersion) < 0;

        if ($dbVersionOutdated){

            //CREATE TABLE will ALTER TABLE if table exists!!!!
            $tableName = $wpdb->prefix . 'refer_link_daily_trackings';
            dbDelta("CREATE TABLE $tableName (
                  `id` int(11) unsigned NOT NULL AUTO_INCREMENT,
                  `link_id` int(11) DEFAULT NULL,
                  `post_id` int(11) DEFAULT NULL,
                  `views` int(11) DEFAULT '0',
                  `clicks` int(11) DEFAULT '0',
                  `date` date DEFAULT NULL,
                  `is_amazon` int(1) DEFAULT NULL,
                  `type` varchar(100) DEFAULT NULL,
                  PRIMARY KEY (`id`),
                  KEY `link_id` (`link_id`),
                  KEY `post_id` (`post_id`),
                  KEY `date` (`date`)
            ) $charsetCollate;");

            $tableName = $wpdb->prefix . 'refer_affiliate_links';
            dbDelta("CREATE TABLE $tableName (
                `id` int(11) unsigned NOT NULL AUTO_INCREMENT,
                `hash` varchar(8) DEFAULT NULL,
                `label` varchar(255) DEFAULT NULL,
                `text` varchar(255) DEFAULT NULL,
                `url` text DEFAULT NULL,
                `is_amazon` int(1) DEFAULT '0',
                `type` varchar(100) DEFAULT '0',
                `last_checked` int(10) DEFAULT 0,
                `clicks` int(11) DEFAULT 0,
                `views` int(11) DEFAULT 0,
                `created_at` timestamp NULL DEFAULT CURRENT_TIMESTAMP,
                `status` varchar(255) DEFAULT NULL,
                PRIMARY KEY (`id`),
                UNIQUE KEY `hash` (`hash`)
            ) $charsetCollate;");

            $update = update_option('affiliate-link-refer-plugin-version', $newVersion);
        }
        
    }
}
