<?php

namespace IncomeSchool\AffiliateLinkRefer;

class Main
{
    /**
     * [$classes description]
     * @var array
     */
    public $classes = [];

    /**
     * [__construct description]
     */
    public function __construct()
    {
        add_filter('cron_schedules', function($schedules) {

            $schedules['refer_one_hour_interval'] = array(
                'interval' => 1800,
                'display' => 'Refer One Hour Interval',
            );

            $schedules['refer_half_hour_interval'] = array(
                'interval' => 1800,
                'display' => 'Refer Half Hour Interval',
            );

            $schedules['refer_fifteen_min_interval'] = array(
                'interval' => 900,
                'display' => 'Refer Fifteen Min Interval',
            );

            $schedules['refer_one_min_interval'] = array(
                'interval' => 60,
                'display' => 'Refer One Min Interval',
            );

            $schedules['refer_five_min_interval'] = array(
                'interval' => 300,
                'display' => 'Refer Five Min Interval',
            );

            $schedules['refer_ten_min_interval'] = array(
                'interval' => 600,
                'display' => 'Refer Ten Min Interval',
            );

            $schedules['refer_daily_interval'] = array(
                'interval' => 86400,
                'display' => 'Refer Daily Interval',
            );
            return $schedules;
        }, 99);

        include_once AFFILIATE_LINK_REFER_PLUGIN_DIR . '/includes/class-database-updater.php';
        $this->classes['database_updater'] = new DatabaseUpdater();

        include_once AFFILIATE_LINK_REFER_PLUGIN_DIR . '/includes/class-admin-interface.php';
        $this->classes['admin_interface'] = new AdminInterface();

        include_once AFFILIATE_LINK_REFER_PLUGIN_DIR . '/includes/class-refer-facts.php';
        $this->classes['refer_facts'] = new ReferFacts();

        include_once AFFILIATE_LINK_REFER_PLUGIN_DIR . '/includes/class-refer-shortcode.php';
        $this->classes['refer_shortcode'] = new ReferShortcode();

        include_once AFFILIATE_LINK_REFER_PLUGIN_DIR . '/includes/class-link-relay.php';
        $this->classes['link_relay'] = new LinkRelay();

        include_once AFFILIATE_LINK_REFER_PLUGIN_DIR . '/includes/class-affiliate-link-parser.php';
        $this->classes['affiliate_link_parser'] = new AffiliateLinkParser();

        include_once AFFILIATE_LINK_REFER_PLUGIN_DIR . '/includes/class-legal-disclaimer-widget.php';
        $this->classes['legal_disclaimer_widget'] = new LegalDisclaimerWidget();

        include_once AFFILIATE_LINK_REFER_PLUGIN_DIR . '/includes/class-legal-disclaimer-display.php';
        $this->classes['legal_disclaimer_display'] = new LegalDisclaimerDisplay();

        include_once AFFILIATE_LINK_REFER_PLUGIN_DIR . '/includes/class-affiliate-link-logger.php';
        $this->classes['affiliate_link_logger'] = new AffiliateLinkLogger();

        include_once AFFILIATE_LINK_REFER_PLUGIN_DIR . '/includes/class-countdown-logger.php';
        $this->classes['countdown_logger'] = new CountdownLogger();

        include_once AFFILIATE_LINK_REFER_PLUGIN_DIR . '/includes/class-report-data-generator.php';
        $this->classes['report_dashboard_data_generator'] = new ReportDataGenerator();

        include_once AFFILIATE_LINK_REFER_PLUGIN_DIR . '/includes/class-countdown-display.php';
        $this->classes['product_countdown_display'] = new CountdownDisplay();

        include_once AFFILIATE_LINK_REFER_PLUGIN_DIR . '/includes/class-affiliate-shortlink-converter.php';
        $this->classes['affiliate_shortlink_converter'] = new AffiliateShortlinkConverter();
        
        include_once AFFILIATE_LINK_REFER_PLUGIN_DIR . '/assets/html/admin/notifications.php';

        add_action('admin_enqueue_scripts', [$this, 'enqueueAdminStyles']);
        add_action('admin_enqueue_scripts', [$this, 'enqueueAdminScripts']);

        add_action('wp_enqueue_scripts', [$this, 'enqueueThemeStyles']);
        add_action('wp_enqueue_scripts', [$this, 'enqueueThemeScripts']);

        //Render shortcodes in excerpts
        add_filter('the_excerpt', 'shortcode_unautop');
        add_filter('the_excerpt', 'do_shortcode');

        add_action('admin_footer', [$this, 'refer_minimize_admin_menu'] );
        add_action( 'admin_head', [$this, 'refer_hide_notices'], 1 );
        add_filter( 'admin_body_class', [$this, 'refer_body_class'], 1 );

        //Register affiliates API route
        add_action('rest_api_init', function() {
            register_rest_route('refer/v1', '/affiliates', array(
                'methods' => 'GET',
                'callback' => array($this, 'referApiGetAffiliates'),
                'permission_callback' => '__return_true'
            ));
        });


        // add_action( 'init', [$this, 'test_type'] );
    }

    // function test_type(){
    //     if( isset($_GET['test_url']) )
    //         var_dump( $this->determine_affiliate_link_type($_GET['test_url']) );
    // }

    public function refer_minimize_admin_menu(){
        if( strpos( $_SERVER['QUERY_STRING'] , 'affiliate-link-refer') != false ){
            echo "<script type='text/javascript' > 
            document.body.className+=' folded';                 
            </script>"; 
            echo "<style>#wpadminbar { display:none !important;}</style>";
        }
    }

    public function refer_hide_notices(){
        if( strpos( $_SERVER['QUERY_STRING'] , 'affiliate-link-refer') != false ){
            remove_all_actions( 'admin_notices' );
        }
    }

    public function refer_body_class( $classes ) {

      if( strpos( $_SERVER['QUERY_STRING'] , 'affiliate-link-refer') != false ){
        $classes .= ' refer_page';
      }
        return $classes;
    }

    /**
     * [addAdminStyles description]
     */
    public function enqueueAdminStyles($hook_suffix)
    {
        wp_enqueue_style('wp-jquery-ui-dialog');
        wp_enqueue_style('wp-color-picker');
        wp_enqueue_style('font-awesome', 'https://stackpath.bootstrapcdn.com/font-awesome/4.7.0/css/font-awesome.min.css', [], '4.7.0');
        wp_enqueue_style('icomoon', AFFILIATE_LINK_REFER_PLUGIN_URL . '/assets/css/icomoon.css');
        wp_enqueue_style('affiliate-link-refer-admin', AFFILIATE_LINK_REFER_PLUGIN_URL .'/assets/css/admin.css');
        wp_enqueue_style('refer-admin', AFFILIATE_LINK_REFER_PLUGIN_URL .'/assets/css/refer-admin.css');
        
        //register styles to enqueue on individual admin pages
        wp_register_style('refer-chart', AFFILIATE_LINK_REFER_PLUGIN_URL .'/assets/css/refer-chart.css');
        wp_register_style('refer-table', AFFILIATE_LINK_REFER_PLUGIN_URL .'/assets/css/refer-table.css');
        wp_register_style('affiliate-link-refer-flipdown', AFFILIATE_LINK_REFER_PLUGIN_URL .'/assets/css/flipdown.css');
        wp_register_style('affiliate-link-refer-theme-style', AFFILIATE_LINK_REFER_PLUGIN_URL .'/assets/css/theme.css');
		wp_register_style('refer-daterangepicker', '//cdn.jsdelivr.net/npm/daterangepicker/daterangepicker.css');

        //we'll need these for the info-circles
        // wp_enqueue_style('jquery-ui-tooltip');

        // var_dump( $hook_suffix );
        //Main dashboard styles
        if( $hook_suffix == "toplevel_page_affiliate-link-refer"){
            wp_enqueue_style('refer-dashboard', AFFILIATE_LINK_REFER_PLUGIN_URL .'/assets/css/refer-dashboard.css');
        }

        if( strpos( $_SERVER['QUERY_STRING'] , 'affiliate-link-refer') != false ){
        	//wp_enqueue_style('bootstrap-grid', AFFILIATE_LINK_REFER_PLUGIN_URL .'/assets/css/bootstrap-grid.min.css');
			wp_enqueue_style('bootstrap-grid', '//stackpath.bootstrapcdn.com/bootstrap/4.5.2/css/bootstrap.min.css');
        }
    }

    /**
     * [addAdminScripts description]
     */
    public function enqueueAdminScripts()
    {
		wp_enqueue_media();
        wp_enqueue_script('jquery-ui-dialog');
        wp_enqueue_script('wp-color-picker');
        //wp_enqueue_script('iris');
        wp_enqueue_script('chart-js', '//cdn.jsdelivr.net/npm/chart.js@2.8.0');
        wp_enqueue_script('affiliate-link-refer-admin', AFFILIATE_LINK_REFER_PLUGIN_URL .'assets/js/admin.js', array("jquery"));
        wp_enqueue_script('refer-admin', AFFILIATE_LINK_REFER_PLUGIN_URL .'assets/js/refer-admin.js', array("jquery"));
        wp_register_script('refer-chart', AFFILIATE_LINK_REFER_PLUGIN_URL .'assets/js/refer-chart.js', array("jquery"));
        wp_register_script('refer-table', AFFILIATE_LINK_REFER_PLUGIN_URL .'assets/js/refer-table.js', array("jquery"));

        //we'll need these for the info-circles
        wp_enqueue_script('jquery-ui-tooltip');

        wp_register_script('popper-js', 'https://unpkg.com/@popperjs/core@2');
        wp_register_script('affiliate-link-refer-flipdown-js', AFFILIATE_LINK_REFER_PLUGIN_URL .'assets/js/flipdown.js');
		wp_enqueue_script('bootstrap-js', '//stackpath.bootstrapcdn.com/bootstrap/4.5.2/js/bootstrap.min.js', array('jquery', 'popper-js'));
		
		wp_register_script('refer-moment-js', '//cdn.jsdelivr.net/momentjs/latest/moment.min.js', ['jquery']);
		wp_register_script('refer-daterangepicker-js', '//cdn.jsdelivr.net/npm/daterangepicker/daterangepicker.min.js', ['jquery','refer-moment-js']);
    }

    /**
     * [enqueueThemeScripts description]
     */
    public function enqueueThemeScripts()
    {
        //if(!is_admin()) {
            global $post;
			wp_enqueue_script('jquery');
            wp_register_script('affiliate-link-refer-theme', AFFILIATE_LINK_REFER_PLUGIN_URL .'assets/js/theme.js', ['jquery'], '1.2', true);


            $params = array(
                'nonce' => wp_create_nonce('update_refer_link_tracking'),
                'postId' => get_queried_object_id(),
                'pluginsUrl' => plugins_url(),
                'ajaxUrl' => admin_url('admin-ajax.php'),
                'affiliate_links' => $this->getAffiliatesObject(),
            );
            wp_localize_script('affiliate-link-refer-theme', 'nonce_object', $params);
            wp_enqueue_script('affiliate-link-refer-theme');

        //}
    }

    // returns an object of affiliate links and their full_urls
    public function getAffiliatesObject(){
      global $wpdb;
      $tableName = $wpdb->prefix . 'refer_affiliate_links';
      $affiliate_links = $wpdb->get_results("SELECT * FROM $tableName");
      $links_object = Array();
      foreach ($affiliate_links as $link) {
        $full_link = property_exists($link, 'full_link' )? $link->full_link : false;
        $links_object[$link->id] = Array(
          'link' => $link->url,
          'full_link' => $full_link,
        );
      }
      return $links_object;
    }

    /**
     * [enqueueThemeScripts description]
     */
    public function enqueueThemeStyles()
    {

        //if(!is_admin()) {
            wp_enqueue_style( 'affiliate-link-refer-theme-style', AFFILIATE_LINK_REFER_PLUGIN_URL .'assets/css/theme.css');
            wp_enqueue_style( 'affiliate-link-refer-flipdown', AFFILIATE_LINK_REFER_PLUGIN_URL .'assets/css/flipdown.css');
            wp_enqueue_script( 'affiliate-link-refer-flipdown-js', AFFILIATE_LINK_REFER_PLUGIN_URL .'assets/js/flipdown.js', ['jquery']);
        //}
    }

    /**
     * [referApiGetAffiliates description]
     */
    public function referApiGetAffiliates()
    {
        global $wpdb, $paged, $max_num_pages;

        // error_log("referApiGetAffiliates");
        $orderByClause = "label ASC";
        $sortByLabel = "DESC";
        $sortByUrl = "ASC";
        $sortByText = "ASC";
        $sortByHash = "ASC";

        if(isset($_GET['sortOtherAffiliateByLabel']) || isset($_GET['sortOtherAffiliateByUrl'])) {
            // $orderByClause = "";
            if(isset($_GET['sortOtherAffiliateByLabel'])) {
                $orderByClause = "label {$_GET['sortOtherAffiliateByLabel']}";
                $sortUrlParamForPaging = "&sortOtherAffiliateByLabel={$_GET['sortOtherAffiliateByLabel']}";
                if($_GET['sortOtherAffiliateByLabel'] == 'ASC') {
                    $sortByLabel = "DESC";
                }
                else {
                    $sortByLabel = "ASC";
                }
            } elseif(isset($_GET['sortOtherAffiliateByUrl'])) {
                $orderByClause = "url {$_GET['sortOtherAffiliateByUrl']}";
                $sortUrlParamForPaging = "&sortOtherAffiliateByUrl={$_GET['sortOtherAffiliateByUrl']}";
                if($_GET['sortOtherAffiliateByUrl'] == 'ASC') {
                    $sortByUrl = "DESC";
                }
                else {
                    $sortByUrl = "ASC";
                }
            }
        }

        $paged = isset($_GET['paged']) ? $_GET['paged'] : 1;
        $postsPerPage = -1;
        $offset = ($paged - 1)*$postsPerPage;
        $tableName = $wpdb->prefix . 'refer_affiliate_links';

        $baseSql = " FROM $tableName WHERE is_amazon = 0 ";
        $sqlParms = [];
        $affiliates = [];
        $total = 0;
        if(isset($_GET['search']) && $_GET['search'] != '') {
            $baseSql .= " AND ( label LIKE %s OR url LIKE %s OR text LIKE %s )";
            $sqlParms = array_merge(['%' . $_GET['search'] . '%', '%' . $_GET['search'] . '%', '%' . $_GET['search'] . '%']);
            $affiliates = $wpdb->get_results($wpdb->prepare("SELECT * " . $baseSql  . " ORDER BY " . $orderByClause . " LIMIT $offset, $postsPerPage", $sqlParms));
            $total = $wpdb->get_var($wpdb->prepare("SELECT count(id) " . $baseSql, $sqlParms));
        } else {
            // error_log("sqlParms = " . print_r($sqlParms, true));
            //$affiliates = $wpdb->get_results("SELECT * " . $baseSql  . " ORDER BY " . $orderByClause . " LIMIT $offset, $postsPerPage");
            $affiliates = $wpdb->get_results("SELECT * " . $baseSql  . " ORDER BY " . $orderByClause);
            $total = $wpdb->get_var("SELECT count(id) " . $baseSql);
        }

        // $total = $wpdb->get_var($wpdb->prepare("SELECT count(id) " . $baseSql, $sqlParms));

        // $maxNumberPages = $max_num_pages = ceil($total/$postsPerPage);

        return $affiliates;
    }

    /**
    * Returns an array of all post names and IDs from a given custom post type
    * With the format {"post_title" => ID} (or reversed if "flip" = true)
    * IF post type is active, else returns false
    * @param $post_type
    * @return array
    */
    public function get_cpt_names_and_ids($post_type, $orderby = 'menu_order', $flip = false){
      
      if(post_type_exists($post_type)){

          $args = array( 
            'post_type' =>  $post_type,
            'posts_per_page' => -1,
            'post_status' => 'publish',
            'orderby' => $orderby,
            'order' => 'ASC',
          );

          $cpt_query = new \WP_Query($args);

          // var_dump($cpt_query->posts);

          if(count($cpt_query->posts) > 0) {
              $cpt_array = array();
              foreach ($cpt_query->posts as $cpt_post) {
                  // var_dump($flip);
                if($flip == true || $flip == 'true'){
                  $cpt_array[$cpt_post->ID] = $cpt_post->post_title;
                }else{
                  $cpt_array[$cpt_post->post_title] = $cpt_post->ID;
                }
              } //end each
              return $cpt_array;
          }else{
              return false;
          }
      }else{
          return false;
      }
    } //end get cpt names and ids

   /**
     * Load a theme template part while supplying data.
     * Uses this->locate_template()
     * @param string $slug The slug name for the generic template.
     * @param array $params An associated array of data that will be extracted into the templates scope
     * @param bool $output Whether to output component or return as string.
     * @return string
     * thank you https://github.com/roots/sage/issues/1414
     */
    static public function get_template_part_with_params($slug, $params = null, $output = true) {
      if(!is_array($params)) $params = (array) $params;
      if($params == null) $params = Array();
      if(!$output) ob_start();
      if(strpos($slug, ".php") === false){    
        $template_file = self::locate_template("assets/html/{$slug}.php", false, false);
      }else{
        $template_file = self::locate_template("assets/html/{$slug}", false, false);
      }
      extract($params, EXTR_SKIP);
      if($template_file && $template_file != ""){
          require($template_file);
      }
      if(!$output) return ob_get_clean();
    }

    /**
     * Thank you https://pippinsplugins.com/template-file-loaders-plugins/
     * Retrieve the name of the highest priority template file that exists.
     *
     * Searches in the STYLESHEETPATH before TEMPLATEPATH so that themes which
     * inherit from a parent theme can just overload one file. If the template is
     * not found in either of those, it looks in the theme-compat folder last.
     *
     * @param string|array $template_names Template file(s) to search for, in order.
     * @param bool $load If true the template file will be loaded if it is found (else just returns filename string)
     * @param bool $require_once Whether to require_once or require. Default true.
     *        Has no effect if $load is false.
     * @return string The template filename if one is located.
     */
    static public function locate_template( $template_names, $load = false, $require_once = true ) {
      // No file found yet
      $located = false;
   
      // Try to find a template file
      foreach ( (array) $template_names as $template_name ) {

        // Continue if template is empty
        if ( empty( $template_name ) )
            continue;

        // Trim off any slashes from the template name
        $template_name = ltrim( $template_name, '/' );

        // Check child theme first
        $path1 = trailingslashit( get_stylesheet_directory() ) . "/".$template_name;
        // Check parent theme next
        $path2 = trailingslashit( get_template_directory() ) . "/".$template_name;
        // Check plugin's root directory next
        $path3 = AFFILIATE_LINK_REFER_PLUGIN_DIR  . "/".$template_name;
        // Check same file folder last
        $path4 = trailingslashit( dirname(__FILE__) ) . "/".$template_name;

        // var_dump($path);
        if ( file_exists( $path1 )) {
            $located = $path1;
            break;

        } elseif ( file_exists( $path2 )) {
            $located = $path2;
            break;

        // Check plugin
        } elseif ( file_exists( $path3 ) ) {
            $located = $path3;
            break;

        // Check this file's directory last
        } elseif ( file_exists( $path4 ) ) {
            $located = $path4;
            break;

        }else{
            var_dump($path3 . " not located");
        }
      }
   
      if ( ( true == $load ) && ! empty( $located ) )
          load_template( $located, $require_once );
   
      return $located;
    } //end locate template

    /***
    * Returns an array of two values: 
    * [0]total # clicks within the given numDays
    * [1]total # clicks within the previous given numDays
    * Returns false on error
    ***/
    public function get_click_totals($numDays){

      $return_array = Array();

      $cachedReportData = get_option('refer-plugin-total-clicks-report-data', array());

      if( is_array( $cachedReportData )){
        if(isset($cachedReportData['data']['900_days'])) {
            $clicks_array = $cachedReportData['data']['900_days_all_type']; //30 month of data
          }else{
            $clicks_array = $cachedReportData['data']['210_days_all_type']; //30 weeks of data
          }
 
          $clicks_array0 = array_slice ( $clicks_array , 0, $numDays );
          $return_array[0] = array_sum ( $clicks_array0 );

          $clicks_array1 = array_slice ( $clicks_array , $numDays, ($numDays * 2) );
          $return_array[1] = array_sum ( $clicks_array1 );

          return $return_array;
        
      }

      return false;

    }

    /***
    * Returns an array of clicks by date over the given time period
    ***/
    public function get_link_clicks($period, $context=false){

      $clicks_array = Array();

      $presets = ['30-Days', '30-Weeks', '30-Months'];
      switch($period){

        /*case '1-week':
          $numDays = 7;
          break;

        default:
        case '1-month':
          $numDays = 30;
          break;

        case '1-year':
          $numDays = 365;
          break;*/
		default:	  
		case '30-Days':
          $numDays = 30;
          break;

        case '30-Weeks':
          $numDays = 210;
          break;

        case '30-Months':
          $numDays = 900;
          break;
			  
      }

      	//Query The total affilate clicks chart Data
		$cachedReportData = get_option('refer-plugin-total-clicks-report-data', array());

		  //uncomment below for testing
		  // $clicks_array = array();
		  // for ($i=0; $i < 29; $i++) { 
		  //   $date = date('Y-m-d', strtotime('-' . $i . ' days'));
		  //   $clicks_array[$date] = rand(0, 100);
		  // }
		  // return $clicks_array;
		
		if($context == 'dashboard' && in_array($period, $presets) == false ){
			global $wpdb;
			$date = explode('-', $period);
			$start_date = date_create_from_format('M j, Y', $date[0]);
			$end_date = date_create_from_format('M j, Y', $date[1]);
			$diff = $end_date->diff($start_date)->format("%a");
			$trackingsTableName = $wpdb->prefix . 'refer_link_daily_trackings';
			$timeLabels = array();
			$clickData = array();
			for($i = 0; $i <= $diff; $i++) {
				$startDay = date('Y-m-d', strtotime('-'. $i .' days', strtotime($date[1])));
				array_unshift($timeLabels, date('n/d/Y', strtotime($startDay)));
				array_unshift($clickData, 0);
				$sqlByDay = sprintf("SELECT clicks FROM %s t WHERE t.date = '$startDay'", $trackingsTableName);
				$clicksByDay = $wpdb->get_results($sqlByDay);
				foreach($clicksByDay as $clickByDay) {
					$clickData[0] = $clickData[0] + $clickByDay->clicks;
				}
				//write_log('days: sql = '.$sqlByDay);
			}
			return $clickData;
		}
		
		if(isset($cachedReportData['data'][$numDays . '_days_all_type'])) {
			$clicks_array = $cachedReportData['data'][$numDays . '_days_all_type'];
		}

      return $clicks_array;
    }

    public  function random_array($n){
        $array = Array();
        for ($i=0; $i < $n; $i++) { 
           $array[] = rand ( 0, 2000 );
        }
        return $array;
    }

    public  function get_link_icon($url){
        $icon = '<span class="link-icon centered-vertically"><span class="icon-hyperlink-2-1"></span></span>';
        $brands = array(
          'target' => "/assets/images/target.png",
          'amzn' => "/assets/images/amazon.png",
          'amazon' => "/assets/images/amazon.png",
          'ebay' => "/assets/images/ebay.png",
          'ecwid' => "/assets/images/ecwid.png",
          'walmart' => "/assets/images/walmart.png",
        );

        foreach ($brands as $key => $value) {
          
          if( strpos($url, $key) != false ){
            $icon = '<span class="link-icon centered-vertically ' . $key . '"><img src="' . AFFILIATE_LINK_REFER_PLUGIN_URL . $value . '" alt="' . $key . '"/></span>';
          }
        }

        return $icon;
    }
    /* Returns the affiliate type based on 'host' component of the the url
    * searches for keywords in link url to determine type of link
    * Choices are designed to be added over time
    * Intended to be used universally to determine affiliate link type
    */
    public function determine_affiliate_link_type($url){

        $urlParts = parse_url($url);

        if( !$urlParts || !array_key_exists('host', $urlParts) )
            return false;

        $choices = array(
            "amazon" => array('amazon','amzn'),
            "target" => array('target'),
            "walmart" => array('walmart')
        );

        foreach($choices as $choice => $keywords ){
            //hacky strpos search TY https://stackoverflow.com/questions/6284553/using-an-array-as-needles-in-strpos
            if( str_replace( $keywords, '', $urlParts['host']) != $urlParts['host'] ){
                return $choice;
            }else if( $urlParts['host'] == $_SERVER['HTTP_HOST'] ){
                return 'self_hosted';
            }
        }

        return false;
    }

    /**
     * [pluginActivate description]
     * @return [type] [description]
     */
    public static function pluginActivate()
    {
        $updater = new \IncomeSchool\AffiliateLinkRefer\DatabaseUpdater;
        $updater = $updater->activate();
    }

    /**
     * [pluginDeactivate description]
     * @return [type] [description]
     */
    public static function pluginDeactivate()
    {
        //
    }
}
