<?php

namespace IncomeSchool\AffiliateLinkRefer;

class ReferShortcode
{
    /**
     * [$code description]
     * @var string
     */
    private $code = 'refer';

    /**
     * [$cache description]
     * @var array
     */
    private $cache = [];

    /**
     * [$logged description]
     * @var array
     */
    private $logged = [];

    /**
     * [__construct description]
     */
    public function __construct()
    {
        add_shortcode($this->code, [$this, 'shortcode']);
        add_shortcode('REFERClock', [$this, 'clock_shortcode']);
    }

    /**
     * [shortcode description]
     * @param  [type] $atts [description]
     * @return [type]       [description]
     */
    public function clock_shortcode($atts, $content = null)
    {
        global $wpdb;
        //example usage: [REFERClock id=46]

        $atts = shortcode_atts([
            'id' => false,
            'echo' => false,
        ], $atts, $this->code);

        if(!$atts['id']) {
            return '';
        }

        $cookie = null;
        if (isset($_COOKIE['refer-cd-seen'])) {
            $cookie = $_COOKIE['refer-cd-seen'];
        }

        // var_dump($_COOKIE);

        if ( $cookie == true ) { 
            return;
        }

        $tableName = $wpdb->prefix . 'refer_countdown_links';

        $query = "SELECT * FROM $tableName WHERE 
            end_date > current_timestamp 
            AND start_date <= current_timestamp 
            AND id = " . $atts['id'] . "
            LIMIT 1";                        

        $items = $wpdb->get_results( $query );

        $item = false;
        if( is_array($items) ){
            if( count($items) > 0 ){
                $item = $items[0];
            }
        }

        //Check if countdown_ends is set to relative and cookie time has not run out
        if( isset( $item->countdown_ends ) && isset( $item->relative_time )){
            if( $item->countdown_ends == 'relative' ){
                if( isset($_COOKIE['refer-' . $item->id . '-relative-end-date']) ){
                    if( $_COOKIE['refer-' . $item->id . '-relative-end-date'] - time() <= 0 ){
                        return false;
                    }
                }
            }
        }

        if(!$item) {
            return '';
        }

        $params = Array(
            'item' => $item,
            'id' => $atts['id'],
            'context' => 'refer_countdown_shortcode',
        );
        
        ob_start();
        $GLOBALS['income_school']['refer_affiliates']->get_template_part_with_params('sale-countdown-template.php', $params ); 
        $content = ob_get_clean();
                
        if( $atts['echo'] ){
            echo $content;
        }else{
            return $content;
        }
    }
/**
     * [shortcode description]
     * @param  [type] $atts [description]
     * @return [type]       [description]
     */
    public function shortcode($atts, $content = null)
    {
        global $post, $wpdb;

        $atts = shortcode_atts([
            'id' => false,
        ], $atts, $this->code);

        if(!$atts['id']) {
            return '';
        }

        if(!$post) {
            return '';
        }

        if(isset($this->cache[$atts['id']])) {
            $affiliate = $this->cache[$atts['id']];
        } else {
            $tableName = $wpdb->prefix . 'refer_affiliate_links';

            $affiliate = $wpdb->get_row($wpdb->prepare("SELECT * FROM $tableName WHERE is_amazon = 0 AND hash = %s LIMIT 1", [$atts['id']]));
            if(!$affiliate) {
                return '';
            }
            $this->cache[$affiliate->hash] = $affiliate;
        }

        $url = site_url('?als=' . $affiliate->hash . '&pid=' . $post->ID);
        if($content) {
            $text = $content;
        } else {
            $text = $affiliate->text ? $affiliate->text : $affiliate->url;
        }

        if(!is_admin()) {
            $this->logView($affiliate);
        }

        return sprintf('<a href="%s" target="_blank" rel="nofollow">%s</a>', $url, $text);
    }

    /**
     * [logView description]
     * @param  [type] $affiliate [description]
     * @return [type]            [description]
     */
    private function logView($affiliate)
    {
        global $post, $wpdb;

        $userId = null;
        $postId = null;

        if($post) {
            $postId = $post->ID;
        } else {
            return false;
        }

        if(isset($this->logged[$affiliate->hash])) {
            return false;
        }

        if(get_current_user_id()) {
            $userId = get_current_user_id();
        }

        $trackingTableName = $wpdb->prefix . 'refer_link_daily_trackings';

        $today = date('Y-m-d');

        $existingDaily = $wpdb->get_row("
            SELECT *
            FROM $trackingTableName
            WHERE
                link_id = $affiliate->id
            AND
                post_id = $post->ID
            AND
                date = '$today'
            LIMIT 1
        ");

        if(!$existingDaily) {
            $wpdb->insert($trackingTableName, [
                'post_id' => $post->ID,
                'link_id' => $affiliate->id,
                'views' => 1,
                'date' => $today
            ]);
        } else {
            $wpdb->update($trackingTableName, [
                'views' => $existingDaily->views+1
            ], [
                'id' => $existingDaily->id
            ]);
        }
        //store the 'cached' clicks total in the links table.
        //TODO: add background process to sync these totals to the record items.
        $affiliateTableName = $wpdb->prefix . 'refer_affiliate_links';

        $wpdb->update($affiliateTableName, [
            'views' => $affiliate->views + 1
        ], [
            'id' => $affiliate->id
        ]);

        // $this->logged[$affiliate->hash];

        return true;
    }
}
