<?php

namespace IncomeSchool\AffiliateLinkRefer;

class ReportDataGenerator
{
    /**
     * Construct
     */
    public function __construct()
    {
        add_action('init', function() {
            if(!wp_next_scheduled( 'refer_report_data_generate')) {
                wp_schedule_event( time(), 'refer_five_min_interval', 'refer_report_data_generate' );
            }
        });
        add_action('refer_report_data_generate', [$this, 'generateReportData'], 99);

        //add_action('init', [$this, 'test_report_gen']);
    }

    public function test_report_gen(){
        //true = override 
        $this->generateTotalCountdownsReportData(true);
        $this->generateTotalClicksReportData(true);
    }


    public function generateReportData()
    {
        $generatedCount = 0;
        $maxGenerated = 1; // max per process to run

        //total click report - used for chart.
        if($generatedCount < $maxGenerated) {
            $generated = $this->generateTotalClicksReportData();
            if($generated) {
                $generatedCount++;
            }
        }

        // totals by word count report data
        if($generatedCount < $maxGenerated) {
            $generated = $this->generateTotalsByWordCountData();
            if($generated) {
                $generatedCount++;
            }
        }

        // totals by author report
        if($generatedCount < $maxGenerated) {
            $generated = $this->generateTotalsByAuthorData();
            if($generated) {
                $generatedCount++;
            }
        }

        // totals by post report
        if($generatedCount < $maxGenerated) {
            $generated = $this->generateTotalsByPostData();
            if($generated) {
                $generatedCount++;
            }
        }

        // totals by category report
        if($generatedCount < $maxGenerated) {
            $generated = $this->generateTotalsByCategoryData();
            if($generated) {
                $generatedCount++;
            }
        }

        // totals by product report
        if($generatedCount < $maxGenerated) {
            $generated = $this->generateTotalsByProductData();
            if($generated) {
                $generatedCount++;
            }
        }
        
		// totals countdowns report
        if($generatedCount < $maxGenerated) {
            $generated = $this->generateTotalCountdownsReportData();
            if($generated) {
                $generatedCount++;
            }
        }
		
		// totals cowntdowns by post report
        if($generatedCount < $maxGenerated) {
            $generated = $this->generateCountdownTotalsByPostData();
            if($generated) {
                $generatedCount++;
            }
        }
    }
	
	

    /**
     * [generateTotalClicksReportData description]
     * @return [type] [description]
     */
    public function generateTotalClicksReportData($override = false)
    {
        global $wpdb;

        if(!$override) {
            $existingData = get_option('refer-plugin-total-clicks-report-data', []);
            if(isset($existingData['date'])) {
                if(date("Y-m-d") == $existingData['date']) {
                    return false;
                }
            }
        }

        $trackingsTableName = $wpdb->prefix . 'refer_link_daily_trackings';

        $intervals = array(
            30,
            210,
            900,
        );

        $affiliateTypes = array(
            'all',
            'amazon',
            'other'
        );

        $chartData = array();

        foreach($intervals as $interval) {

            foreach($affiliateTypes as $affiliateType) {

                $whereClause = '';
                if($affiliateType != 'all') {
                    $whereClause = " AND t.is_amazon ";
                    if($affiliateType == 'other') {
                        //$whereClause .= '0';
						$whereClause .= '!= 1';
                    }
                    elseif($affiliateType == 'amazon') {
                        $whereClause .= '= 1';
                    }
                }

                $timeLabels = array();
                $clickData = array();
                switch($interval) {
                    /*case 7:
                        //iterate loop 28 times (7 days, every 6 hours)
                        for($i = 1; $i <= 7; $i++) {
                            $startDay = date("Y-m-d", strtotime('-'. ($override? $i-1 : $i) .' days'));
                            array_unshift($timeLabels, date('n/d/Y', strtotime($startDay)));
                            array_unshift($clickData, 0);
                            $sqlByDay = sprintf("SELECT clicks FROM %s t WHERE t.date = '$startDay' %s", $trackingsTableName, $whereClause);
                            $clicksByDay = $wpdb->get_results($sqlByDay);
                            foreach($clicksByDay as $clickByDay) {
                                $clickData[0] = $clickData[0] + $clickByDay->clicks;
                            }
                        }
                        break;*/
                    case 30:
                        for($i = 1; $i <= 30; $i++) {
                            $startDay = date('Y-m-d', strtotime('-'. ($override? $i-1 : $i) .' days'));
                            array_unshift($timeLabels, date('n/d/Y', strtotime($startDay)));
                            array_unshift($clickData, 0);
                            $sqlByDay = sprintf("SELECT clicks FROM %s t WHERE t.date = '$startDay' %s", $trackingsTableName, $whereClause);
                            $clicksByDay = $wpdb->get_results($sqlByDay);
							if($clicksByDay){
								foreach($clicksByDay as $clickByDay) {
									$clickData[0] = $clickData[0] + $clickByDay->clicks;
								}
							}else{
								$clickData[0] = 0;
							}
							//write_log('days: sql = '.$sqlByDay);
                        }
                        break;
                    case 210:
                        $weeks = [];
                        for($i = 1; $i < 30; $i++) {
                            $startDay = date('Y-m-d', strtotime('-'. ($override? $i-1 : $i) .' weeks'));
                            $endDay = date('Y-m-d', strtotime('-'. ($override? $i : $i+1) .' weeks'));
                            $week = date('W Y', strtotime('-'. ($override? $i-1 : $i) .' weeks'));
                            array_unshift($timeLabels, $week);
                            array_unshift($clickData, 0);
                            $sqlByDay = sprintf("SELECT clicks FROM %s t WHERE t.date <= '$startDay' AND t.date >= '$endDay' %s", $trackingsTableName, $whereClause);
                            $clicksByDay = $wpdb->get_results($sqlByDay);
							if($clicksByDay){
								foreach($clicksByDay as $clickByDay) {
									$clickData[0] = $clickData[0] + $clickByDay->clicks;
								}
							}else{
								$clickData[0] = 0;
							}
							//write_log('weeks: sql = '.$sqlByDay);
                        }
                        break;
                    case 900:
                        $months = [];
                        for($i = 1; $i < 30; $i++) {
                            $startDay = date('Y-m-01', strtotime('-'. ($override? $i-1 : $i) .' months'));
                            $endDay = date('Y-m-t', strtotime('-'. ($override? $i-1 : $i) .' months'));
                            $month = date('M Y', strtotime('-'. ($override? $i-1 : $i) .' months'));
                            array_unshift($timeLabels, $month);
                            array_unshift($clickData, 0);
                            $sqlByDay = sprintf("SELECT clicks FROM %s t WHERE t.date >= '$startDay' AND t.date <= '$endDay' %s", $trackingsTableName, $whereClause);
                            $clicksByDay = $wpdb->get_results($sqlByDay);
							if($clicksByDay){
								foreach($clicksByDay as $clickByDay) {
									$clickData[0] = $clickData[0] + $clickByDay->clicks;
								}
							}else{
								$clickData[0] = 0;
							}
							//write_log('months: sql = '.$sqlByDay);
                        }
                        break;
                    default:
                        break;
                }
    
                $intervalData = array();
                foreach($timeLabels as $index => $label) {
                    $intervalData[$label] = isset($clickData[$index]) ? $clickData[$index] : 0;
                }
                $chartData[$interval . '_days_' . $affiliateType . '_type'] = $intervalData;
            }
        }

        $reportData = [
            'date' => date("Y-m-d"),
            'data' => $chartData
        ];

        update_option('refer-plugin-total-clicks-report-data', $reportData, false);

        return true;
    }

    /**
     * [generateTotalClicksReportData description]
     * @return [type] [description]
     */
    public function generateTotalCountdownsReportData($override = false)
    {
        global $wpdb;

        if(!$override) {
            $existingData = get_option('refer-plugin-total-countdowns-report-data', []);
            if(isset($existingData['date'])) {
                if(date("Y-m-d") == $existingData['date']) {
                    return false;
                }
            }
        }

        $countdownTableName = $wpdb->prefix . 'refer_countdown_links';
        $trackingsTableName = $wpdb->prefix . 'refer_countdown_daily_trackings';

        $intervals = array(
            30,
            210,
            900,
        );

        $chartData = array();

        foreach($intervals as $interval) {
            $timeLabels = array();
            $clickData = array();
            switch($interval) {
                /*case 7:
                    //iterate loop 28 times (7 days, every 6 hours)
                    for($i = 1; $i <= 7; $i++) {
                        $startDay = date("Y-m-d", strtotime('-'. ($override? $i-1 : $i) .' days'));
                        array_unshift($timeLabels, date('n/d/Y', strtotime($startDay)));
                        array_unshift($clickData, 0);
                        $sqlByDay = sprintf("SELECT clicks FROM %s t WHERE t.date = '$startDay'", $trackingsTableName);
                        $clicksByDay = $wpdb->get_results($sqlByDay);
                        foreach($clicksByDay as $clickByDay) {
                            $clickData[0] = $clickData[0] + $clickByDay->clicks;
                        }
                    }
                    break;*/
                case 30:
                    for($i = 1; $i <= 30; $i++) {
                        $startDay = date("Y-m-d", strtotime('-'. ($override? $i-1 : $i) .' days'));
                        array_unshift($timeLabels, date('n/d/Y', strtotime($startDay)));
                        array_unshift($clickData, 0);
                        $sqlByDay = sprintf("SELECT clicks FROM %s t WHERE t.date = '$startDay'", $trackingsTableName);
                        $clicksByDay = $wpdb->get_results($sqlByDay);
                        if($clicksByDay){
							foreach($clicksByDay as $clickByDay) {
								$clickData[0] = $clickData[0] + $clickByDay->clicks;
							}
						}else{
							$clickData[0] = 0;
						}
                    }
                    break;
                case 210:
                    $weeks = [];
                    for($i = 0; $i < 30; $i++) {
                        $startDay = date('Y-m-01', strtotime('-'. ($override? $i-1 : $i) .' weeks'));
                        $endDay = date('Y-m-t', strtotime('-'. ($override? $i-1 : $i) .' weeks'));
                        $month = date('W Y', strtotime('-'. ($override? $i-1 : $i) .' weeks'));
                        array_unshift($timeLabels, $month);
                        array_unshift($clickData, 0);
                        $sqlByDay = sprintf("SELECT clicks FROM %s t WHERE t.date >= '$startDay' AND t.date <= '$endDay'", $trackingsTableName);
                        $clicksByDay = $wpdb->get_results($sqlByDay);
                        if($clicksByDay){
							foreach($clicksByDay as $clickByDay) {
								$clickData[0] = $clickData[0] + $clickByDay->clicks;
							}
						}else{
							$clickData[0] = 0;
						}
                    }
                    break;
                case 900:
                    $months = [];
                    for($i = 0; $i < 30; $i++) {
                        $startDay = date('Y-m-01', strtotime('-'. ($override? $i-1 : $i) .' months'));
                        $endDay = date('Y-m-t', strtotime('-'. ($override? $i-1 : $i) .' months'));
                        $month = date('M Y', strtotime('-'. ($override? $i-1 : $i) .' months'));
                        array_unshift($timeLabels, $month);
                        array_unshift($clickData, 0);
                        $sqlByDay = sprintf("SELECT clicks FROM %s t WHERE t.date >= '$startDay' AND t.date <= '$endDay'", $trackingsTableName);
                        $clicksByDay = $wpdb->get_results($sqlByDay);
                        if($clicksByDay){
							foreach($clicksByDay as $clickByDay) {
								$clickData[0] = $clickData[0] + $clickByDay->clicks;
							}
						}else{
							$clickData[0] = 0;
						}
                    }
                    break;
                default:
                    break;
            }

            $intervalData = array();
            foreach($timeLabels as $index => $label) {
                $intervalData[$label] = isset($clickData[$index]) ? $clickData[$index] : 0;
            }
            $chartData[$interval . '_days'] = $intervalData;
        }


        $reportData = [
            'date' => date("Y-m-d"),
            'data' => $chartData
        ];

        update_option('refer-plugin-total-countdowns-report-data', $reportData, false);

        return true;
    }

    /**
     * [generateTotalClicksReportData description]
     * @return [type] [description]
     */
    public function generateTotalsByWordCountData($override = false)
    {
        global $wpdb;

        $existingData = get_option('refer-plugin-totals-by-word-count-report-data', []);

        if(!$override) {
            if(isset($existingData['date'])) {
                if(date("Y-m-d") == $existingData['date']) {
                    return false;
                }
            }
        }

        $postTable = $wpdb->prefix . 'posts';
        $linksTableName = $wpdb->prefix . 'refer_affiliate_links';
        $trackingsTableName = $wpdb->prefix . 'refer_link_daily_trackings';

        $intervals = array(
            30,
            210,
            900,
            'all'
        );

        $affiliateTypes = array(
            'all',
            'amazon',
            'other'
        );

        foreach($intervals as $interval) {

            $dateRangeWhereClause = '';
            if($interval != 'all') {
                $numDays = $interval+1;
                $yesterday = date('Y-m-d', strtotime("-1 days"));
                $pastDate = date('Y-m-d', strtotime("-${numDays} days"));
                $dateRangeWhereClause = " AND DATE(t.date) BETWEEN DATE('{$pastDate}') AND DATE('{$yesterday}')";
            }

            foreach($affiliateTypes as $affiliateType) {

                $whereClause = '';
                if($affiliateType != 'all') {
                    $whereClause = " AND l.is_amazon = ";
                    if($affiliateType == 'other') {
                        $whereClause .= '0';
                    }
                    elseif($affiliateType == 'amazon') {
                        $whereClause .= '1';
                    }
                }

                $sqlWordCount = sprintf("SELECT
                    t.post_id,
                    p.post_title,
                    p.post_content,
                    SUM(t.views) as views,
                    SUM(t.clicks) as clicks
                FROM %s p, %s l, %s t
                WHERE t.post_id = p.ID
                    AND p.post_type <> 'revision'
                    AND t.link_id = l.ID{$whereClause}{$dateRangeWhereClause}
                GROUP BY t.post_id
                ORDER BY clicks DESC, views DESC", $postTable, $linksTableName, $trackingsTableName);

                $wordCountResults = $wpdb->get_results($sqlWordCount);

                $_under1500WordCount = array(
                    "title" => "Under 1500",
                    "views" => 0,
                    "clicks" => 0
                );

                $_1500To3000WordCount = array(
                    "title" => "1500 - 3000",
                    "views" => 0,
                    "clicks" => 0
                );

                $_over3000WordCount = array(
                    "title" => "Over 3000",
                    "views" => 0,
                    "clicks" => 0
                );

                foreach($wordCountResults as $wordCountResult) {
                    $contentLength = str_word_count($wordCountResult->post_content);
                    if($contentLength  < 1500) {
                        $_under1500WordCount['views'] += $wordCountResult->views;
                        $_under1500WordCount['clicks'] += $wordCountResult->clicks;
                    }
                    elseif($contentLength >= 1500 && $contentLength  <= 3000) {
                        $_1500To3000WordCount['views'] += $wordCountResult->views;
                        $_1500To3000WordCount['clicks'] += $wordCountResult->clicks;
                    }
                    elseif($contentLength > 3000) {
                        $_over3000WordCount['views'] += $wordCountResult->views;
                        $_over3000WordCount['clicks'] += $wordCountResult->clicks;
                    }
                }

                $reportData[$interval . '_days_' . $affiliateType . '_type'] = array($_under1500WordCount, $_1500To3000WordCount, $_over3000WordCount);

            }

        }

        $reportData = [
            'date' => date("Y-m-d"),
            'data' => $reportData
        ];

        update_option('refer-plugin-totals-by-word-count-report-data', $reportData, false);

        return true;
    }

    /**
     * [generateTotalsByAuthorData description]
     * @return [type] [description]
     */
    public function generateTotalsByAuthorData($override = false)
    {
        global $wpdb;

        if(!$override) {

            $existingData = get_option('refer-plugin-totals-by-author-report-data', []);

            if(isset($existingData['date'])) {
                if(date("Y-m-d") == $existingData['date']) {
                    return false;
                }
            }
        }

        $postTable = $wpdb->prefix . 'posts';
        $authorTableName = $wpdb->prefix . 'users';
        $linksTableName = $wpdb->prefix . 'refer_affiliate_links';
        $trackingsTableName = $wpdb->prefix . 'refer_link_daily_trackings';

        $intervals = array(
            30,
            210,
            900,
            'all'
        );

        $affiliateTypes = array(
            'all',
            'amazon',
            'other'
        );

        foreach($intervals as $interval) {

            $dateRangeWhereClause = '';
            if($interval != 'all') {
                $numDays = $interval+1;
                $yesterday = date('Y-m-d', strtotime("-1 days"));
                $pastDate = date('Y-m-d', strtotime("-${numDays} days"));
                $dateRangeWhereClause = " AND DATE(t.date) BETWEEN DATE('{$pastDate}') AND DATE('{$yesterday}')";
            }

            foreach($affiliateTypes as $affiliateType) {

                $whereClause = '';
                if($affiliateType != 'all') {
                    $whereClause = " AND l.is_amazon = ";
                    if($affiliateType == 'other') {
                        $whereClause .= '0';
                    }
                    elseif($affiliateType == 'amazon') {
                        $whereClause .= '1';
                    }
                }

                $sqlByAuthor = sprintf("SELECT
                    a.ID,
                    count(distinct(p.ID)) as 'post_count',
                    a.display_name,
                    SUM(t.views) as views,
                    SUM(t.clicks) as clicks,
                    SUM(t.clicks) / count(distinct(p.ID)) as 'avg'
                FROM %s p, %s a, %s l, %s t
                WHERE t.post_id = p.ID
                    AND p.post_type <> 'revison'
                    AND p.post_author = a.ID
                    AND t.link_id = l.ID{$whereClause}{$dateRangeWhereClause}
                GROUP BY a.ID", $postTable, $authorTableName, $linksTableName, $trackingsTableName);
                // echo $sqlByAuthor;
                $authorStats = $wpdb->get_results($sqlByAuthor);

                $rowsData = array();
                foreach($authorStats as $authorStat) {
                    $rowsData[] = array(
                        'name' => $authorStat->display_name,
                        'views' => $authorStat->views,
                        'clicks' => $authorStat->clicks,
                        'avg' => $authorStat->avg,
                    );
                }

                $reportData[$interval . '_days_' . $affiliateType . '_type'] = $rowsData;
            }
        }

        $reportData = [
            'date' => date("Y-m-d"),
            'data' => $reportData
        ];

        update_option('refer-plugin-totals-by-author-report-data', $reportData, false);

        return true;
    }

    /**
     * [generateTotalsByPostData description]
     * @return [type] [description]
     */
    public function generateTotalsByPostData($override = false)
    {
        global $wpdb;

        if(!$override) {
            $existingData = get_option('refer-plugin-totals-by-post-report-data', []);
            if(isset($existingData['date'])) {
                if(date("Y-m-d") == $existingData['date']) {
                    return false;
                }
            }
        }

        $postTable = $wpdb->prefix . 'posts';
        $linksTableName = $wpdb->prefix . 'refer_affiliate_links';
        $trackingsTableName = $wpdb->prefix . 'refer_link_daily_trackings';

        $intervals = array(
            30,
            210,
            900,
            'all'
        );

        $affiliateTypes = array(
            'all',
            'amazon',
            'other'
        );

        foreach($intervals as $interval) {

            $dateRangeWhereClause = '';
            if($interval != 'all') {
                $numDays = $interval+1;
                $yesterday = date('Y-m-d', strtotime("-1 days"));
                $pastDate = date('Y-m-d', strtotime("-${numDays} days"));
                $dateRangeWhereClause = " AND DATE(t.date) BETWEEN DATE('{$pastDate}') AND DATE('{$yesterday}')";
            }

            foreach($affiliateTypes as $affiliateType) {

                $whereClause = '';
                if($affiliateType != 'all') {
                    $whereClause = " AND l.is_amazon = ";
                    if($affiliateType == 'other') {
                        $whereClause .= '0';
                    }
                    elseif($affiliateType == 'amazon') {
                        $whereClause .= '1';
                    }
                }

                $postStats = $wpdb->get_results(sprintf("SELECT
                    t.post_id,
                    p.post_title,
                    SUM(t.views) as views,
                    SUM(t.clicks) as clicks
                FROM %s p, %s l, %s t
                WHERE t.post_id = p.ID
                    AND p.post_type <> 'revision'
                    AND t.link_id = l.ID{$whereClause}{$dateRangeWhereClause}
                GROUP BY t.post_id
                ", $postTable, $linksTableName, $trackingsTableName));

                $rowsData = array();
                foreach($postStats as $postStat) {
                    $rowsData[] = array(
                        'title' => $postStat->post_title,
                        'views' => $postStat->views,
                        'clicks' => $postStat->clicks
                    );
                }

                $reportData[$interval . '_days_' . $affiliateType . '_type'] = $rowsData;
            }
        }

        $reportData = [
            'date' => date("Y-m-d"),
            'data' => $reportData
        ];

        update_option('refer-plugin-totals-by-post-report-data', $reportData, false);

        return true;
    }

    /**
     * [generateCountdownTotalsByPostData description]
     * @return [type] [description]
     */
    public function generateCountdownTotalsByPostData($override = false)
    {
        global $wpdb;

        if(!$override) {
            $existingData = get_option('refer-plugin-countdown-totals-by-post-report-data', []);
            if(isset($existingData['date'])) {
                if(date("Y-m-d") == $existingData['date']) {
                    return false;
                }
            }
        }

        $postTable = $wpdb->prefix . 'posts';
        $countdownTableName = $wpdb->prefix . 'refer_countdown_links';
        $trackingsTableName = $wpdb->prefix . 'refer_countdown_daily_trackings';

        $intervals = array(
            30,
            210,
            900,
            'all'
        );

        foreach($intervals as $interval) {

            $dateRangeWhereClause = '';
            if($interval != 'all') {
                $numDays = $interval+1;
                $yesterday = date('Y-m-d', strtotime("-1 days"));
                $pastDate = date('Y-m-d', strtotime("-${numDays} days"));
                $dateRangeWhereClause = " AND DATE(t.date) BETWEEN DATE('{$pastDate}') AND DATE('{$yesterday}')";
            }

            $postStats = $wpdb->get_results(sprintf("SELECT
                t.post_id,
                p.post_title,
                SUM(t.views) as views,
                SUM(t.clicks) as clicks
            FROM %s p, %s l, %s t
            WHERE t.post_id = p.ID
                AND p.post_type <> 'revision'
            GROUP BY t.post_id
            ", $postTable, $countdownTableName, $trackingsTableName));

            $rowsData = array();
            foreach($postStats as $postStat) {
                $rowsData[] = array(
                    'title' => $postStat->post_title,
                    'views' => $postStat->views,
                    'clicks' => $postStat->clicks
                );
            }

            $reportData[$interval . '_days' ] = $rowsData;
            
        }

        $reportData = [
            'date' => date("Y-m-d"),
            'data' => $reportData
        ];

        // var_dump($reportData);

        update_option('refer-plugin-countdown-totals-by-post-report-data', $reportData, false);

        return true;
    }

    /**
     * [generateTotalsByCategoryData description]
     * @return [type] [description]
     */
    public function generateTotalsByCategoryData($override = false)
    {
        global $wpdb;

        if(!$override) {

            $existingData = get_option('refer-plugin-totals-by-category-report-data', []);

            if(isset($existingData['date'])) {
                if(date("Y-m-d") == $existingData['date']) {
                    return false;
                }
            }
        }

        $postTable = $wpdb->prefix . 'posts';
        $linksTableName = $wpdb->prefix . 'refer_affiliate_links';
        $trackingsTableName = $wpdb->prefix . 'refer_link_daily_trackings';
        $termsTableName = $wpdb->prefix . 'terms';
        $termRelationshipsTableName = $wpdb->prefix . 'term_relationships';

        $intervals = array(
            30,
            210,
            900,
            'all'
        );

        $affiliateTypes = array(
            'all',
            'amazon',
            'other'
        );

        foreach($intervals as $interval) {

            $dateRangeWhereClause = '';
            if($interval != 'all') {
                $numDays = $interval+1;
                $yesterday = date('Y-m-d', strtotime("-1 days"));
                $pastDate = date('Y-m-d', strtotime("-${numDays} days"));
                $dateRangeWhereClause = " AND DATE(t.date) BETWEEN DATE('{$pastDate}') AND DATE('{$yesterday}')";
            }

            foreach($affiliateTypes as $affiliateType) {

                $whereClause = '';
                if($affiliateType != 'all') {
                    $whereClause = " AND l.is_amazon = ";
                    if($affiliateType == 'other') {
                        $whereClause .= '0';
                    }
                    elseif($affiliateType == 'amazon') {
                        $whereClause .= '1';
                    }
                }

                $sqlByTaxonomy = sprintf("SELECT
                    terms.term_id,
                    terms.name,
                    SUM(t.views) as views,
                    SUM(t.clicks) as clicks
                FROM %s p, %s terms, %s tr, %s l, %s t
                WHERE t.post_id = p.ID
                    AND p.post_type <> 'revison'
                    AND t.post_id = tr.object_id
                    AND terms.term_id = tr.term_taxonomy_id
                    AND t.link_id = l.ID{$whereClause}{$dateRangeWhereClause}
                GROUP BY terms.term_id", $postTable, $termsTableName, $termRelationshipsTableName, $linksTableName, $trackingsTableName);

                $taxStats = $wpdb->get_results($sqlByTaxonomy);

                $rowsData = array();
                foreach($taxStats as $taxStat) {
                    $rowsData[] = array(
                        'title' => $taxStat->name,
                        'views' => $taxStat->views,
                        'clicks' => $taxStat->clicks
                    );
                }

                $reportData[$interval . '_days_' . $affiliateType . '_type'] = $rowsData;
            }
        }

        $reportData = [
            'date' => date("Y-m-d"),
            'data' => $reportData
        ];

        update_option('refer-plugin-totals-by-category-report-data', $reportData, false);

        return true;
    }


     /**
     * [generateTotalsByCategoryData description]
     * @return [type] [description]
     */
    public function generateTotalsByProductData($override = false)
    {
        global $wpdb;

        if(!$override) {
            $existingData = get_option('refer-plugin-totals-by-product-report-data', []);

            if(isset($existingData['date'])) {
                if(date("Y-m-d") == $existingData['date']) {
                    return false;
                }
            }
        }

        $postTable = $wpdb->prefix . 'posts';
        $linksTableName = $wpdb->prefix . 'refer_affiliate_links';
        $trackingsTableName = $wpdb->prefix . 'refer_link_daily_trackings';

        $intervals = array(
            30,
            210,
            900,
            'all'
        );

        $affiliateTypes = array(
            'all',
            'amazon',
            'other',
        );

        foreach($intervals as $interval) {

            $dateRangeWhereClause = '';
            if($interval != 'all') {
                $numDays = $interval+1;
                $yesterday = date('Y-m-d', strtotime("-1 days"));
                $pastDate = date('Y-m-d', strtotime("-${numDays} days"));
                $dateRangeWhereClause = " AND DATE(t.date) BETWEEN DATE('{$pastDate}') AND DATE('{$yesterday}')";
            }

            foreach($affiliateTypes as $affiliateType) {

                $whereClause = '';
                if($affiliateType != 'all') {
                    $whereClause = " AND l.is_amazon = ";
                    if($affiliateType == 'other') {
                        $whereClause .= '0';
                    }
                    elseif($affiliateType == 'amazon') {
                        $whereClause .= '1';
                    }
                }

                $sqlByProduct = sprintf("SELECT
                    COALESCE(NULLIF(l.label, ''), l.url) AS 'name',
                    SUM(t.views) as views,
                    SUM(t.clicks) as clicks
                FROM %s p, %s l, %s t
                WHERE t.post_id = p.ID
                    AND p.post_type <> 'revision'
                    AND l.id = t.link_id{$whereClause}{$dateRangeWhereClause}
                GROUP BY l.id", $postTable, $linksTableName, $trackingsTableName);
                // echo $sqlByProduct;
                $productStats = $wpdb->get_results($sqlByProduct);

                $rowsData = array();
                foreach($productStats as $productStat) {
                    $rowsData[] = array(
                        'name' => $productStat->name,
                        'views' => $productStat->views,
                        'clicks' => $productStat->clicks
                    );
                }

                $reportData[$interval . '_days_' . $affiliateType . '_type'] = $rowsData;
            }
        }

        $reportData = [
            'date' => date("Y-m-d"),
            'data' => $reportData
        ];

        update_option('refer-plugin-totals-by-product-report-data', $reportData, false);

        return true;
    }

}
